Package: AGHmatrix
Title: Relationship Matrices for Diploid and Autopolyploid Species
Version: 2.1.4
Description: Computation of A (pedigree), G (genomic-base), and H (A corrected
    by G) relationship matrices for diploid and autopolyploid species. Several methods
    are implemented considering additive and non-additive models.
Authors@R: c(person("Rodrigo","Amadeu", role = c("aut","cre"), email = "rramadeu@gmail.com"),
             person("Luis", "Ferrao", role = c("aut","ctb")),
             person("Catherine", "Cellon", role = "ctb"),
             person("Leticia", "Lara", role = "ctb"),
             person("Marcio", "Resende", role = "ctb"),
             person("Ivone", "Oliveira", role = "ctb"),
             person("Patricio", "Munoz", role = "ctb"),
             person("Augusto", "Garcia", role = "ctb"))
Author: Rodrigo Amadeu [aut, cre],
  Luis Ferrao [aut, ctb],
  Catherine Cellon [ctb],
  Leticia Lara [ctb],
  Marcio Resende [ctb],
  Ivone Oliveira [ctb],
  Patricio Munoz [ctb],
  Augusto Garcia [ctb]
Depends: R (>= 3.6.0)
Imports: Matrix (>= 1.2-7.1), zoo (>= 1.8.6)
Suggests: knitr, MASS, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
License: GPL-3
URL: https://github.com/rramadeu/AGHmatrix
Maintainer: Rodrigo Amadeu <rramadeu@gmail.com>
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-10-03 15:25:06 UTC; GNIEZ
Repository: CRAN
Date/Publication: 2023-10-03 18:20:02 UTC
Built: R 4.6.0; ; 2026-04-27 06:16:49 UTC; unix

Package: AGHmatrix
Type: Package
Title: Relationship Matrices for Diploid and Autopolyploid Species
Version: 3.0.1
Description: Fast computation of A (pedigree), G (genomic-base), and H (A corrected
    by G) relationship matrices for diploid and autopolyploid species. Several methods
    are implemented considering additive and non-additive models.
Authors@R: 
    c(
        person("Rodrigo","Amadeu", role = c("aut","cre"), email = "rramadeu@gmail.com"),
        person("Luis", "Ferrao", role = c("aut","ctb")),
        person("Thiago", "Oliveira", role = c("aut","ctb")),
        person("Catherine", "Cellon", role = "ctb"),
        person("Leticia", "Lara", role = "ctb"),
        person("Marcio", "Resende", role = "ctb"),
        person("Ivone", "Oliveira", role = "ctb"),
        person("Patricio", "Munoz", role = "ctb"),
        person("Augusto", "Garcia", role = "ctb")
    )
Maintainer: Rodrigo Amadeu <rramadeu@gmail.com>
Depends: R (>= 3.6.0)
Imports: Rcpp (>= 1.1.0), matrixStats (>= 1.5.0)
Suggests: knitr, MASS, rmarkdown, testthat, zoo
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Encoding: UTF-8
License: GPL-3
URL: https://github.com/rramadeu/AGHmatrix
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-07-22 02:46:31 UTC; rstudio
Author: Rodrigo Amadeu [aut, cre],
  Luis Ferrao [aut, ctb],
  Thiago Oliveira [aut, ctb],
  Catherine Cellon [ctb],
  Leticia Lara [ctb],
  Marcio Resende [ctb],
  Ivone Oliveira [ctb],
  Patricio Munoz [ctb],
  Augusto Garcia [ctb]
Repository: CRAN
Date/Publication: 2026-07-22 06:30:02 UTC

# Generated by roxygen2: do not edit by hand

export(Amatrix)
export(AmatrixPolyCross)
export(Gmatrix)
export(Hmatrix)
export(Mcheck)
export(asciitonumber)
export(datatreat)
export(expandAmatrix)
export(filterpedigree)
export(formatmatrix)
export(missingdata)
importFrom(Rcpp,evalCpp)
importFrom(matrixStats,colVars)
importFrom(stats,approx)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,write.table)
useDynLib(AGHmatrix, .registration = TRUE)

[![](https://cranlogs.r-pkg.org/badges/grand-total/AGHmatrix)](https://cran.r-project.org/package=AGHmatrix)

<img src="https://raw.githubusercontent.com/rramadeu/AGHmatrix/master/inst/misc/logo.jpg" width="20%" height="20%" /> 

**Relationship Matrices for Diploid and Autopolyploid Species**

## Overview
AGHmatrix software is an R-package to build relationship matrices using pedigree (A matrix) and/or molecular markers (G matrix) with the possibility to build a combined matrix of Pedigree corrected by Molecular (H matrix). The package works with diploid and autopolyploid data.

## Matrices computation implemented in the `AGHmatrix` 
Currently the package computes the following 17 different relationship matrices:

### Pedigree-based relationship matrix (A matrix)

|                   | Additive                   | Non-Additive     |
|-------------------|----------------------------|------------------|
| **Diploid**       | Henderson (1976)           | Cockerham (1954) |
| **Autopolyploid** | Kerr (2012), Slater (2013) |                  |

### Molecular-based relationship matrix (G matrix)
|                   | Additive                                  | Non-Additive                   |
|-------------------|-------------------------------------------|--------------------------------|
| **Diploid**       | Yang (2010), VanRaden (2012), Liu (2020)  | Su (2012), Vitezica (2013)     |
| **Polyploid**     | Slater (2016), de Bem Oliveira (2019)     | Slater (2016), Endelman (2018) |


### Combined pedigree and molecular-based relationship matrix (H matrix)

| **Any ploidy/effect**                        |
|----------------------------------------------|
| Legarra (2009), Munoz (2014), Martini (2018) |

Additionally there is a beta implementation to compute A matrix when parentage is not deterministic as in a polycross design. See `?AmatrixPolycross`.

## Citation
To cite this R package:

## Citation
To cite this R package:

Amadeu RR, Garcia AA, Munoz PR, Ferrão LF. AGHmatrix: genetic relationship matrices in R. Bioinformatics. 2023 Jul 1;39(7):btad445. https://doi.org/10.1093/bioinformatics/btad445

## Contact
Rodrigo R Amadeu  
rramadeu at gmail dot com  
https://rramadeu.github.io

## Installing and loading
Within R:
```{r, eval=FALSE}
## Install stable version
install.packages("AGHmatrix")

## Install development version
install.packages("devtools")
devtools::install_github("rramadeu/AGHmatrix")

## Load 
library(AGHmatrix)
```

## Relationship matrices using pedigree data - A matrix
`Amatrix` process the pedigree and build the A-matrix related to that given pedigree. The matrix is built based in the recursive method presented in Mrode (2014) and described by Henderson (1976). This method is expanded for higher ploidies (n-ploidy) as detailed in Kerr et al. (2012). After loading the package you have to load your data file into the software. To do this, you can use the function `read.data()` or `read.csv()` for example. Your data should be available in R as a `data.frame` structure in the following order: column 1 must be the individual/genotype names (id), columns 2 and 3 must be the parent names. For the algorithm, it does not matter who is the mother and who is the father (so, no sex column). There is a pedigree data example (`ped.mrode`) that can be used to look at the structure and order the data.

To load `ped.mrode`:

```{r}
data(ped.mrode)
ped.mrode
str(ped.mrode) #check the structure
```

The example `ped.mrode` has 3 columns, column 1 contains the names of the individual/genotypes, column 2 contains the names of the first parent, column 3 contains the names of the second parental (example from Table 2.1 of Mrode 2014). There is no header template, and the unknown value must be equal 0. Your data has to be in the same format of `ped.mrode`.

Internally the algorithm first pre-process the pedigree: the individuals are numerated $1$ to $n$. Then, it is verified whether the genotypes in the pedigree are in chronological order (i.e. if the parents of a given individual are located before to this individual in the pedigree data set). If this order is not followed, the algorithm performs the necessary changes to correct them in a iterative way. After this pre-processing, the matrix computation proceeds as in Henderson (1976) for diploid - for additive or dominance relationship - and as in Kerr et al. (2012) for autotetraploids - for additive relationship. For autotetraploids, there is the option to include double-reduction fraction. For diploids there is the option to compute the dominant relationship matrix (Cockerham, 1954).

It follows some usage examples with the `ped.mrode`.
```{r, eval=FALSE}
#Computing additive relationship matrix for diploids (Henderson 1976):
Amatrix(ped.mrode, ploidy=2)

#Computing dominant relationship matrix for diploids (Cockerham 1954):
Amatrix(ped.mrode, ploidy=2, dominance=TRUE)

#Computing additive relationship matrix for autotetraploids (Kerr 2012):
Amatrix(ped.mrode, ploidy=4)

#Computing additive relationship matrix for autooctaploids (Kerr 2012):
Amatrix(ped.mrode, ploidy=8)

#Computing additive relationship matrix for autotetraploids
# and double-reduction of 0.1 (Kerr 2012):
Amatrix(ped.mrode, ploidy=4, w=0.1)

#Computing additive relationship matrix for autotetraploids
# and double-reduction of 0.1 as in Slater et al. (2014):
Amatrix(ped.mrode, ploidy=4, w=0.1, slater = TRUE) 
#not recommended, but kept in the package to reproduce some former analysis

#Computing additive relationship matrix for autohexaploids
# and double-reduction of 0.1 (Kerr 2012):
Amatrix(ped.mrode, ploidy=6, w=0.1)
```

More information about `Amatrix` can be found with:
```{r, eval=FALSE}
?Amatrix
```

## Diploid G matrix: relationship matrices using the molecular data
`Gmatrix` handles the molecular-marker matrix and builds the relationship matrix. Molecular markers data should be organized in a matrix format (individuals in rows and markers in columns) coded as 0, 1, 2 and missing data value (numeric or `NA`). Import your molecular marker data into `R` with the function `read.table()` or `read.csv()` and convert to a matrix format with the function `as.matrix()`. The function `Gmatrix` can be used to construct the additive relationship either as proposed by Yang et al. (2010) or the proposed by VanRaden (2008). The function can also construct the dominance relationship matrix either as proposed by Su et al. (2012) or as proposed by Vitezica et al. (2013). As an example, here we build the four matrices using real data from Resende et al. (2012).

To load `snp.pine` and to check its structure:
```{r}
data(snp.pine)
snp.pine[1:5,1:5]
str(snp.pine)
```

In this dataset, we have 926 individuals with 4853 markers and the missing data value is `-9`.

It follows some examples with the `snp.pine` data where the unknown value (`missingValue`) is `-9`. Here we set minimum allele frequency to `0.05`, so markers with minor allele frequency lower than 0.05 are removed from the dataset prior to the G matrix construction.

```{r, eval=FALSE}
#Computing the additive relationship matrix based on VanRaden 2008
G_VanRadenPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                          maf=0.05, method="VanRaden")

#Computing the additive relationship matrix based on Yang 2010
G_YangPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                      maf=0.05, method="Yang")

#Computing the dominance relationship matrix based on Su 2012
G_SuPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                    maf=0.05, method="Su")

#Computing the dominance relationship matrix based on Vitezica 2013
G_VitezicaPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                          maf=0.05, method="Vitezica")
```

More information about `Gmatrix` can be found with:
```{r, eval=FALSE}
?Gmatrix
```

## Autopolyploid G matrix: relationship matrices using the molecular data
Molecular markers data should be organized in a matrix format (individual in rows and markers in columns) coded according to the dosage level: 0, 1, 2, ..., ploidy level, and missing data value (numeric user-defined or `NA`). As an example, an autotetraploid should be coded as 0, 1, 2, 3, 4, and a missing data value. In autopolyploids, the function `Gmatrix` can be used to construct: i) the additive relationship based on VanRaden (2008) and extended by Ashraf (2016); ii) the full-autopolyploid including additive and non-additive model as equations 8 and 9 in Slater et al. (2016); iii) the pseudo-diploid model as equations 5, 6, and 7 Slater et al. (2016). iv) the digenic-dominant model based on Endelman et al. (2018). As an example, here we build the matrices using data from Endelman et al. (2018) (`snp.sol`). There is also an option to build weighted relationship matrices as in Liu et al. (2020).

The argument `ploidy.correction` defines the denominator of the formula for the `VanRaden` method. If `ploidy.correction=TRUE`, it uses the parametric correction as $\sum_i p f_i(1-f_i)$, where $p$ is the ploidy level and $f_i$ is the minor allele frequency of the $i_th$ marker. If `ploidy.correction=FALSE`, it uses the sampling variance correction as $\sum_i \frac{1}{p} s^2(m_i)$, where $s^2(m_i)$ is the sampling variance of the $i_th$ marker. Both corrections are equivalent when sampling size goes to the infinity. The default is to use the sampling variance as the correction (i.e., `ploidy.correction=FALSE`).

```{r, eval=FALSE}
#Loading the data
data(snp.sol)
str(snp.sol)

#Computing the additive relationship matrix based on VanRaden 2008
# adapted by Ashraf 2016
G_VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)

#Computing the dominance (digenic) matrix based on Endelman 2018 (Eq. 19)
G_Dominance <- Gmatrix(snp.sol, method="Endelman", ploidy=4)

#Computing the full-autopolyploid matrix based on Slater 2016 (Eq. 8
#and 9)
G_FullAutopolyploid <- Gmatrix(snp.sol, method="Slater", ploidy=4)

#Computing the pseudodiploid matrix based on Slater 2016 (Eq. 5, 6,
#and 7)
G_Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE) 

#Computing G matrix with specific weight for each marker as 
# in Liu et al. (2020).
Gmatrix_weighted <- Gmatrix(snp.sol, method="VanRaden", weights = runif(3895,0.001,0.1), ploidy=4)
```

More information about `Gmatrix` can be found with:
```{r, eval=FALSE}
?Gmatrix
```

## Ratio (non-dosage) G matrix: relationship matrices using the molecular data without dosage calling
Molecular markers data should be organized in a matrix format (individual in rows and markers in columns) coded according to a fraction that represents its molecular information, this can be any number between 0 to 1. Such ratio can represent the count of alternative alleles over the read depth for each individual-marker combination (GBS-like technique). It can be the signal of the alternative allele over the sum of the signals of the alternative and reference alleles (GCMS-like technique). It can also be used for family-pool genotypes.

```{r, eval=FALSE}
#Loading the data
library(AGHmatrix)
data(snp.sol) 
snp.sol.ratio = snp.sol/4 #transforming it in a ratio of the minor allele frequency
Gmatrix <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, ratio=FALSE)  
Gmatrix.ratio <- Gmatrix(snp.sol.ratio, method="VanRaden", ploidy=4, ratio=TRUE)
Gmatrix[1:5,1:5]==Gmatrix.ratio[1:5,1:5]

## it also has the ploidy.correction option
Gmatrix.alternative <- Gmatrix(snp.sol, 
                               method="VanRaden", 
                               ploidy=4, 
                               ratio=FALSE, 
                               ploidy.correction=TRUE) 

Gmatrix.ratio.alternative <- Gmatrix(snp.sol.ratio, 
                                     method="VanRaden", 
                                     ploidy=4, 
                                     ratio=TRUE, 
                                     ploidy.correction=TRUE)
Gmatrix[1:5,1:5]==Gmatrix.alternative[1:5,1:5]
Gmatrix.alternative[1:5,1:5]==Gmatrix.ratio.alternative[1:5,1:5]
```

## Combined relationship matrix - H matrix
H matrix is the relationship matrix using combined information from the pedigree and genomic relationship matrices. First, you need to compute the matrices separated and then use them as input to build the combined H matrix. Two methods are implemented: `Munoz` shrinks the G matrix towards the A matrix scaling the molecular relatadness by each relationship classes; `Martini` is a modified version from Legarra et al. 2009 where combines A and G matrix using scaling factors. As an example, here we build the matrices using data from Endelman et al. (2018) (`ped.sol` and `snp.sol`).

```{r, eval=FALSE}
data(ped.sol)
data(snp.sol)

#Computing the numerator relationship matrix 10% of double-reduction
Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)
Gmat <- Gmatrix(snp.sol, ploidy=4,
                maf=0.05, method="VanRaden")
Gmat <- round(Gmat,3) #see appendix

#Computing H matrix (Martini)
Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini", 
                        ploidy=4, missingValue=-9, maf=0.05)

#Computing H matrix (Munoz)
Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol, 
                      ploidy=4, method="Munoz", 
                      missingValue=-9, maf=0.05)
```
## Covariance matrices due to epistatic terms
Here we present how to compute the epistasis relationship matrices using Hadamard
products (i.e. cell-by-cell product), denoted by `*`. For more information please see Munoz
et al. (2014). In this example we are using the molecular-based relationship matrices. First, build the additive and dominance matrices:

```{r, eval=FALSE}
data(snp.pine)
A <- Gmatrix(SNPmatrix=snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
D <- Gmatrix(SNPmatrix=snp.pine, method="Vitezica", missingValue=-9,maf=0.05)
```

For the first degree epistatic terms:
```{r, eval=FALSE}
#Additive-by-Additive Interactions
A_A <- A*A
#Dominance-by-Additive Interactions
D_A <- D*A
#Dominance-by-Dominance Interactions
D_D <- D*D
```

For the second degree epistatic terms:
```{r, eval=FALSE}
#Additive-by-Additive-by-Additive Interactions
A_A_A <- A*A*A
#Additive-by-Additive-by-Dominance Interactions
A_A_D <- A*A*D
#Additive-by-Dominance-by-Dominance Interactions
A_D_D <- A*D*D
#Dominance-by-Dominance-by-Dominance Interactions
D_D_D <- D*D*D
```

And so on...

## Exporting your matrix as three columns and sparse format (ASReml - csv format)
That is the lower diagonal matrix formatted in three columns in .csv format (other ASCII extension could be used as well). In order to do this, we need to build a matrix, its inverse, and export it using `formatmatrix` function. ASReml can invert the relationship matrix as well, probably more efficiently than R for large matrices (i.e. `solve()` function), so no need to invert the matrix in R if matrix is large. This function has as options: `round.by`, which let you decide the number of decimals you want; `exclude.0`, if `TRUE`, remove all the zeros from your data (i.e., transforms into sparse); and, name that defines the name to be used in the exported file. Use the default if not sure what parameter use in these function. Here an example using `ped.mrode` data:

```{r, eval=FALSE}
#Loading the data example
data(ped.mrode)

#Computing the matrix
A <- Amatrix(data=ped.mrode, ploidy=4, w=0.1)

#Building its inverse
Ainv <- solve(A)

#Exporting it. The function "formatmatrix" 
# will convert it and save in your working directory
formatmatrix(Ainv, round.by=12, exclude.0=TRUE, name="Ainv")
```

## Relationship matrices using pedigree data for polycrosses - A matrix (beta)
Creates an additive relationship matrix A based on a non-deterministic pedigree with 4+ columns where each column represents a possible parent. This function was built with the following designs in mind. 1) A mating design where you have equally possible parents. For example, a generation of insects derived from the mating of three insects in a cage. All the insects in this generation will have the same expected relatedness with all the possible parents (1/3). If there are only two parents in the cage, the function assumes no-inbreeding and the pedigree is deterministic (the individual is offspring of the cross between the two parents). Another example, a population of 10 open-pollinated plants where you harvest the seeds without tracking the mother. 2) When `fixedParent` is TRUE: a mating design where you know one parent and might know the other possible parents. For example, a polycross design where you have seeds harvested from a mother plant and possible polén donors.

The following pedigree has the id of the individual followed by possible parents. The possible parents are filled from left to right, in the `pedigree` data frame: id 1,2,3,4 have unknown parents and are assumed unrelated; id 5 has three possible parents (1,2,3); id 6 has three possible parents (2,3,4); id 7 has two parents (deterministic case here, the parents are 3 and 4); id 8 has four possible parents (5,6,7,1).

```{r}
pedigree = data.frame(id=1:8,
                      parent1 = c(0,0,0,0,1,2,3,5),
                      parent2 = c(0,0,0,0,2,3,4,6),
                      parent3 = c(0,0,0,0,3,4,0,7),
                      parent4 = c(0,0,0,0,0,0,0,1),
                      parent5 = 0)

print(pedigree)
AmatrixPolyCross(pedigree)
```

If `fixedParent=TRUE`, the above pedigree will be interpreted with the possible parents are filled from left to right after the known parent, in the `pedigree` data frame: id 1,2,3,4 have unknown parents and are assumed unrelated; id 5 is offspring of parent 1 in a deterministic way and two other possible parents (2,3); id 6 is offspring of parent 2 in a deterministic way and two other possible parents (3,4); id 7 has two parents (deterministic case here, the parents are 3 and 4); as before; id 8 is offspring of parent 5 in a deterministic way and has three other possible parents (6,7,1).

```{r}
AmatrixPolyCross(pedigree,fixedParent=TRUE)
```

## A matrix benchmark

It follows a small memory and computational time profiling for the `Amatrix()` function. The required RAM was computed based on the peak of the process for different pedigree sizes (based on /usr/bin/time -v output). The time profiling was done using AMD Milan 2.95GHz, so it might be an underestimated value when compared with lower speed processors. Numerator relationship matrices for pedigrees with less than 20,000 rows can built with low-specs user-end machines (<8GB RAM) using our package.

<img src="https://github.com/rramadeu/Tutorials_File/raw/master/Amatrix_benchmark.png" width="500">


## Bibliography
Amadeu, RR, et al. 2016 AGHmatrix: R package to construct relationship matrices for autotetraploid and diploid species: a blueberry example. The Plant Genome 9, 4. https://doi.org/10.3835/plantgenome2016.01.0009

Ashraf, BH, et al. 2016. Estimating genomic heritabilities at the level of family-pool samples of perennial ryegrass using genotyping-by-sequencing. Theoretical and Applied Genetics 129, 45-52. https://doi.org/0.1007/s00122-015-2607-9

Cockerham, CC. 1954 An extension of the concept of partitioning hereditary variance for analysis of covariances among relatives when epistasis is present. Genetics 39, 859–882

de Bem Oliveira, I, et al. 2019 Genomic prediction of autotetraploids; influence of relationship matrices, allele dosage, and continuous genotyping calls in phenotype prediction. G3: Genes, Genomes, Genetics, 9(4), pp.1189-1198.

Endelman, JB, et al. 2018 Genetic variance partitioning and genome-wide prediction with allele dosage information in autotetraploid potato. Genetics 209, 77-87. https://doi.org/10.1534/genetics.118.300685

Hamilton, MG, et al. 2017 Computation of the inverse additive relationship matrix for autopolyploid and multiple-ploidy populations. Theoretical and Applied Genetics 131, 851-890.  https://doi.org/10.1007/s00122-017-3041-y

Henderson, C. 1976 A simple method for computing the inverse of a numerator relationship matrix used in prediction of breeding values. Biometrics 32, 69–83. https://doi.org/10.2307/2529339

Kerr, RJ, et al., 2012 Use of the numerator relation ship matrix in genetic analysis of autopolyploid species. Theoretical and Applied Genetics 124, 1271–1282. https://doi.org/10.1007/s00122-012-1785-y

Legarra, A, et al. 2009 A relationship matrix including full pedigree and genomic information. Journal of Dairy Science 92, 4656–4663.

Liu, A, et al. 2020. Weighted single-step genomic best linear unbiased prediction integrating variants selected from sequencing data by association and bioinformatics analyses. Genet Sel Evol 52, 48.

Martini, JW, et al. 2018, The effect of the H$^{1}$ scaling factors $\tau$ and $\omega$ on the structure of H in the single-step procedure. Genetics Selection Evolution 50(1), 16. https://doi.org/10.1186/s12711-018-0386-x

Mrode, RA. 2014 *Linear models for the prediction of animal breeding values*. Cabi. 3rd ed.

Munoz, PR, et al., 2014 Unraveling additive from nonadditive effects using genomic relationship matrices. Genetics 198, 1759–1768. https://doi.org/10.1534/genetics.114.171322

R Core Team, 2016 *R*: A Language and Environment for Statistical Computing. R Foundation for Statistical Computing, Vienna, Austria.

Resende, MF, et al. 2012 Accuracy of genomic selection methods in a standard data set of loblolly pine (*Pinus taeda* l.). Genetics 190, 1503–1510. https://doi.org/10.1534/genetics.111.137026

Slater, AT, et al. 2014 Improving the analysis of low heritability complex traits for enhanced genetic gain in potato. Theoretical and applied genetics 127, 809–820. https://doi.org/10.1007/s00122-013-2258-7

Slater, AT, et al. 2016 Improving genetic gain with genomic selection in autotetraploid potato. The Plant Genome 9. https://doi.org/10.3835/plantgenome2016.02.0021 

Su, G, et al. 2012 Estimating additive and non-additive genetic variances and predicting genetic merits using genome-wide dense single nucleotide polymorphism markers. PloS one 7, e45293. https://doi.org/10.1371/journal.pone.0045293

VanRaden, P. 2008 Efficient methods to compute genomic predictions. Journal of dairy science 91, 4414–4423. https://doi.org/10.3168/jds.2007-0980

Vitezica, ZG, et al. 2013 On the additive and dominant variance and covariance of individuals within the genomic selection scope. Genetics 195, 1223–1230. https://doi.org/10.1534/genetics.113.155176

Yang, J, et al. 2010 Common snps explain a large proportion of the heritability for human height. Nature genetics 42, 565–569. https://doi.org/10.1038/ng.608

## ----knitr_init, echo=FALSE, cache=FALSE---------------------------------
library(knitr)
library(rmarkdown)

knitr::opts_chunk$set(collapse = TRUE,
                      comment = "#>",
                      fig.width = 6,
                      fig.height = 6,
                      fig.align = "center",
                      dev = "png",
                      dpi = 36,
                      cache = TRUE)



## ---- echo=FALSE, results='hide'-----------------------------------------
library(AGHmatrix)


## ---- eval=FALSE---------------------------------------------------------
## ## Install stable version
## install.packages("AGHmatrix")
## 
## ## Install development version
## #install.packages("devtools")
## #devtools::install_github("prmunoz/AGHmatrix")
## 
## ## Load
## library(AGHmatrix)


## ------------------------------------------------------------------------
data(ped.mrode)
ped.mrode
str(ped.mrode) #check the structure


## ---- eval=FALSE---------------------------------------------------------
## #Computing additive relationship matrix for diploids:
## Amatrix(ped.mrode, ploidy=2)
## 
## #Computing non-additive relationship matrix considering diploidy:
## Amatrix(ped.mrode, ploidy=2, dominance=TRUE)
## 
## #Computing additive relationship matrix considering autotetraploidy:
## Amatrix(ped.mrode, ploidy=4)
## 
## #Computing additive relationship matrix considering autooctaploidy:
## Amatrix(ped.mrode, ploidy=8)
## 
## #Computing additive relationship matrix considering autotetraploidy
## # and double-reduction of 10%:
## Amatrix(ped.mrode, ploidy=4, w=0.1)
## 
## #Computing additive relationship matrix considering autotetraploidy
## # and double-reduction of 10% as Slater et al. (2014):
## Amatrix(ped.mrode, ploidy=4, w=0.1, slater = TRUE)
## 
## #Computing additive relationship matrix considering autohexaploidy
## # and double-reduction of 10%:
## Amatrix(ped.mrode, ploidy=6, w=0.1)


## ---- eval=FALSE---------------------------------------------------------
## ?Amatrix


## ------------------------------------------------------------------------
data(snp.pine)
snp.pine[1:5,1:5]
str(snp.pine)


## ---- eval=FALSE---------------------------------------------------------
## #Computing the additive relationship matrix based on VanRaden 2008
## G_VanRadenPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
##                           maf=0.05, method="VanRaden")
## 
## #Computing the additive relationship matrix based on Yang 2010
## G_YangPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
##                       maf=0.05, method="Yang")
## 
## #Computing the dominance relationship matrix based on Su 2012
## G_SuPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
##                     maf=0.05, method="Su")
## 
## #Computing the dominance relationship matrix based on Vitezica 2013
## G_VitezicaPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
##                           maf=0.05, method="Vitezica")


## ---- eval=FALSE---------------------------------------------------------
## ?Gmatrix


## ---- eval=FALSE---------------------------------------------------------
## #Loading the data
## data(snp.sol)
## str(snp.sol)
## 
## #Computing the additive relationship matrix based on VanRaden 2008
## # adapted by Ashraf 2016
## G_VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)
## 
## #Computing the dominance (digenic) matrix based on Endelman 2018 (Eq. 19)
## G_Dominance <- Gmatrix(snp.sol, method="Endelman", ploidy=4)
## 
## #Computing the full-autopolyploid matrix based on Slater 2016 (Eq. 8
## # and 9)
## G_FullAutopolyploid <- Gmatrix(snp.sol, method="Slater", ploidy=4)
## 
## #Computing the pseudodiploid matrix based on Slater 2016 (Eq. 5, 6,
## # and 7)
## G_Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE)


## ---- eval=FALSE---------------------------------------------------------
## ?Gmatrix


## ---- eval=FALSE---------------------------------------------------------
## data(ped.sol)
## data(snp.sol)
## 
## #Computing the numerator relationship matrix 10% of double-reduction
## Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)
## 
## #Computing the additive relationship matrix based on VanRaden (modified)
## Gmat <- Gmatrix(snp.sol, ploidy=4, missingValue=-9,
##                 maf=0.05, method="VanRaden")
## 
## #Computing H matrix (Martini)
## Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini",
##                         ploidy=4, missingValue=-9, maf=0.05)
## 
## #Computing H matrix (Munoz)
## Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol,
##                       ploidy=4, method="Munoz",
##                       missingValue=-9, maf=0.05)


## ---- eval=FALSE---------------------------------------------------------
## data(snp.pine)
## A <- Gmatrix(SNPmatrix=snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
## D <- Gmatrix(SNPmatrix=snp.pine, method="Vitezica", missingValue=-9,maf=0.05)


## ---- eval=FALSE---------------------------------------------------------
## #Additive-by-Additive Interactions
## A_A <- A*A
## #Dominance-by-Additive Interactions
## D_A <- D*A
## #Dominance-by-Dominance Interactions
## D_D <- D*D


## ---- eval=FALSE---------------------------------------------------------
## #Additive-by-Additive-by-Additive Interactions
## A_A_A <- A*A*A
## #Additive-by-Additive-by-Dominance Interactions
## A_A_D <- A*A*D
## #Additive-by-Dominance-by-Dominance Interactions
## A_D_D <- A*D*D
## #Dominance-by-Dominance-by-Dominance Interactions
## D_D_D <- D*D*D


## ---- eval=FALSE---------------------------------------------------------
## #Loading the data example
## data(ped.mrode)
## 
## #Computing the matrix
## A <- Amatrix(data=ped.mrode, ploidy=4, w=0.1)
## 
## #Building its inverse
## Ainv <- solve(A)
## 
## #Exporting it. The function "formatmatrix"
## # will convert it and save in your working directory
## formatmatrix(Ainv, round.by=12, exclude.0=TRUE, name="Ainv")


## ----eval=FALSE,echo=FALSE-----------------------------------------------
## #To knit an this vignette into an .R file
## knitr::purl("vignettes/Tutorial_AGHmatrix.Rmd")


## ------------------------------------------------------------------------
sessionInfo()

library(AGHmatrix)
Gmatrix

inds <- 10
markers <- 100
markersdata <- matrix(sample(x=0:4, size=inds*markers, replace=TRUE), nrow=inds, ncol=markers)

markersdata
G1 <-Gmatrix(markersdata,ploidy=4,method="VanRaden")
G2 <-Gmatrix(markersdata,ploidy=4,method="Endelman")

SNPmatrix<-markersdata

inds <- 3
markers <- 4
markersdata <- matrix(sample(x=0:4, size=inds*markers, replace=TRUE), nrow=inds, ncol=markers)

Frequency <- colSums(SNPmatrix)/(nrow(SNPmatrix)*ploidy)
Frequency <- cbind(Frequency,1-Frequency)

Frequency <- cbind(apply(SNPmatrix, 2, function(x) alelleFreq(x,0))
                   , apply(SNPmatrix, 2, function(x) alelleFreq(x, 2)))

Q1 <- matrix(rep(c(9,4,25,16),3),nrow=3,byrow=TRUE)/144*6
Q2 <- matrix(c(0,4,10,4,
               9,0,0,12,
               0,0,15,0),nrow=3,byrow=TRUE)*3/12
Q3 <- matrix(c(0,2,2,0,
               6,0,0,6,
               0,0,6,0),nrow=3,byrow=TRUE)*0.5
Q<-Q1-Q2+Q3
tcrossprod(Q)/drop((6*crossprod(c(3/12,2/12,5/12,4/12)^2,c(1-c(3/12,2/12,5/12,4/12))^2)))

M1 <- matrix(c(0,2,2,1,
              3,0,0,3,
              0,0,3,0),nrow=3,byrow=TRUE)

M2 <-matrix(c(4,2,2,3,
             1,4,4,1,
             4,4,1,4),nrow=3,byrow=TRUE)

Gmatrix(M1,ploidy=4)
Gmatrix(M2,ploidy=4)

## Including Data from Endelman 2018
MolPotato <- read.table("~/git/AGHmatrix/data/FileS2.csv",header=TRUE,sep=",")
ind <- MolPotato[[1]]
MolPotato <- as.matrix(MolPotato[,-1])
rownames(MolPotato) <- ind
snp.potato <- MolPotato

save(snp.potato,file="snp.potato.rdata")

library(AGHmatrix)
G1 <- Gmatrix(MolPotato,ploidy=4)
G2 <- Gmatrix(MolPotato,ploidy=4,method="Slater")
G3 <- Gmatrix(MolPotato,ploidy=4,method="Endelman")

PedPotato <- read.table("~/git/AGHmatrix/data/FileS3.csv",header=TRUE,sep=",")
PedPotato[,2] <- PedPotato[PedPotato[,2],1]
PedPotato[which(PedPotato[,3]==0),3] <- NA
PedPotato[which(PedPotato[,4]==0),4] <- NA
PedPotato[,3] <- PedPotato[PedPotato[,3],1]
PedPotato[,4] <- PedPotato[PedPotato[,4],1]

ped.potato <- PedPotato[,c(2,3,4)]
ped.potato$GID <- as.character(ped.potato$GID)
ped.potato$Mother <- as.character(ped.potato$Mother)
ped.potato$Father <- as.character(ped.potato$Father)

ped.potato[which(is.na(ped.potato[,3])),3] <- "0"
ped.potato[which(is.na(ped.potato[,2])),2] <- "0"

A1<-Amatrix(ped.potato)
names(ped.potato)[1]<-"Ind"
A1<-Amatrix(ped.potato)
save(ped.potato,file="ped.potato.rdata")

data("snp.potato")
?Gmatrix

Gmatrix.VanRaden <- Gmatrix(snp.potato, method="VanRaden", ploidy=4)
Gmatrix.Endelman <- Gmatrix(snp.potato, method="Endelman", ploidy=4) 
Gmatrix.Slater <- Gmatrix(snp.potato, method="Slater", ploidy=4)
Gmatrix.Pseudodiploid <- Gmatrix(snp.potato, method="VanRaden", ploidy=4, pseudo.diploid=TRUE) 

Amatrix.potato <- Amatrix(ped.potato, ploidy=4)

##########################################
# 									
# Package: AGHmatrix 		
#
# File: explore_matrix.R 						
# Contains: explorematrix 						
# 									
# Written by Rodrigo Rampazo Amadeu 					
# 									
# First version: Feb-2014 						
# Last update: 14-Apr-2015 						
# License: GPL-3
# 									
#########################################

#' Explore a relationship matrix
#'
#' Given a data from 'Amatrix', 'Gmatrix', or 'Hmatrix' functions, return a list with exploratory analysis of the matrix.
#'
#' @param data output of function 'Amatrix' or 'Gmatrix' or 'Hmatrix'.
#' @param type "A" or "G" or "H". Default=NULL.
#' @param print if TRUE, print analysis. Default=TRUE.
#' @param w proportion of parental gametas IBD due to double reduction. Default=0. 
#' @param name csv file name
#'
#' @return list with exploratory analysis about the matrix: summary off-diagonal, summary diagonal, sort data (top shared genotypes)...
#'
#' @examples 
#' data(ped.mrode)
#' Amat <- Amatrix(ped.mrode)
#' explorematrix(Amat)
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#'
#' @export

explorematrix <- function(data = NULL,
                          type=NULL,
                          print=TRUE,
                          w=0,
                          name=NULL){

    cat("Doing the exploratory analysis... \n")
    Time = proc.time()
    A <- data
    double.reduction <- w
    dim <- c( nrow = nrow(A), ncol = ncol(A) )
    summary.off.diag <- summary(A[upper.tri(A,diag=FALSE)])
    sort.data <- sort.data(A) #Find top shared genotype in the data
    summary.diag <- summary(diag(A))
    sort.diag <- sort.diag(diag(A)) #Find top inbreedings in the diag
    summary.inbreeding <- summary(diag(A-1))

    number.markers.used <- data$number.markers.used
    listA <- list(type=type, dim = dim, matrix = A, summary.off.diag = summary.off.diag, sort.data = sort.data, summary.diag = summary.diag, sort.diag = sort.diag, summary.inbreeding=summary.inbreeding, class="explore.gen.data")

    Time = as.matrix(proc.time()-Time)
    cat("Completed! Time =", Time[3]/60," minutes \n")

    structure(listA,class="explore.gen.data")
}

## ----knitr_init, echo=FALSE, cache=FALSE--------------------------------------
library(knitr)
library(rmarkdown)

knitr::opts_chunk$set(collapse = TRUE,
                      comment = "#>",
                      fig.width = 7,
                      fig.height = 8,
                      fig.align = "center",
                      dev = "png",
                      dpi = 72,
                      cache = TRUE)


## ---- echo=FALSE, results='hide'----------------------------------------------
library(AGHmatrix)

## ---- eval=FALSE--------------------------------------------------------------
#  ## Install stable version
#  install.packages("AGHmatrix")
#  
#  ## Install development version
#  install.packages("devtools")
#  devtools::install_github("rramadeu/AGHmatrix")
#  
#  ## Load
#  library(AGHmatrix)

## -----------------------------------------------------------------------------
data(ped.mrode)
ped.mrode
str(ped.mrode) #check the structure

## ---- eval=FALSE--------------------------------------------------------------
#  #Computing additive relationship matrix for diploids (Henderson 1976):
#  Amatrix(ped.mrode, ploidy=2)
#  
#  #Computing dominant relationship matrix for diploids (Cockerham 1954):
#  Amatrix(ped.mrode, ploidy=2, dominance=TRUE)
#  
#  #Computing additive relationship matrix for autotetraploids (Kerr 2012):
#  Amatrix(ped.mrode, ploidy=4)
#  
#  #Computing additive relationship matrix for autooctaploids (Kerr 2012):
#  Amatrix(ped.mrode, ploidy=8)
#  
#  #Computing additive relationship matrix for autotetraploids
#  # and double-reduction of 0.1 (Kerr 2012):
#  Amatrix(ped.mrode, ploidy=4, w=0.1)
#  
#  #Computing additive relationship matrix for autotetraploids
#  # and double-reduction of 0.1 as in Slater et al. (2014):
#  Amatrix(ped.mrode, ploidy=4, w=0.1, slater = TRUE)
#  #not recommended, but kept in the package to reproduce some former analysis
#  
#  #Computing additive relationship matrix for autohexaploids
#  # and double-reduction of 0.1 (Kerr 2012):
#  Amatrix(ped.mrode, ploidy=6, w=0.1)

## ---- eval=FALSE--------------------------------------------------------------
#  ?Amatrix

## -----------------------------------------------------------------------------
data(snp.pine)
snp.pine[1:5,1:5]
str(snp.pine)

## ---- eval=FALSE--------------------------------------------------------------
#  #Computing the additive relationship matrix based on VanRaden 2008
#  G_VanRadenPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                            maf=0.05, method="VanRaden")
#  
#  #Computing the additive relationship matrix based on Yang 2010
#  G_YangPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                        maf=0.05, method="Yang")
#  
#  #Computing the dominance relationship matrix based on Su 2012
#  G_SuPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                      maf=0.05, method="Su")
#  
#  #Computing the dominance relationship matrix based on Vitezica 2013
#  G_VitezicaPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                            maf=0.05, method="Vitezica")

## ---- eval=FALSE--------------------------------------------------------------
#  ?Gmatrix

## ---- eval=FALSE--------------------------------------------------------------
#  #Loading the data
#  data(snp.sol)
#  str(snp.sol)
#  
#  #Computing the additive relationship matrix based on VanRaden 2008
#  # adapted by Ashraf 2016
#  G_VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)
#  
#  #Computing the dominance (digenic) matrix based on Endelman 2018 (Eq. 19)
#  G_Dominance <- Gmatrix(snp.sol, method="Endelman", ploidy=4)
#  
#  #Computing the full-autopolyploid matrix based on Slater 2016 (Eq. 8
#  #and 9)
#  G_FullAutopolyploid <- Gmatrix(snp.sol, method="Slater", ploidy=4)
#  
#  #Computing the pseudodiploid matrix based on Slater 2016 (Eq. 5, 6,
#  #and 7)
#  G_Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE)
#  
#  #Computing G matrix with specific weight for each marker as
#  # in Liu et al. (2020).
#  Gmatrix_weighted <- Gmatrix(snp.sol, method="VanRaden", weights = runif(3895,0.001,0.1), ploidy=4)

## ---- eval=FALSE--------------------------------------------------------------
#  ?Gmatrix

## ---- eval=FALSE--------------------------------------------------------------
#  #Loading the data
#  library(AGHmatrix)
#  data(snp.sol)
#  snp.sol.ratio = snp.sol/4 #transforming it in a ratio of the minor allele frequency
#  Gmatrix <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, ratio=FALSE)
#  Gmatrix.ratio <- Gmatrix(snp.sol.ratio, method="VanRaden", ploidy=4, ratio=TRUE)
#  Gmatrix[1:5,1:5]==Gmatrix.ratio[1:5,1:5]
#  
#  ## it also has the ploidy.correction option
#  Gmatrix.alternative <- Gmatrix(snp.sol,
#                                 method="VanRaden",
#                                 ploidy=4,
#                                 ratio=FALSE,
#                                 ploidy.correction=TRUE)
#  
#  Gmatrix.ratio.alternative <- Gmatrix(snp.sol.ratio,
#                                       method="VanRaden",
#                                       ploidy=4,
#                                       ratio=TRUE,
#                                       ploidy.correction=TRUE)
#  Gmatrix[1:5,1:5]==Gmatrix.alternative[1:5,1:5]
#  Gmatrix.alternative[1:5,1:5]==Gmatrix.ratio.alternative[1:5,1:5]

## ---- eval=FALSE--------------------------------------------------------------
#  data(ped.sol)
#  data(snp.sol)
#  
#  #Computing the numerator relationship matrix 10% of double-reduction
#  Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)
#  Gmat <- Gmatrix(snp.sol, ploidy=4,
#                  maf=0.05, method="VanRaden")
#  Gmat <- round(Gmat,3) #see appendix
#  
#  #Computing H matrix (Martini)
#  Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini",
#                          ploidy=4, missingValue=-9, maf=0.05)
#  
#  #Computing H matrix (Munoz)
#  Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol,
#                        ploidy=4, method="Munoz",
#                        missingValue=-9, maf=0.05)

## ---- eval=FALSE--------------------------------------------------------------
#  data(snp.pine)
#  A <- Gmatrix(SNPmatrix=snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
#  D <- Gmatrix(SNPmatrix=snp.pine, method="Vitezica", missingValue=-9,maf=0.05)

## ---- eval=FALSE--------------------------------------------------------------
#  #Additive-by-Additive Interactions
#  A_A <- A*A
#  #Dominance-by-Additive Interactions
#  D_A <- D*A
#  #Dominance-by-Dominance Interactions
#  D_D <- D*D

## ---- eval=FALSE--------------------------------------------------------------
#  #Additive-by-Additive-by-Additive Interactions
#  A_A_A <- A*A*A
#  #Additive-by-Additive-by-Dominance Interactions
#  A_A_D <- A*A*D
#  #Additive-by-Dominance-by-Dominance Interactions
#  A_D_D <- A*D*D
#  #Dominance-by-Dominance-by-Dominance Interactions
#  D_D_D <- D*D*D

## ---- eval=FALSE--------------------------------------------------------------
#  #Loading the data example
#  data(ped.mrode)
#  
#  #Computing the matrix
#  A <- Amatrix(data=ped.mrode, ploidy=4, w=0.1)
#  
#  #Building its inverse
#  Ainv <- solve(A)
#  
#  #Exporting it. The function "formatmatrix"
#  # will convert it and save in your working directory
#  formatmatrix(Ainv, round.by=12, exclude.0=TRUE, name="Ainv")

## -----------------------------------------------------------------------------
pedigree = data.frame(id=1:8,
                      parent1 = c(0,0,0,0,1,2,3,5),
                      parent2 = c(0,0,0,0,2,3,4,6),
                      parent3 = c(0,0,0,0,3,4,0,7),
                      parent4 = c(0,0,0,0,0,0,0,1),
                      parent5 = 0)

print(pedigree)
AmatrixPolyCross(pedigree)

## -----------------------------------------------------------------------------
AmatrixPolyCross(pedigree,fixedParent=TRUE)

## ---- eval=TRUE,echo=FALSE----------------------------------------------------
x = c(1000,5000,10000,20000,30000,40000,50000,60000,70000,80000,90000,100000)/1000 #Pedigree Size
y = c(252156,622500,1795260,6481064,14313448,25227680,49081224,70622336,96017144,125320048,158444856,194731908)/1e+6 #RAM GB 
ytime = c(0.0025, 0.080, 0.2, 0.89, 1.62,3.01,4.52,7.12,9.15,13.13,15.13,20) #minutes
df = data.frame(size=x,ram=y,time=ytime)

plot(x=df$size,y=df$ram, 
     ylab = "RAM (GB) at the peak of Amatrix() function",
     xlab = "Pedigree size (in 1,000 rows)",
     type="b",
     axes=FALSE)
axis(side = 2, at = c(0,4,8,16,32,48,64,96,144,192),cex.axis=.75)
axis(side = 1, at = c(1,5,10,20,30,40,50,60,70,80,90,100),cex.axis=.75)

plot(x=df$size,y=df$time, type="b",
     ylab = "Time to run (minutes) the Amatrix() function",
     xlab = "Pedigree size (in 1,000 rows)",
     axes=FALSE)
axis(side = 2, at = seq(0,20,2),cex.axis=.75)
axis(side = 1, at = c(1,5,10,20,30,40,50,60,70,80,90,100),cex.axis=.75)

## ----eval=FALSE,echo=FALSE----------------------------------------------------
#  #To knit an this vignette into an .R file
#  knitr::purl("vignettes/Tutorial_AGHmatrix.Rmd")

## -----------------------------------------------------------------------------
sessionInfo()

#########################################
# 									
# Package: AGHmatrix 							
# 									
# File: converttofrequency.R						
# Contains: convertofrequency						
# 									
# Written by Rodrigo Rampazo Amadeu 				
# 									
# First version: Feb-2014 						
# Last update: 14-Apr-2015 						
# License: GPL-3
# 									
#########################################

#' Converts molecular (AA,AB,BB) data in a frequency format (0,0.5,1).
#'
#' Converts molecular data in frequency format. Molecular data can be coded with numbers, letters or any ascii characters.
#'
#' @param file path of your file (marker in rows and individual in columns).
#' @param ploidy ploidy of your data 2 or 4, Default=2.
#' @param format 1 if the data code as 0,1,2,..; 2 if if the data code as ...,-1,0,1,...; 3 if if the data code as "BB","AB","AA" or "BBBB", "ABBB", "AABB", "AAAB", "AAAA"; 4 if different. Than, you need to specify in genotype parameter.
#' @param unk unknown value assumed (default=NA)
#' @param genotype available if format=4. Please insert your genotype categories here as a list. e.g.:  genotype=c("CCCC","CGGG","CCGG","CGGG","GGGG") in this example CCCC will be coded as 0, CGGG as 0.25, CCGG as 0.5, CGGG as 0.75, G as 1. Default=NULL.
#' @param output choose a the name of output file. Default="convdata".
#' @param dominant if TRUE, returns the dominant parameterization.
#' @param transpose if TRUE, returns the transposable matrix.
#'
#' @return csv file markers x individuals with frequencies instead of molecular code.
#' 
#' @examples 
#' converttofrequency()
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#'
#' @export

converttofrequency <- function(
    file=NULL,
    ploidy=2,
    format=4,
    unk=NA,
    genotype=NULL,
    dominant=FALSE,
    output="convdata",
    transpose=FALSE)
    {
    if( !is.null(file)){
        y <- as.matrix(read.csv(file,header=FALSE,sep=","))
    }else(stop(deparse("Select a file name")))

    if(transpose)
        y<-t(y)

    ind.names <- y[,1]
    ind.names <- c(as.matrix(ind.names[-1]))
    markers <- y[1,]
    markers <- c(as.matrix(markers[-1]))
    cat("Check if the following information is correct.","\n","If not, correct the data ...", "\n")
    cat("Considering",length(ind.names),"individual names:",head(ind.names),"...","\n")
    cat("Considering",length(markers),"markers:",head(markers),"...","\n")
    y <- y[-1,-1]
    y[y==unk] <- NA
    if(format==1 || format==2){ #format 1 = 0,1,2; = 0,1,2,3,4; ...
        y <- matrix(as.numeric(y),nrow=length(ind.names))
        if(dominant)
        y<- y/ploidy
        if(format==2){ #format 2 = -1,0,1; = -2,-1,0,1,2; ...
            y <- y + 0.5
        }
    }
    if(format==3 || format==4){ #format 3 = BB,AB,AA
        if(format==3){
            if(ploidy==2)
                genotype<-c("BB","AB","AA")
            if(ploidy==4)
                genotype<-c("BBBB","ABBB","AABB","AAAB","AAAA")
        }
        if(format==4 && is.null(genotype))
            stop(deparse("Choose a format of your data. If equal to 4, indicate the genotype"))
        match.alg <- genotype
        cat("Converting the data to frequency using the following transformation...","\n")
        cat(c(0:(length(genotype)-1))/(length(genotype)-1),"\n")
        cat(genotype,"\n")
        code <- c(0,seq(1:(length(match.alg)-1)))
        for(i in 1:length(match.alg))
            y[y==match.alg[i]]=code[i]
        y <- matrix(as.numeric(y),nrow=length(ind.names))
        y <- y/ploidy
    }

if(dominant){
    cat("Converting the data to dominant type...","\n")
    cat(c(0:(length(genotype)-1))/(length(genotype)-1),"\n")
    cat(c(0,rep(1,length(genotype)-2),0),"\n")
    y <- (y!=0)*(y!=1)
}

#y <- rbind(markers,y)
colnames(y) <- markers
rownames(y) <- ind.names
#  rownames(y) <- colnames(y) <- NULL
                                        #  y[1,1]<-"Ind"
    if(dominant){
        write.table(y,file=paste(output,"dom.csv",sep=""),col.names=T,quote=FALSE,row.names=T,sep=",")
        cat(paste("Data saved as: ",output,"dom.csv",sep=""))
}else{
    write.table(y,file=paste(output,".csv",sep=""),col.names=T,quote=FALSE,row.names=T,sep=",")
    cat(paste("Data saved as: ",output,".csv",sep=""))
}
}

#' Pedigree Data
#'
#' Data from pedigree example proposed by Mrode 2005
#'
#' @docType data
#'
#' @usage data(ped.mrode)
#'
#' @format table
#'
#' @keywords datasets
#'
#'
#' @references R. A. Mrode, R. Thompson. Linear Models for the Prediction of Animal Breeding Values. CABI, 2005.
#'
#' @examples data(ped.mrode)
#'
#' @name ped.mrode
NULL

#' Molecular data for autopolyploid examples
#'
#' Dataset extract from  supplementary material from Endelman et al. (2018). 
#' SNP marker matrix from Pine tree coded as 0,1,2,3,4 and missing value as -9.
#'
#' @docType data
#'
#' @usage data(snp.sol)
#'
#' @format data.frame
#'
#' @keywords datasets
#'
#' @references Endelman, JB, et al., 2018 Genetic variance partitioning and genome-wide prediction with allele dosage information in autotetraploid potato. Genetics, 209(1) pp. 77-87.
#' 
#' @examples data(snp.sol)
#'
#' @name snp.sol
NULL

########################################
# 									
# Package: AGHmatrix 							
# 									
# File: Hmatrix.R 							
# Contains: Hmatrix 							
# 									
# Written by Rodrigo Rampazo Amadeu 					
# 									
# First version: Feb-2014 						
# Last update: 09-Jul-2019 						
# License: GPL-3
# 									
#######################################

#' Construction of Combined Relationship Matrix H
#'
#' Given a matrix A and a matrix G returns a H matrix. H matrix is the relationship matrix using combined information from the pedigree and genomic relationship matrices. First, you need to compute the matrices separated and then use them as input to build the combined H matrix. 
#' Two methods are implemented: `Munoz` shrinks the G matrix towards the A matrix scaling the molecular relatadness by each relationship classes; 
#' `Martini` is a modified version from Legarra et al. (2009) where combines A and G matrix using scaling factors. When method is equal `Martini` and `tau=1` and `omega=1` you have the same H matrix as in Legarra et al. (2009).
#'
#' @param A A matrix from function Amatrix
#' @param G G matrix from function Gmatrix
#' @param markers matrix marker which generated the Gmatrix
#' @param c constant value of H computation, default: c=0
#' @param method "Martini" or "Munoz", default="Martini"
#' @param missingValue missing value in data, default=-9.
#' @param maf max of missing data accepted to each markerm default=0.05.
#' @param ploidy data ploidy (an even number between 2 and 20), default=2.
#' @param tau to be used for Martini's method, default=1. 
#' @param omega to be used of Martini's method, default=1.
#' @param roundVar only used for Munoz's method, how many digits to consider the relationship be of same class, default=2.
#' @param ASV if TRUE, transform matrix into average semivariance (ASV) equivalent (K = K / (trace(K) / (nrow(K)-1))). Details formula 2 of Fieldmann et al. (2022). Default = FALSE.
#' 
#' @return H Matrix with the relationship between the individuals based on pedigree and corrected by molecular information
#'
#' @examples 
#' \dontrun{
#' data(ped.sol)
#' data(snp.sol)
#' #Computing the numerator relationship matrix 10% of double-reduction
#' Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)
#' #Computing the additive relationship matrix based on VanRaden (modified)
#' Gmat <- Gmatrix(snp.sol, ploidy=4, 
#'                 maf=0.05, method="VanRaden")
#' Gmat <- round(Gmat,3) #to be easy to invert
#' 
#' #Computing H matrix (Martini)
#' Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini", 
#'                      ploidy=4, 
#'                      maf=0.05)
#'                      
#' #Computing H matrix (Munoz)
#' Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol, 
#'                       ploidy=4, method="Munoz",
#'                       roundVar=2,
#'                       maf=0.05)
#' }
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#' @author Thiago de Paula Oliveira \email{toliveira@abacusbio.com}
#' @importFrom stats approx
#' @references \emph{Feldmann MJ, et al. 2022. Average semivariance directly yields accurate estimates of the genomic variance in complex trait analyses. G3 (Bethesda), 12(6).}
#' @references \emph{Munoz, PR. 2014 Unraveling additive from nonadditive effects using genomic relationship matrices. Genetics 198, 1759-1768}
#' @references \emph{Martini, JW, et al. 2018 The effect of the H-1 scaling factors tau and omega on the structure of H in the single-step procedure. Genetics Selection Evolution 50(1), 16}
#' @references \emph{Legarra, A, et al. 2009 A relationship matrix including full pedigree and genomic information. Journal of Dairy Science 92, 4656-4663}
#' @export
Hmatrix <- function(A=NULL,
                    G=NULL,
                    markers=NULL,
                    c=0,
                    method="Martini",
                    tau=1,
                    omega=1,
                    missingValue=-9,
                    maf=0,
                    ploidy=2,
                    roundVar=3,
                    ASV=FALSE){
  #-----------------------------------------------------------------------------
  # pre-checks
  #-----------------------------------------------------------------------------
  if (is.null(A) || is.null(G))
    stop("Please provide both A and G.")
  
  valid_methods <- c("Martini", "Munoz")
  if (!is.character(method) || length(method) != 1L || 
      !(method %in% valid_methods)) {
    stop("Invalid `method`: ", sQuote(method),
         ". Allowed methods: ", paste(sQuote(valid_methods), collapse = ", "), 
         ".", call. = FALSE)
  }
  
  # Basic input checks
  if (!is.matrix(A) || !is.matrix(G))
    stop("A and G must be matrices.")
  
  if (is.null(rownames(A)) || is.null(colnames(A)) ||
      is.null(rownames(G)) || is.null(colnames(G)))
    stop("A and G must be square with dimnames (IDs).")
  
  if (!identical(rownames(A), colnames(A)) ||
      !identical(rownames(G), colnames(G)))
    stop("A and G must be square with matching row/colnames in each.")
  
  Aorig <- A; Gorig <- G
  An <- rownames(Aorig); Gn <- rownames(Gorig)
  
  Time <- proc.time()
  cat("Comparing the matrices... \n")
  #-----------------------------------------------------------------------------
  # Missing pattern
  #-----------------------------------------------------------------------------
  # in A but not in G
  idxGmiss <- is.na(match(An, Gn))
  # in G but not in A
  idxAmiss <- is.na(match(Gn, An))
  if (any(!idxAmiss)) {
    # order we'll keep in G submatrix
    Gnhat <- Gn[!idxAmiss]             
  } else Gnhat <- Gn
  if (any(!idxGmiss)) {
    # order we'll keep in A submatrix (Munoz method)
    Anhat <- An[!idxGmiss]
  } else Anhat <- An
  
  #-----------------------------------------------------------------------------
  # keep G on its available IDs
  #-----------------------------------------------------------------------------
  G <- Gorig[Gnhat, Gnhat, drop = FALSE]
  elapsed <- (proc.time() - Time)[["elapsed"]]
  cat("Completed! Time =", elapsed / 60, "minutes\n")
  
  #-----------------------------------------------------------------------------
  # main dispatch
  #-----------------------------------------------------------------------------
  cat("Computing the H matrix... \n")
  Time <- proc.time()
  
  #-----------------------------------------------------------------------------
  # Martini
  #-----------------------------------------------------------------------------
  if (method == "Martini") {
    # build the global ID order
    idH <- unique(c(rownames(G), rownames(Aorig)))
    idH <- rev(idH)
    
    # reindex A once in that order
    A <- Aorig[idH, idH, drop = FALSE]
    inG <- !is.na(match(idH, rownames(G)))
    # partition indices
    idxAonly <- which(!inG)  # in A only
    idxG     <- which(inG)   # in G (genotyped)
    
    # pull the blocks we need
    A12 <- A[idxAonly, idxG, drop=FALSE]
    A22 <- A[idxG,     idxG, drop=FALSE]
    # same order as A22
    G22 <- G[idH[idxG], idH[idxG], drop=FALSE]
    
    # C++ core (does all inversions and multiplications)
    blk <- H_martini_blocks(A12, A22, G22, tau = tau, omega = omega)
    H11 <- blk$H11
    H12 <- blk$H12
    H21 <- blk$H21
    H22corr <- blk$H22corr
    
    # start from A and add corrections in-place by blocks
    H <- A
    if (length(idxAonly)) {
      H[idxAonly, idxAonly] <- H[idxAonly, idxAonly] + H11
      H[idxAonly, idxG]     <- H[idxAonly, idxG]     + H12
      H[idxG,     idxAonly] <- H[idxG,     idxAonly] + H21
    }
    if (length(idxG)) {
      H[idxG, idxG] <- H[idxG, idxG] + H22corr
    }
    
    if (ASV) H <- get_ASV(H)
    
    # attributes
    nm <- attr(Gorig, "nmarkers", exact = TRUE)
    if (is.null(nm)) nm <- NA_integer_
    attr(H, "method")   <- method
    attr(H, "ploidy")   <- ploidy
    attr(H, "nmarkers") <- nm
    
    Time <- as.matrix(proc.time()-Time)
    cat("\n","Completed! Time =", Time[3]/60," minutes \n")
    return(H)
  }
  
  #-----------------------------------------------------------------------------
  # Munoz
  #-----------------------------------------------------------------------------
  if (method == "Munoz") {
    # A in A-order on Anhat; G in G-order on Gnhat.
    A <- Aorig[Anhat, Anhat, drop = FALSE]
    G <- Gorig[Gnhat, Gnhat, drop = FALSE]
    
    # MarkersMatrix computed on raw markers, then subset in G's order
    if (is.null(markers))
      stop("Aborting: For Munoz method you need to specify 'markers'.")
    Mm <- as.matrix(data.matrix(markers))
    if (!is.na(missingValue)) Mm[Mm == missingValue] <- NA
    markersmatrix <- Gmatrix_MarkersMask(Mm)
    # give it names (needed for character subsetting)
    if (!is.null(rownames(Mm))) {
      dimnames(markersmatrix) <- list(rownames(Mm), rownames(Mm))
    }
    if (!is.null(rownames(markersmatrix))) {
      markersmatrix <- markersmatrix[Gnhat, Gnhat, drop = FALSE]
    } else {
      if (nrow(markersmatrix) != length(Gnhat))
        stop("MarkersMatrix has no rownames and size doesn't match G.")
    }
    
    # C++ calculation
    agg <- munoz_var_mean_by_Aclass(G, A, round_digits = roundVar)
    varA     <- agg$varA
    classes  <- as.numeric(agg$classes)
    v_by_cls <- as.numeric(agg$var_by_class)
    v_by_cls[v_by_cls == 0]         <- NA_real_
    v_by_cls[is.infinite(v_by_cls)] <- NA_real_
    if (anyNA(v_by_cls)) {
      filled <- approx(x = seq_along(v_by_cls), y = v_by_cls,
                       xout = seq_along(v_by_cls), rule = 2)$y
      # rebuild varA
      class_id <- match(round(A, roundVar), classes)
      varA <- matrix(filled[class_id], nrow(A), ncol(A),
                     dimnames = dimnames(A))
    }
    
    # beta and H
    if (any(markersmatrix <= 0, na.rm = TRUE))
      stop("MarkersMatrix contains zeros or negatives; cannot compute 1/count.")
    invM <- 1 / markersmatrix
    beta <- 1 - (c + invM / varA)
    Hloc <- beta * (G - A) + A
    
    # write back into A's frame at Anhat (legacy)
    H <- Aorig
    H[Anhat, Anhat] <- Hloc
    
    if (ASV) H <- get_ASV(H)
    
    # attributes
    nm <- tryCatch(ncol(markers), error = function(e) NA_integer_)
    attr(H, "method")   <- method
    attr(H, "ploidy")   <- ploidy
    attr(H, "nmarkers") <- nm
    
    Time <- as.matrix(proc.time() - Time)
    cat("\n","Completed! Time =", Time[3]/60," minutes \n")
    cat("\n","Returning H = A matrix corrected by G... \n")
    return(H)
  }
}

#########################################################################
# 									
# Package: AGHmatrix 							
# 									
# File: expandAmatrix.R
# Contains: expandAmatrix 							
# 									
# Written by Rodrigo Rampazo Amadeu
# 									
# First version: Oct-2021 
# Last update: 03-Nov-2019 						
# License: GPL-3
# 								
#########################################################################

#' Add new crosses to a current A matrix
#'
#' Expand a current A matrix with a new pedigree. The parents in the new pedigree should also be in the A matrix.
#'
#' @param newPedigree pedigree data name (3-column way format). Unknown value should be equal 0.
#' @param A numerator relationship matrix output from Amatrix function.
#' @param returnAll if TRUE returns old A with new A, if FALSE returns only new A
#' 
#' @return Matrix with the Relationship between the individuals.
#'
#' @examples
#' data(ped.sol)
#' ped.initial = ped.sol[1:1120,]
#' ped.new = ped.sol[-c(1:1120),]
#' #Computing additive relationship matrix:
#' A = Amatrix(ped.initial, ploidy=2)
#' Anew = expandAmatrix(ped.new, A)
#' 
#' #Comparing with one-step building..
#' Afull = Amatrix(ped.sol, ploidy=2)
#' test = Anew-Afull
#' which(test!=0)
#' 
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#' 
#' @export

expandAmatrix <- function(newPedigree = NULL,
                          A=NULL,
                          returnAll=TRUE){

  if( is.null(newPedigree))
    stop(deparse("newPedigree argument is missing"))
  
  if( is.null(A))
    stop(deparse("A argument is missing"))
  
    
  if(any(is.na(match(newPedigree[,2],rownames(A))))+any(is.na(match(newPedigree[,3],rownames(A)))))
    stop(deparse("There are individuals in the new pedigree with missing parents in A"))
  
  ## Creating a line of 0s
  A = rbind(A,rep(0,nrow(A)))
  A = cbind(A,c(rep(0,ncol(A)),1))
  colnames(A)[ncol(A)] = rownames(A)[ncol(A)] = "0"
  
  NewA = A[match(newPedigree[,2],rownames(A)),match(newPedigree[,2],rownames(A))] +
    A[match(newPedigree[,2],rownames(A)),match(newPedigree[,3],rownames(A))] + 
    A[match(newPedigree[,3],rownames(A)),match(newPedigree[,2],rownames(A))] + 
    A[match(newPedigree[,3],rownames(A)),match(newPedigree[,3],rownames(A))]
  
  NewA = NewA/4
  diag(NewA) = 1+diag(A[match(newPedigree[,2],rownames(A)),match(newPedigree[,3],rownames(A))])/2
  
  if(!returnAll){
    rownames(NewA)=colnames(NewA)=newPedigree[,1]
    return(NewA)
  }else{
    NewOldA = A[match(newPedigree[,2],rownames(A)),] + A[match(newPedigree[,3],rownames(A)),]
    NewOldA = NewOldA/2
    NewOldA = NewOldA[,-ncol(NewOldA)]
    NewOldA = cbind(rbind(A[-ncol(A),-ncol(A)],NewOldA),
                    rbind(t(NewOldA),NewA))
    return(NewOldA)
  }
}

#########################################
# 									
# Package: AGHmatrix 							
# 									
# File: datatreat.R							
# Contains: datatreat asciitonumber 
# 									
# Written by Rodrigo Rampazo Amadeu 					
# 									
# First version: Feb-2014 					
# Last update: 24-Apr-2015 						
# License: GPL-3	
# 									
#########################################

#' Organizes pedigree data in a chronological way using C++
#'
#' This function organizes pedigree data in a chronological way and returns a list:
#' i) parental 1 values (numeric); ii) parental 2 values (numeric); iii) individual names.
#' Also saves a `.txt` file if `save = TRUE`.
#'
#' @param data A 3-column data frame or matrix with individual, sire, and dam
#' @param unk Code for unknown parent (default = 0)
#' @param n.max Max number of iterations (default = 50)
#' @param save If TRUE, saves the reordered pedigree to a text file
#'
#' @return A list with elements: sire (numeric), dire (numeric), ind_data (names)
#' @author Rodrigo R. Amadeu \email{rramadeu@gmail.com}
#' @author Thiago de Paula Oliveira
#' 
#' @export
#'
#' @examples
#' # data(ped.mrode)
#' # datatreat(ped.mrode)
datatreat <- function(data = NULL,
                      n.max = 50,
                      unk = 0,
                      save = FALSE) {
  if (is.null(data)) {
    stop("Select a data name")
  }
  
  # Already character matrix
  if (inherits(data, "character") && is.matrix(data)) {
    data_char <- data
  } else if (is.data.frame(data) && ncol(data) == 3) {
    # Convert data frame row-wise
    data_char <- matrix(
      unlist(lapply(data, as.character), use.names = FALSE),
      ncol = 3
    )
  } else {
    stop("Input must be a character matrix or a data.frame with 3 columns.")
  }
  
  datatreat_cpp(data_char, n_max = n.max, unk = as.character(unk), save = save)
}


# This function creates a list with numeric indices given a pedigree data. 
# Also it checks if all the listed parent name are listed before in the individual 
# name column and if the parent exist in the matrix . 
#' Convert pedigree names to numeric indices using C++
#'
#' @param pedigree.data A 3-column matrix or data frame with individual, sire, and dam
#' @param unk Code for unknown parent (default = "0")
#' @return A list with sire, dam, and individual names
#' @author Rodrigo R. Amadeu \email{rramadeu@gmail.com}
#' @author Thiago de Paula Oliveira
#' @export
asciitonumber <- function(pedigree.data, unk = 0) {
  pedigree.data <- as.matrix(pedigree.data)
  if (ncol(pedigree.data) != 3) {
    stop("Data with more than 3 columns, please verify")
  }
  ascii_to_number(pedigree.data, as.character(unk))
}

# This function verify which rows in a pedigree data has missing parental or conflictuos data
verifyped <- function(pedigree,
                      unk=0
){
  flag<-FALSE
  if( ncol(pedigree) != 3 ){
    print("Data with more than 3 columns, please verify")
    flag<-TRUE
    return(flag)
  }
  
  if(length(unique(pedigree[,1]))<nrow(pedigree)){
    print("Data with repeated entry, please verify the following entries lines")
    print(which(duplicated(pedigree[,1]),arr.ind=TRUE))
    flag<-TRUE
    return(flag)
  }
  
  #Treating all as numeric
  ind.data <- as.vector(pedigree[,1])
  sire.data <- as.vector(pedigree[,2])
  dire.data <- as.vector(pedigree[,3])
  sire <- match(sire.data, c(ind.data,"0"))
  dire <- match(dire.data, c(ind.data,"0"))
  ind <- as.vector(c(1:length(ind.data)))
  
  missing <- c()   
  #Verify the individual w/ same name in sire/dire
  missing$conflict <- c(which(sire == ind),which( dire == ind))
  if(length(missing$conflict)>0){
    print("The following rows have the individual name equals to the parental name. Please verify.")
    print(pedigree[missing$conflict,])
    flag<-TRUE
  }
  
  #Verify the missing sire (Parent 1)
  missing$sire.na <- c( which(is.na(sire)))
  if(length(missing$sire)>0){
    print("The following rows have the parental 1 name (column 2) missing in the pedigree. Please verify.")
    print(pedigree[missing$sire,])
    flag<-TRUE
  }
  
  #Verify the missing dire (Parent 2)
  missing$dire.na <- c( which(is.na(dire)))
  if(length(missing$dire)>0){
    print("The following rows have the parental 2 name (column 3) missing in the pedigree. Please verify.")
    print(pedigree[missing$dire,])
    flag<-TRUE
  }
  return(flag)
}

#########################################
# 									
# Package: AGHmatrix 							
# 									
# File: filterpedigree.R							
# Contains: filterpedigree
# 									
# Written by Rodrigo Rampazo Amadeu 					
# 									
# First version: Apr-2021 					
# Last update: 09-Apr-2021 						
# License: GPL-3	
# 									
#########################################

#' Filter the pedigree to keep only the genealogy of a subset of individuals
#'
#' Filter the pedigree to keep only the genealogy of a subset of individuals
#' @param inds vector with strings of individuals to keep their genealogy in the matrix
#' @param data name of the pedigree data frame. Default=NULL.
#'
#' @return a data frame with pedigree containing the genealogy of the selected individuals
#' 
#' @examples 
#' data(ped.sol)
#' new.ped.sol = filterpedigree(inds = c("MSW168-2","W14090-3","W14090-4"),data=ped.sol)
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#'
#' @export

filterpedigree <- function(inds, data){
  output <- NULL
  progress <- round(length(inds)/10)
  perc <- 10
  for(i in 1:length(inds)){
    if(length(inds)>100){
      if(i %% progress ==0){
        cat(paste0(perc,"% \n"))
        perc=perc+10
      }
    }
    ped_out<- data[which(data[,1] == inds[i]),]
    if(nrow(ped_out)==0){
      stop(deparse(paste(inds[i],"doesn't exist in this pedigree.")))
    }
    trigger <- 1
    while(trigger>0){
      init <- nrow(ped_out)
      ped_in <- data[which(data[,1] %in% c(ped_out[,2],ped_out[,3])),]
      ped_out <- unique(rbind(ped_in, ped_out))
      trigger <- nrow(ped_out)-init
    }
    output = rbind(output,ped_out)
    output = unique(output)
  }
  return(output)
}

#####################################################################
# 									
# Package: AGHmatrix 							
# 									
# File: Gmatrix.R
# Contains: Gmatrix slater_par check_Gmatrix_data			
# 									
# Written by Rodrigo Rampazo Amadeu 			
# Contributors: Marcio Resende Jr, Leticia AC Lara, Ivone Oliveira, Luis Felipe V Ferrao
# Thiago Oliveira
#
# First version: Feb-2014 					
# Last update: 21-Jul-2026 						
# License: GPL-3	
# 									
#####################################################################

#' Construction of Relationship Matrix G
#'
#' Given a matrix (individual x markers), a method, a missing value, and a maf 
#' threshold, return a additive or non-additive relationship matrix.
#' For diploids, the methods "Yang" and "VanRaden" for additive relationship 
#' matrices, and "Su" and "Vitezica" for non-additive relationship matrices 
#' are implemented. For autopolyploids, the method "VanRaden" for additive 
#' relationship, method "Slater" for full-autopolyploid model including 
#' non-additive effects, and pseudo-diploid parametrization are implemented.
#' Weights are implemented for "VanRaden" method as described in Liu (2020).
#' 
#' @param SNPmatrix matrix (n x m), where n is is individual names and m is 
#' marker names (coded inside the matrix as 0, 1, 2, ..., ploidy, and, 
#' missingValue). 
#' @param method "Yang" or "VanRaden" for marker-based additive relationship 
#' matrix. "Su" or "Vitezica" for marker-based dominance relationship matrix. 
#' "Slater" for full-autopolyploid model including non-additive effects.
#' "Endelman" for autotetraploid dominant (digentic) relationship matrix. 
#' "MarkersMatrix" for a matrix with the amount of shared markers between 
#' individuals (3).
#' Default is "VanRaden", for autopolyploids will be computed a scaled 
#' product (similar to Covarrubias-Pazaran, 2006).
#' @param missingValue missing value in data. Default=-9.
#' @param thresh.missing threshold on missing data, SNPs below of this 
#' frequency value will be maintained, if equal to 1, no threshold and 
#' imputation is considered. Default = 1.
#' @param maf minimum allele frequency accepted to each marker. Default=0.
#' @param verify.posdef verify if the resulting matrix is positive-definite. 
#' Default=FALSE.
#' @param ploidy data ploidy (an even number between 2 and 20). Default=2.
#' @param pseudo.diploid if TRUE, uses pseudodiploid parametrization of 
#' Slater (2016).
#' @param ratio if TRUE, molecular data are considered ratios and its 
#' computed the scaled product of the matrix (as in "VanRaden" method).
#' @param impute.method "mean" to impute the missing data by the mean per 
#' marker, "mode" to impute the missing data by the mode per marker, 
#' "global.mean" to impute the missing data by the mean across all markers,
#' "global.mode" to impute the missing data my the mode across all marker. 
#' Default = "none".
#' @param integer if FALSE, not check for integer numbers. Default=TRUE.
#' @param ratio.check if TRUE, run Mcheck with ratio data.
#' @param weights vector with weights for each marker. Only works if 
#' method="VanRaden". Default is a vector of 1's (equal weight).
#' @param ploidy.correction It sets the denominator (correction) of the 
#' rossprod. Used only when ploidy > 2 for "VanRaden" and ratio models.
#' If TRUE, it uses the sum of "Ploidy" times "Frequency" times "(1-Frequency)" 
#' of each marker as method 1 in VanRaden 2008 and Endelman (2018).
#' When ratio=TRUE, it uses "1/Ploidy" times "Frequency" times "(1-Frequency)". 
#' If FALSE, it uses the sum of the sampling variance of each marker. 
#' Default = FALSE. 
#' @param rmv.mono if monomorphic markers should be removed. Default=FALSE.
#' @param thresh.htzy threshold heterozigosity, remove SNPs below this 
#' threshold. Default=0.
#' @param ASV if TRUE, transform matrix into average semivariance (ASV) 
#' equivalent (K = K / (trace(K) / (nrow(K)-1))). Details formula 2 of 
#' Fieldmann et al. (2022). Default = FALSE.
#' @return Matrix with the marker-bases relationships between the individuals
#'
#' @useDynLib AGHmatrix, .registration = TRUE
#' @importFrom Rcpp evalCpp
#' @importFrom matrixStats colVars
#' @examples
#' \dontrun{
#' ## Diploid Example
#' data(snp.pine)
#' str(snp.pine)
#' Gmatrix.Yang <- Gmatrix(snp.pine, method="Yang", missingValue=-9, maf=0.05)
#' Gmatrix.VanRaden <- Gmatrix(snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
#' Gmatrix.Su <- Gmatrix(snp.pine, method="Su", missingValue=-9, maf=0.05)
#' Gmatrix.Vitezica <- Gmatrix(snp.pine, method="Vitezica", missingValue=-9, maf=0.05)
#' 
#' ## Autetraploid example
#' data(snp.sol)
#' Gmatrix.VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)
#' Gmatrix.Endelman <- Gmatrix(snp.sol, method="Endelman", ploidy=4) 
#' Gmatrix.Slater <- Gmatrix(snp.sol, method="Slater", ploidy=4)
#' Gmatrix.Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE) 
#' Gmatrix.weighted <- Gmatrix(snp.sol, method="VanRaden", weights = runif(3895,0.001,0.1), ploidy=4)
#' }
#' 
#' @author Rodrigo R. Amadeu \email{rramadeu@gmail.com}
#' @author Thiago de Paula Oliveira \email{toliveira@abacusbio.com}
#' @author Marcio Resende Jr.
#' @author Letícia A. C. Lara
#' @author Ivone Oliveira
#' @author Felipe V. Ferrao
#' @references \emph{Covarrubias-Pazaran, G. 2016. Genome assisted prediction 
#' of quantitative traits using the R package sommer. PLoS ONE 11(6):1-15.}
#' @references \emph{Endelman, JB, et al., 2018. Genetic variance partitioning 
#' and genome-wide prediction with allele dosage information in autotetraploid 
#' potato. Genetics, 209(1) pp. 77-87.}
#' @references \emph{Feldmann MJ, et al. 2022. Average semivariance directly 
#' yields accurate estimates of the genomic variance in complex trait analyses. 
#' G3 (Bethesda), 12(6).}
#' @references \emph{Liu, A, et al. 2020. Weighted single-step genomic best 
#' linear unbiased prediction integrating variants selected from sequencing 
#' data by association and bioinformatics analyses. Genet Sel Evol 52, 48.}
#' @references \emph{Slater, AT, et al. 2016. Improving genetic gain with 
#' genomic selection in autotetraploid potato. The Plant Genome 9(3), pp.1-15.}
#' @references \emph{Su, G, et al. 2012. Estimating additive and non-additive 
#' genetic variances and predicting genetic merits using genome-wide dense 
#' single nucleotide polymorphism markers. PloS one, 7(9), p.e45293.}
#' @references \emph{VanRaden, PM, 2008. Efficient methods to compute genomic 
#' predictions. Journal of dairy science, 91(11), pp.4414-4423.}
#' @references \emph{Vitezica, ZG, et al. 2013. On the additive and dominant 
#' variance and covariance of individuals within the genomic selection scope. 
#' Genetics, 195(4), pp.1223-1230.}
#' @references \emph{Yang, J, et al. 2010. Common SNPs explain a large 
#' proportion of the heritability for human height. Nature genetics, 42(7), 
#' pp.565-569.}
#'
#' @export
Gmatrix <- function(SNPmatrix = NULL, 
                    method = "VanRaden", 
                    missingValue = -9, 
                    maf = 0, 
                    thresh.missing = 0.50,
                    verify.posdef = FALSE, 
                    ploidy = 2,
                    pseudo.diploid = FALSE, 
                    integer = TRUE,
                    ratio = FALSE, 
                    impute.method = "mean", 
                    rmv.mono = FALSE, 
                    thresh.htzy = 0,
                    ratio.check = TRUE,
                    weights = NULL, 
                    ploidy.correction = FALSE, 
                    ASV = FALSE) {
  Time <- proc.time()
  #-----------------------------------------------------------------------------
  # ==== Initial preprocessing ====
  #-----------------------------------------------------------------------------
  check_matrix((SNPmatrix))
  SNPmatrix <- as.matrix(data.matrix(SNPmatrix))
  ids <- rownames(SNPmatrix)
  
  if (ploidy %% 2 != 0 || ploidy < 2 || ploidy > 20) {
    stop("ploidy must be an even number between 2 and 20")
  }
  
  markers <- colnames(SNPmatrix)
  
  if (!is.null(weights) && length(weights) != ncol(SNPmatrix)) {
    stop("weights length (", length(weights), 
         ") does not match number of markers (", ncol(SNPmatrix), ")")
  }
  
  if (ratio) method <- "VanRaden"
  
  if (!is.na(missingValue)) {
    SNPmatrix[SNPmatrix == missingValue] <- NA
  }
  
  check_Gmatrix_data(SNPmatrix = SNPmatrix, 
                     method = method, 
                     ploidy = ploidy, 
                     ratio = ratio, 
                     integer = integer)
  
  #-----------------------------------------------------------------------------
  # ==== Missing-data checks / filtering (only when requested) ====
  #-----------------------------------------------------------------------------
  do_mcheck <- (!ratio || (ratio && ratio.check)) &&
    !identical(impute.method, "none")
  nmarkers_pre_mcheck <- ncol(SNPmatrix) 
  
  if (do_mcheck) {
    SNPmatrix <- Mcheck(
      SNPmatrix,
      ploidy          = ploidy,
      thresh.maf      = maf,
      rmv.mono        = rmv.mono,
      thresh.htzy     = thresh.htzy,
      thresh.missing  = thresh.missing,
      impute.method   = impute.method
    )
  }
  
  # Compute MarkersMatrix AFTER Mcheck filtering/imputation
  if (identical(method, "MarkersMatrix")) {
    if (exists("Gmatrix_MarkersMask", mode = "function")) {
      Gmatrix <- Gmatrix_MarkersMask(SNPmatrix)
    } else {
      mask <- !is.na(SNPmatrix)
      storage.mode(mask) <- "double"
      Gmatrix <- tcrossprod(mask, mask)
    }
    if (!is.null(ids)) dimnames(Gmatrix) <- list(ids, ids)
    return(Gmatrix)
  }
  
  
  #-----------------------------------------------------------------------------
  ## ===== Prepare frequencies =====
  #-----------------------------------------------------------------------------
  P <- colMeans(SNPmatrix, na.rm = TRUE)
  if (ploidy == 2) { 
    fr   <- diploid_p0p2_TwoPQ_cpp(SNPmatrix)
    p0   <- as.numeric(fr$p0)
    p2   <- as.numeric(fr$p2)
    TwoPQ <- as.numeric(fr$TwoPQ)
    
    Frequency <- cbind(p0, p2)
    FreqP     <- matrix(p2, nrow = nrow(SNPmatrix), ncol = ncol(SNPmatrix), 
                        byrow = TRUE)
    }
  
  if (ploidy > 2 && pseudo.diploid) {
    SNPmatrix[, P > ploidy / 2] <- ploidy - SNPmatrix[, P > ploidy / 2]
    Frequency <- colMeans(SNPmatrix, na.rm = TRUE) / ploidy
    Frequency <- cbind(1 - Frequency, Frequency)
    FreqP <- matrix(Frequency[, 2], nrow = nrow(SNPmatrix), 
                    ncol = ncol(SNPmatrix), byrow = TRUE)
    SNPmatrix[SNPmatrix %in% c(1:(ploidy - 1))] <- 1
    SNPmatrix[SNPmatrix == ploidy] <- 2
  }
  
  #-----------------------------------------------------------------------------
  ## ===== Main method  =====
  #-----------------------------------------------------------------------------
  if (method == "VanRaden") {
    if (is.null(weights)) {
      if (ploidy == 2 && !ratio) {
        Gmatrix <- Gmatrix_vanraden(SNPmatrix, Frequency[, 2], TwoPQ)
      } else {
        Gmatrix <- Gmatrix_vanraden_poly_unweighted(SNPmatrix, ploidy, ratio, 
                                                    ploidy.correction)
      }
    } else {
      weights <- weights[match(colnames(SNPmatrix), markers)]
      if (ploidy == 2 && !ratio) {
        Gmatrix <- Gmatrix_VanRaden_weighted(SNPmatrix, weights, 
                                             Frequency[, 2], TwoPQ)
      } else {
        Gmatrix <- 
          Gmatrix_vanraden_poly_weighted(SNPmatrix, as.numeric(weights), 
                                         ploidy, ratio, ploidy.correction)
      }
    }
  }
  
  if (method == "Yang") {
    if (ploidy != 2) stop("Yang method is defined for diploids (ploidy = 2).")
    
    # FreqPQ replicated by rows (legacy layout)
    FreqPQ <- matrix(
      rep(2 * Frequency[, 1] * Frequency[, 2], each = nrow(SNPmatrix)),
      ncol = ncol(SNPmatrix)
    )
    
    # Per-marker diagonal expectation terms
    G.all <- (SNPmatrix^2 - (1 + 2 * FreqP) * SNPmatrix + 2 * (FreqP^2)) / FreqPQ
    G.ii  <- as.matrix(colSums(t(G.all), na.rm = TRUE))
    
    # Standardised Z
    Z <- (SNPmatrix - (2 * FreqP)) / sqrt(FreqPQ)
    Z[is.na(Z)] <- 0
    
    G.ii.hat <- 1 + (G.ii) / nmarkers_pre_mcheck
    
    # Final G and diagonal replacement
    Gmatrix <- tcrossprod(Z, Z) / nmarkers_pre_mcheck
    diag(Gmatrix) <- G.ii.hat
  }
  
  if (method == "Su") {
    Gmatrix <- Gmatrix_Su(SNPmatrix)
  }
  
  if (method == "Vitezica") {
    Gmatrix <- Gmatrix_Vitezica(SNPmatrix, FreqP)
  }
  
  if (method == "Slater") {
    P <- colMeans(SNPmatrix, na.rm = TRUE)
    SNPmatrix[, which(P > ploidy/2)] <- ploidy - SNPmatrix[, which(P > ploidy/2)]
    SNPmatrix <- slater_par_cpp(SNPmatrix, ploidy = ploidy)
    
    NumberMarkers <- ncol(SNPmatrix)
    Frequency     <- colMeans(SNPmatrix, na.rm = TRUE)
    FreqP         <- matrix(rep(Frequency, each = nrow(SNPmatrix)),
                            nrow = nrow(SNPmatrix))
    
    drop.alleles  <- which(Frequency == 0)
    if (length(drop.alleles) > 0) {
      Frequency <- Frequency[-drop.alleles]
      SNPmatrix <- SNPmatrix[, -drop.alleles, drop = FALSE]
      FreqP     <- FreqP[, -drop.alleles, drop = FALSE]
    }
    
    Gmatrix <- Gmatrix_Slater(SNPmatrix, Frequency, FreqP, NumberMarkers) 
  }
  
  if (method == "Endelman") {
    if (ploidy != 4) stop("'Endelman' method is just implemented for ploidy=4")
    Gmatrix <- Gmatrix_Endelman(SNPmatrix, ploidy)
  }
  
  if (verify.posdef) {
    e.values <- eigen(Gmatrix, symmetric = TRUE)$values
    indicator <- sum(e.values <= 0)
    if (indicator > 0) cat("\t Matrix is NOT positive definite. It has ",
                           indicator, " eigenvalues <= 0 \n")
  }
  
  if (ASV) {
    Gmatrix <- get_ASV(Gmatrix)
  }
  if (!is.null(ids)) dimnames(Gmatrix) <- list(ids, ids)
  
  cat("Completed! Time =", round(proc.time()[3] - Time[3], 2), " seconds \n")

  return(Gmatrix)
}


## Internal Functions ##
get_ASV = function(x){
  return( x / ( sum(diag(x)) / (nrow(x) - 1)) )
}

# Internal function to check input Gmatrix arguments
check_Gmatrix_data <- function(SNPmatrix,ploidy,method, ratio=FALSE, integer=TRUE){
  if (is.null(SNPmatrix)) {
    stop(deparse("Please define the variable SNPdata"))
  }
  if (all(method != c("Yang", "VanRaden", "Slater", "Su", "Vitezica", "MarkersMatrix","Endelman"))) {
    stop("Method to build Gmatrix has to be either `Yang` or `VanRaden` for marker-based additive relationship matrix, or `Su` or `Vitezica` or `Endelman` for marker-based dominance relationship matrx, or `MarkersMatrix` for matrix with amount of shared-marks by individuals pairs")
  }
  
  #  if( method=="Yang" && ploidy>2)
  #    stop("Change method to 'VanRaden' for ploidies higher than 2 for marker-based additive relationship matrix")
  
  if( method=="Su" && ploidy>2)
    stop("Change method to 'Slater' for ploidies higher than 2 for marker-based non-additive relationship matrix")
  
  if( method=="Vitezica" && ploidy>2)
    stop("Change method to 'Slater' for ploidies higher than 2 for marker-based non-additive relationship matrix")
  
  if(!is.matrix(SNPmatrix)){
    cat("SNPmatrix class is:",class(SNPmatrix),"\n")
    stop("SNPmatrix class must be matrix. Please verify it.")
  }
  
  if(!ratio){
    if( ploidy > 20 | (ploidy %% 2) != 0)
      stop(deparse("Only even ploidy from 2 to 20"))
    
    t <- max(SNPmatrix,na.rm = TRUE)
    if( t > ploidy )
      stop(deparse("Check your data, it has values above ploidy number"))
    
    t <- min(SNPmatrix,na.rm=TRUE)
    if( t < 0 )
      stop(deparse("Check your data, it has values under 0"))
    
    if(integer)
      if(prod(SNPmatrix == round(SNPmatrix),na.rm = TRUE)==0)
        stop(deparse("Check your data, it has not integer values"))
  }
  
  if(ratio){
    t <- max(SNPmatrix,na.rm = TRUE)
    if( t > 1)
      stop(deparse("Check your data, it has values above 1. It is expected a ratio values [0;1]."))
    
    t <- min(SNPmatrix,na.rm=TRUE)
    if( t < 0 )
      stop(deparse("Check your data, it has values under 0. It is expected a ratio values [0;1]."))
  }
}

slater_par_l <- function(X,ploidy){
  prime.index <- c(3,5,7,11,13,17,19,23,29,31,37,
                   41,43,47,53,59,61,67,71,73,79)
  
  NumberMarkers <- ncol(X)
  nindTotal <- nrow(X)
  X <- X+1
  
  ## Breaking intervals to use less RAM
  temp <- seq(1,NumberMarkers,10000)
  temp <- cbind(temp,temp+9999)
  temp[length(temp)] <- NumberMarkers
  prime.index <- prime.index[1:(ploidy+1)]
  
  ## Uses Diagonal (which is Sparse mode, uses less memmory)
  for(i in 1:nrow(temp)){
    X.temp <- X[,c(temp[i,1]:temp[i,2])]
    NumberMarkers <- ncol(X.temp)
    X.temp <- X.temp %*% t(kronecker(diag(NumberMarkers),prime.index))
    X.temp[which(as.vector(X.temp) %in%
                   c(prime.index*c(1:(ploidy+1))))] <- 1
    X.temp[X.temp!=1] <- 0
    if(i==1){
      X_out <- X.temp
    }else{
      X_out <- cbind(X_out,X.temp)
    }   
  }
  gc()
  return(X_out)
}

test_that("VanRaden method computes correctly", {
  M <- matrix(c(0, 1, 2,
                1, 0, 2), nrow = 2, byrow = TRUE)
  colnames(M) <- c("M1", "M2", "M3")
  G <- Gmatrix(M, method = "VanRaden", ploidy = 2)
  expected <- matrix(c(0.6666667, -0.6666667,
                       -0.6666667, 0.6666667), 2)
  expect_equal(G[1:2,1:2], expected, tolerance = 1e-6, ignore_attr = TRUE)
})

test_that("Yang method computes expected G matrix values", {
  M <- matrix(c(0, 1, 2,
                1, 0, 2), nrow = 2, byrow = TRUE)
  colnames(M) <- c("M1", "M2", "M3")
  
  G <- Gmatrix(M, method = "Yang", ploidy = 2)
  
  # M1 and M2 have allele frequency p = 0.25 -> 2p(1-p) = 0.375
  # M3 is monomorphic so ignored in variance
  # Centered Z matrix for polymorphic markers:
  # M1: [-0.5, 0.5], M2: [0.5, -0.5]
  # Product terms:
  # G[1,1] = (0.25/0.375 + 0.25/0.375)/3 = 0.4444 + adjustment = 0.7778
  # G[1,2] = (-0.25/0.375 + -0.25/0.375)/3 = -0.4444
  expected <- matrix(c(0.7777778, -0.4444444,
                       -0.4444444, 0.7777778), nrow = 2, byrow = TRUE)
  
  expect_true(isSymmetric(G))
  expect_equal(round(G[1:2,1:2], 7), round(expected, 7), ignore_attr = TRUE)
})


test_that("Su method computes expected matrix", {
  M <- matrix(c(0, 1, 2,
                2, 1, 0), nrow = 2, byrow = TRUE)
  colnames(M) <- c("M1", "M2", "M3")
  
  G <- Gmatrix(M, method = "Su", ploidy = 2)
  expected <- matrix(c(1, 1,
                       1, 1), 2, 2)
  
  expect_equal(G[1:2,1:2], expected, ignore_attr = TRUE)
})

test_that("Vitezica method computes expected matrix", {
  M <- matrix(c(0, 1, 2,
                2, 1, 0), nrow = 2, byrow = TRUE)
  colnames(M) <- c("M1", "M2", "M3")
  
  G <- Gmatrix(M, method = "Vitezica", ploidy = 2)
  expected <- matrix(c(1, 1,
                       1, 1), 2, 2)
  
  expect_equal(G[1:2,1:2], expected, ignore_attr = TRUE)
})

test_that("Slater method works with non-zero variance markers", {
  # Input
  M <- matrix(c(0, 2, 4,
                4, 2, 0), nrow = 2, byrow = TRUE)
  colnames(M) <- c("M1","M2","M3")
  ploidy <- 4
  
  ## Legacy Slater steps
  
  # Flip dosages where mean > ploidy/2 
  P <- colMeans(M, na.rm = TRUE)
  M_flip <- M
  if (any(P > ploidy/2)) {
    idx <- which(P > ploidy/2)
    M_flip[, idx] <- ploidy - M_flip[, idx]
  }
  
  # presence/absence encode per Slater
  M_sl <- slater_par_l(M_flip, ploidy = ploidy)
  
  # denominator used by legacy code
  NumberMarkers_legacy <- ncol(M_sl)
  
  # Presence/absence frequency
  Frequency <- colMeans(M_sl, na.rm = TRUE)
  
  # drop columns with zero frequency
  drop <- which(Frequency == 0)
  if (length(drop) > 0L) {
    M_use     <- M_sl[, -drop, drop = FALSE]
    Frequency <- Frequency[-drop]
  } else {
    M_use <- M_sl
  }
  
  # FreqP and FreqPQ
  n <- nrow(M_use)
  FreqP  <- matrix(rep(Frequency, each = n), nrow = n)
  FreqPQ <- matrix(rep(Frequency * (1 - Frequency), each = n), nrow = n)
  
  # Compute G
  Z <- (M_use - FreqP) / sqrt(FreqPQ)
  Z[!is.finite(Z)] <- 0  # legacy sets NaN from 0/0 to 0
  
  G_expected <- (Z %*% t(Z)) / NumberMarkers_legacy
  
  # Diagonal
  diag_terms <- (M_use^2 - 2 * FreqP * M_use + FreqP^2) / FreqPQ
  Gii <- 1 + rowSums(diag_terms, na.rm = TRUE) / NumberMarkers_legacy
  diag(G_expected) <- Gii
  
  #      [,1]      [,2]
  # [1,] 1.266667 -0.266667
  # [2,] -0.266667 1.266667
  
  # Actual G from C++
  G <- Gmatrix(M, method = "Slater", ploidy = 4)
  
  expect_equal(round(G[1:2, 1:2], 6), round(G_expected, 6))
  expect_true(isSymmetric(G[1:2, 1:2]))
})

# Parity tests between new Hmatrix() and Hmatrix_legacy()
Hmatrix_legacy <- function(A=NULL,
                    G=NULL,
                    markers=NULL,
                    c=0,
                    method="Martini",
                    tau=1,
                    omega=1,
                    missingValue=-9,
                    maf=0,
                    ploidy=2,
                    roundVar=3,
                    ASV=FALSE
){
  Aorig <- A
  Gorig <- G
  
  Time = proc.time()
  cat("Comparing the matrices... \n")
  An <- rownames(Aorig)
  Gn <- rownames(Gorig)
  missingGmatrix <- which(is.na(match(An,Gn)))
  missingAmatrix <- which(is.na(match(Gn,An)))
  if(length(missingAmatrix)>0){
    Gnhat <- Gn[-missingAmatrix]
  }else{
    Gnhat <- Gn
  }
  
  if(length(missingGmatrix)>0){
    Anhat <- An[-missingGmatrix]    
  }else{
    Anhat <- An
  }
  
  A <- Aorig#[Anhat,Anhat]
  G <- Gorig[Gnhat,Gnhat]
  
  missingGmatrix <- An[missingGmatrix]
  missingAmatrix <- Gn[missingAmatrix]
  
  Time = as.matrix(proc.time()-Time)
  cat("Completed! Time =", Time[3]/60," minutes \n")
  
  cat("Computing the H matrix... \n")
  
  Time = proc.time()
  
  if(method=="Martini"){
    idA <-rownames(A)
    idG <- rownames(G)
    idH <- unique(c(idG,idA))
    idH <- rev(idH)
    A <- A[idH,idH]
    
    index = is.na(match(idH,idG))
    A11 <- A[index,index]
    A12 <- A[index,!index]
    A21 <- A[!index,index]
    A22 <- A[!index,!index]
    G22 <- G[idH[!index],idH[!index]]
    #if(is.singular.matrix(G22))
    #  stop(deparse("Matrix G22 is singular (not invertible)"))
    A22inv = solve(A22) #A is always invertible
    G22inv = try(solve(G22),silent=TRUE)
    if(inherits(G22inv,"try-error")){
      cat(G22inv)
      stop("G22 not inverting with solve(), try a different/modified G matrix")
    }
    H22 = solve((tau*G22inv+(1-omega)*A22inv))
    H11 = A12 %*% A22inv %*% (H22-A22) %*% A22inv %*% A21  
    H12 = A12 %*% A22inv %*% (H22-A22)
    H21 = (H22-A22) %*% A22inv%*%A21
    H22 = (H22-A22)
    H = A+cbind(rbind(H11,H21),rbind(H12,H22))
    
    
    if (ASV) {
      H = get_ASV(H)
    }
    
    Time = as.matrix(proc.time()-Time)
    cat("\n","Completed! Time =", Time[3]/60," minutes \n")
    return(H)
  }
  
  if(method=="Munoz"){
    A <- Aorig[Anhat,Anhat]
    if(is.null(markers))
      stop("Aborting: For Munoz method you need to specify method object")
    markersmatrix <- Gmatrix(markers,method="MarkersMatrix",ploidy=ploidy,missingValue=missingValue,maf=maf)
    
    #Computing the Variance of G by A classes (A rounded by roundVar)
    classes <- as.numeric(levels(as.factor(A)))
    classes <- unique(round(classes,roundVar))
    n <- length(classes)
    varA <- meanG <- matrix(NA,nrow=nrow(A),ncol=nrow(A))
    varAclasses <- c()
    for(i in 1:n){
      varA[round(A,roundVar)==classes[i]] <- varAclasses[i] <- var(G[round(A,roundVar)==classes[i]])
      meanG[round(A,roundVar)==classes[i]]  <- mean(G[round(A,roundVar)==classes[i]])
    }
    
    varAclasses[varAclasses==0] = NA
    varAclasses[is.infinite(varAclasses)] <- NA
    tmp <- which(is.na(varAclasses))
    for(i in 1:length(tmp)){
      varAclasses[tmp[i]]<-zoo::na.approx(varAclasses)[tmp[i]]
      varA[round(A,roundVar)==classes[tmp[i]]] <- varAclasses[tmp[i]]
    }
    
    #Computaing beta and H
    beta <- 1 - (c+(1/(markersmatrix[Gnhat,Gnhat]))/varA)
    H <- beta*(G-A)+A ######
    Aorig[Anhat,Anhat] = H
    
    if (ASV) {
      Aorig = get_ASV_l(Aorig)
    }
    
    Time = as.matrix(proc.time()-Time)
    cat("\n","Completed! Time =", Time[3]/60," minutes \n")
    cat("\n","Returning H = A matrix corrected by G... \n")
    
    return(Aorig)
  }
}

get_ASV_l <- function(x){ x / ( sum(diag(x)) / (nrow(x) - 1)) }

make_spd <- function(n, seed = 1L){
  set.seed(seed)
  M <- matrix(rnorm(n*n), n)
  S <- crossprod(M)               # SPD
  diag(S) <- diag(S) + n*1e-3
  S
}

# Build A and G
make_AG_ids <- function(){
  # A over 8 IDs, G over subset of 4
  idA <- paste0("I", 1:8)
  idG <- c("I3","I6","I2","I7")
  A <- diag(8)
  rownames(A) <- colnames(A) <- idA
  G <- make_spd(length(idG), seed = 42)
  rownames(G) <- colnames(G) <- idG
  list(A=A, G=G, idA=idA, idG=idG)
}

# Simple markers matrix for Muñoz
make_markers <- function(ids, m = 6L, seed = 99L){
  set.seed(seed)
  # diploid-style dosages 0/1/2, no missing
  X <- matrix(sample(0:2, length(ids)*m, replace = TRUE), nrow = length(ids), 
              ncol = m)
  rownames(X) <- ids; colnames(X) <- paste0("M", seq_len(m))
  storage.mode(X) <- "double"
  X
}

compare_H_martini <- function(A, G, tau = 1, omega = 1, c = 0, 
                              ASV = FALSE, tol = 1e-10){
  # new
  H_new <- Hmatrix(A=A, G=G, method = "Martini", tau = tau, omega = omega, 
                   c = c, ASV = ASV)
  # legacy
  H_old <- Hmatrix_legacy(A=A, G=G, method = "Martini", tau = tau, 
                          omega = omega, c = c, ASV = ASV)
  # numeric parity (ignore non-structural attrs)
  expect_equal(strip_attrs(H_new), strip_attrs(H_old), tolerance = tol, 
               scale = 1)
  # basic properties
  expect_true(isSymmetric(H_new))
  expect_identical(rownames(H_new), rownames(H_old))
  expect_identical(colnames(H_new), colnames(H_old))
  invisible(list(new = H_new, old = H_old))
}

compare_H_munoz <- function(A, G, markers, c = 0, ploidy = 2, roundVar = 0, 
                            ASV = FALSE, tol = 1e-10){
  H_new <- Hmatrix(A=A, G=G, markers=markers, method = "Munoz", c = c, 
                   ploidy = ploidy, roundVar = roundVar, ASV = ASV)
  H_old <- Hmatrix_legacy(A=A, G=G, markers=markers, method = "Munoz", c = c, 
                          ploidy = ploidy, roundVar = roundVar, ASV = ASV)
  expect_equal(strip_attrs(H_new), strip_attrs(H_old), tolerance = tol, 
               scale = 1)
  expect_true(isSymmetric(H_new))
  expect_identical(rownames(H_new), rownames(H_old))
  expect_identical(colnames(H_new), colnames(H_old))
  invisible(list(new = H_new, old = H_old))
}

strip_attrs <- function(x){
  at <- attributes(x); if (is.null(at)) return(x)
  keep <- c("dim","dimnames","class")
  attributes(x) <- at[names(at) %in% keep]
  x
}

# Tests

test_that("H (Martini) parity on simple SPD inputs, default tau=omega=1",{
  objs <- make_AG_ids(); with(objs, {
    compare_H_martini(A, G, tau = 1, omega = 1, c = 0, ASV = FALSE)
  })
})

test_that("H (Martini) parity holds for different tau/omega and ASV on",{
  objs <- make_AG_ids(); with(objs, {
    compare_H_martini(A, G, tau = 0.7, omega = 0.2, c = 0, ASV = FALSE)
    compare_H_martini(A, G, tau = 1.5, omega = 0.0, c = 0, ASV = TRUE)
  })
})

test_that("H (Munoz) parity with clean classes (roundVar=0), no ASV",{
  objs <- make_AG_ids(); with(objs, {
    markers <- make_markers(idG, m = 8)
    compare_H_munoz(A, G, markers = markers, c = 0, ploidy = 2,
                    roundVar = 0, ASV = FALSE)
  })
})

test_that("H (Munoz) parity with ASV on",{
  objs <- make_AG_ids(); with(objs, {
    markers <- make_markers(idG, m = 5)
    compare_H_munoz(A, G, markers = markers, c = 0.2, ploidy = 2, 
                    roundVar = 0, ASV = TRUE)
  })
})

test_that("Informative attributes",{
  objs <- make_AG_ids(); with(objs, {
    H <- Hmatrix(A=A, G=G, method = "Martini", tau = 1.2, omega = 0.3, 
                 c = 0.0, ASV = TRUE)
    # Attributes should not be NA/NULL
    at <- attributes(H)
    expect_true(is.list(at))
    # Commonly exposed attrs 
    if (!is.null(attr(H, "method"))) expect_identical(attr(H, "method"), "Martini")
    if (!is.null(attr(H, "tau")))     expect_equal(attr(H, "tau"), 1.2)
    if (!is.null(attr(H, "omega")))   expect_equal(attr(H, "omega"), 0.3)
    if (!is.null(attr(H, "c")))       expect_equal(attr(H, "c"), 0.0)
    if (!is.null(attr(H, "ASV")))     expect_identical(attr(H, "ASV"), TRUE)
  })
})

test_that("Martini path reorders/expands A to idH = unique(c(idG,idA))",{
  objs <- make_AG_ids(); with(objs, {
    # permute A order and add an extra id in A not in G
    permA <- sample(idA)
    A2 <- A[permA, permA]
    # new and legacy should agree
    res <- compare_H_martini(A2, G, tau = 0.9, omega = 0.1)
    expect_identical(rownames(res$new), rownames(res$old))
  })
})

# Parity tests between legacy R maths and the new C++ 

# Legacy Helpers
Mcheck_l = function(SNPmatrix = NULL,
                  ploidy=2,
                  missingValue = -9,
                  thresh.maf = 0.05,
                  thresh.missing = 0.9,
                  thresh.htzy = 0, 
                  impute.method = "mean",
                  rmv.mono=TRUE){
  
  # SNP missing data
  ncol.init <- ncol(SNPmatrix)
  
  if (!is.na(missingValue)) {
    m <- match(SNPmatrix, missingValue, 0)
    SNPmatrix[m > 0] <- NA
  }
  
  missing <- apply(SNPmatrix, 2, function(x) sum(is.na(x))/nrow(SNPmatrix))
  missing.low = missing <= thresh.missing
  cat("\nMissing data check: \n")
  if(any(missing.low)){
    cat("\tTotal SNPs:", ncol(SNPmatrix),"\n")
    cat("\t",ncol(SNPmatrix) - sum(missing.low), "SNPs dropped due to missing data threshold of", thresh.missing,"\n")
    cat("\tTotal of:",sum(missing.low), " SNPs \n")
    idx.rm <- which(missing.low)
    SNPmatrix <- SNPmatrix[, idx.rm, drop=FALSE]
  } else{
    cat("\tNo SNPs with missing data, missing threshold of = ", thresh.missing,"\n")
  }
  
  # Minor alele frequency
  MAF <- apply(SNPmatrix, 2, function(x) {
    AF <- mean(x, na.rm = T)/ploidy
    MAF <- ifelse(AF > 0.5, 1 - AF, AF) # Minor allele freq can be ref allele or not
  })
  snps.low <- MAF < thresh.maf
  cat("\nMAF check: \n")
  if(any(snps.low)){
    cat("\t",sum(snps.low), "SNPs dropped with MAF below", thresh.maf,"\n")
    cat("\tTotal:",ncol(SNPmatrix) - sum(snps.low), " SNPs \n")
    idx.rm <- which(snps.low)
    SNPmatrix <- SNPmatrix[, -idx.rm, drop=FALSE]
  } else{
    cat("\tNo SNPs with MAF below", thresh.maf,"\n")
  }
  
  # monomorphic SNPs
  if(rmv.mono){
    mono = (apply(SNPmatrix, 2, var, na.rm=TRUE)==0)
    mono[is.na(mono)] = TRUE
    cat("\nMonomorphic check: \n")
    if(any(mono)){
      cat("\t",sum(mono), "monomorphic SNPs \n")
      cat("\tTotal:",ncol(SNPmatrix) - sum(mono), "SNPs \n")
      idx.rm <- which(mono)
      SNPmatrix <- SNPmatrix[, -idx.rm, drop=FALSE]
    } else{
      cat("\tNo monomorphic SNPs \n")
    }
  }
  
  
  # Imputation
  if(impute.method=="global.mean"){
    ix <- which(is.na(SNPmatrix))
    if (length(ix) > 0) {
      SNPmatrix[ix] <- mean(SNPmatrix,na.rm = TRUE)
    }
  }
  
  if(impute.method=="global.mode"){
    ix <- which(is.na(SNPmatrix))
    if (length(ix) > 0) {
      SNPmatrix[ix] <- as.integer(names(which.max(table(SNPmatrix))))
    }
  }
  
  if(impute.method=="mean"){
    imputvalue = apply(SNPmatrix,2,mean,na.rm=TRUE)
    ix = which(is.na(SNPmatrix),arr.ind=TRUE)
    SNPmatrix[ix] = imputvalue[ix[,2]]
  }
  
  if(impute.method=="mode"){
    imputvalue = apply(SNPmatrix, 2, function(x) as.integer(names(which.max(table(x)))))
    ix = which(is.na(SNPmatrix),arr.ind=TRUE)
    SNPmatrix[ix] = imputvalue[ix[,2]]
  }    
  
  
  # Heterozigosity
  htrz <- apply(SNPmatrix, 2, function(x) sum( x!= 0 & x != ploidy,na.rm=T)/nrow(SNPmatrix))
  htrz.low = htrz < thresh.htzy 
  cat("\nHeterozigosity data check: \n")
  if(any(htrz.low)){
    cat("\tTotal SNPs:", ncol(SNPmatrix),"\n")
    cat("\t",ncol(SNPmatrix) - sum(htrz.low), "SNPs dropped due to heterozygosity threshold of", thresh.htzy,"\n")
    cat("\tTotal of:",sum(htrz.low), " SNPs \n")
    idx.rm <- which(htrz.low)
    SNPmatrix <- SNPmatrix[, idx.rm, drop=FALSE]
  } else{
    cat("\tNo SNPs with heterozygosity, missing threshold of = ", thresh.htzy,"\n")
  }
  
  # Total of SNPs
  cat("\nSummary check: \n")
  cat("\tInitial: ", ncol.init, "SNPs \n")
  cat("\tFinal: ", ncol(SNPmatrix), " SNPs (", ncol.init - ncol(SNPmatrix), " SNPs removed) \n \n")
  return(SNPmatrix)
}

get_ASV_l <- function(x){ x / ( sum(diag(x)) / (nrow(x) - 1)) }

slater_par_l <- function(X, ploidy){
  prime.index <- c(3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79)
  NumberMarkers <- ncol(X)
  nindTotal <- nrow(X)
  X <- X + 1
  temp <- seq(1, NumberMarkers, 10000)
  temp <- cbind(temp, temp + 9999)
  temp[length(temp)] <- NumberMarkers
  prime.index <- prime.index[1:(ploidy+1)]
  for(i in 1:nrow(temp)){
    X.temp <- X[, c(temp[i,1]:temp[i,2]), drop=FALSE]
    NumberMarkers <- ncol(X.temp)
    X.temp <- X.temp %*% t(kronecker(diag(NumberMarkers), prime.index))
    X.temp[which(as.vector(X.temp) %in% c(prime.index * c(1:(ploidy+1))))] <- 1
    X.temp[X.temp != 1] <- 0
    if(i==1){ X_out <- X.temp } else { X_out <- cbind(X_out, X.temp) }
  }
  gc()
  X_out
}

check_Gmatrix_data <- function(SNPmatrix, ploidy, method, ratio=FALSE, integer=TRUE){
  if (is.null(SNPmatrix)) stop("Please define the variable SNPdata")
  if (all(method != c("Yang", "VanRaden", "Slater", "Su", "Vitezica", "MarkersMatrix","Endelman"))) {
    stop("Unsupported method")
  }
  if(!is.matrix(SNPmatrix)) stop("SNPmatrix class must be matrix. Please verify it.")
  if(!ratio){
    if( ploidy > 20 | (ploidy %% 2) != 0) stop("Only even ploidy from 2 to 20")
    t <- max(SNPmatrix, na.rm = TRUE); if(t > ploidy) stop("Values above ploidy")
    t <- min(SNPmatrix, na.rm = TRUE); if(t < 0) stop("Values under 0")
    if(integer) if(prod(SNPmatrix == round(SNPmatrix), na.rm = TRUE)==0) stop("Non-integer values found")
  } else {
    t <- max(SNPmatrix, na.rm = TRUE); if(t > 1) stop("Ratio values above 1")
    t <- min(SNPmatrix, na.rm = TRUE); if(t < 0) stop("Ratio values under 0")
  }
}

Gmatrix_legacy <- function (SNPmatrix = NULL, method = "VanRaden", 
                            missingValue = -9, maf = 0, thresh.missing = .50,
                            verify.posdef = FALSE, ploidy=2,
                            pseudo.diploid = FALSE, integer=TRUE,
                            ratio = FALSE, impute.method = "mean", rmv.mono=FALSE, thresh.htzy=0,
                            ratio.check = TRUE, weights = NULL, ploidy.correction = FALSE, ASV=FALSE){
  Time <- proc.time(); markers <- colnames(SNPmatrix)
  if(!is.null(weights)) if(length(weights)!=ncol(SNPmatrix)) stop("weight should match #markers")
  if(ratio) method <- "VanRaden"
  if (!is.na(missingValue)) { m <- match(SNPmatrix, missingValue, 0); SNPmatrix[m > 0] <- NA }
  check_Gmatrix_data(SNPmatrix=SNPmatrix, method=method, ploidy=ploidy, ratio=ratio, integer=integer)
  NumberMarkers <- ncol(SNPmatrix)
  if(ratio==FALSE){
    SNPmatrix <- Mcheck_l(SNPmatrix, ploidy=ploidy, thresh.maf=maf, rmv.mono=rmv.mono,
                        thresh.htzy=thresh.htzy, thresh.missing=thresh.missing, impute.method=impute.method)
  }
  if(ratio && ratio.check){
    SNPmatrix <- Mcheck_l(SNPmatrix, ploidy=ploidy, thresh.maf=maf, rmv.mono=rmv.mono,
                        thresh.htzy=thresh.htzy, thresh.missing=thresh.missing, impute.method=impute.method)
  }
  if(method=="Slater"){
    P <- colSums(SNPmatrix,na.rm=TRUE)/nrow(SNPmatrix)
    SNPmatrix[, which(P>ploidy/2)] <- ploidy - SNPmatrix[, which(P>(ploidy/2))]
    SNPmatrix <- slater_par_l(SNPmatrix, ploidy=ploidy)
    NumberMarkers <- ncol(SNPmatrix)
    Frequency <- colSums(SNPmatrix, na.rm=TRUE)/nrow(SNPmatrix)
    FreqP <- matrix(rep(Frequency, each=nrow(SNPmatrix)), ncol=ncol(SNPmatrix))
  }
  if(ploidy==2){
    alelleFreq <- function(x, y) {(2 * length(which(x == y)) + length(which(x == 1)))/(2 * length(which(!is.na(x))))}
    Frequency <- cbind(apply(SNPmatrix, 2, function(x) alelleFreq(x,0)), apply(SNPmatrix, 2, function(x) alelleFreq(x,2)))
    FreqP <- matrix(rep(Frequency[, 2], each = nrow(SNPmatrix)), ncol = ncol(SNPmatrix))
  }
  if(ploidy>2 && pseudo.diploid){
    P <- colSums(SNPmatrix,na.rm = TRUE)/nrow(SNPmatrix)
    SNPmatrix[,which(P>ploidy/2)] <- ploidy-SNPmatrix[,which(P>(ploidy/2))]
    Frequency <- colSums(SNPmatrix,na.rm=TRUE)/(ploidy*nrow(SNPmatrix))
    Frequency <- cbind(1-Frequency,Frequency)
    FreqP <- matrix(rep(Frequency[, 2], each = nrow(SNPmatrix)), ncol = ncol(SNPmatrix))
    SNPmatrix[SNPmatrix %in% c(1:(ploidy-1))] <- 1
    SNPmatrix[SNPmatrix==ploidy] <- 2
  }
  if (method == "MarkersMatrix"){
    Gmatrix <- !is.na(SNPmatrix)
    Gmatrix <- tcrossprod(Gmatrix, Gmatrix)
    return(Gmatrix)
  }
  if (method == "VanRaden"){
    if(is.null(weights)){
      if(ploidy==2 & ratio==FALSE){
        TwoPQ <- 2 * t(Frequency[, 1]) %*% Frequency[, 2]
        SNPmatrix <- SNPmatrix - 2 * FreqP
        SNPmatrix[is.na(SNPmatrix)] <- 0
        Gmatrix <- (tcrossprod(SNPmatrix, SNPmatrix))/as.numeric(TwoPQ)
      } else {
        if(ploidy.correction){
          if(ratio==FALSE){
            Frequency <- apply(X=SNPmatrix, FUN=mean, MARGIN=2, na.rm=TRUE)/ploidy
            K <- sum(ploidy * Frequency * (1-Frequency))
          } else {
            Frequency <- apply(X=SNPmatrix, FUN=mean, MARGIN=2, na.rm=TRUE)
            K <- sum(1/ploidy * Frequency * (1-Frequency))
          }
        }
        SNPmatrix <- scale(SNPmatrix, center=TRUE, scale=FALSE)
        if(!ploidy.correction){ K <- sum(apply(X=SNPmatrix, FUN=var, MARGIN=2, na.rm=TRUE)) }
        SNPmatrix[which(is.na(SNPmatrix))] <- 0
        Gmatrix <- tcrossprod(SNPmatrix)/K
      }
    } else {
      weights <- weights[match(colnames(SNPmatrix), markers)]
      if(ploidy==2 & ratio==FALSE){
        TwoPQ <- 2 * t(Frequency[, 1]) %*% Frequency[, 2]
        SNPmatrix <- SNPmatrix - 2 * FreqP
        SNPmatrix[is.na(SNPmatrix)] <- 0
        Gmatrix <- tcrossprod(tcrossprod(SNPmatrix, diag(weights)), SNPmatrix)/as.numeric(TwoPQ)
      } else {
        if(ploidy.correction){
          if(ratio==FALSE){
            Frequency <- apply(X=SNPmatrix, FUN=mean, MARGIN=2, na.rm=TRUE)/ploidy
            K <- sum(ploidy * Frequency * (1-Frequency))
          } else {
            Frequency <- apply(X=SNPmatrix, FUN=mean, MARGIN=2, na.rm=TRUE)
            K <- sum(Frequency * (1-Frequency))
          }
        }
        SNPmatrix <- scale(SNPmatrix, center=TRUE, scale=FALSE)
        if(!ploidy.correction){ K <- sum(apply(X=SNPmatrix, FUN=var, MARGIN=2, na.rm=TRUE)) }
        SNPmatrix[which(is.na(SNPmatrix))] <- 0
        Gmatrix <- tcrossprod(tcrossprod(SNPmatrix, diag(weights)), SNPmatrix)/K
      }
    }
  }
  if (method == "Yang"){
    FreqPQ <- matrix(rep(2 * Frequency[, 1] * Frequency[, 2], each = nrow(SNPmatrix)), ncol = ncol(SNPmatrix))
    G.all <- (SNPmatrix^2 - (1 + 2 * FreqP) * SNPmatrix + 2 * (FreqP^2))/FreqPQ
    G.ii <- as.matrix(colSums(t(G.all), na.rm = TRUE))
    SNPmatrix <- (SNPmatrix - (2 * FreqP))/sqrt(FreqPQ)
    G.ii.hat <- 1 + (G.ii)/NumberMarkers
    SNPmatrix[is.na(SNPmatrix)] <- 0
    Gmatrix <- (tcrossprod(SNPmatrix, SNPmatrix))/NumberMarkers
    diag(Gmatrix) <- G.ii.hat
  }
  if (method == "Su"){
    TwoPQ <- 2*(FreqP)*(1-FreqP)
    SNPmatrix[SNPmatrix==2 | SNPmatrix==0] <- 0
    SNPmatrix <- SNPmatrix - TwoPQ
    SNPmatrix[is.na(SNPmatrix)] <- 0
    Gmatrix <- tcrossprod(SNPmatrix,SNPmatrix)/sum(TwoPQ[1,]*(1-TwoPQ[1,]))
  }
  if (method == "Vitezica"){
    TwoPQ <- 2*(FreqP[1,])*(1-FreqP[1,])
    SNPmatrix[is.na(SNPmatrix)] <- 0
    SNPmatrix <- (SNPmatrix==0)*-2*(FreqP^2) + (SNPmatrix==1)*2*(FreqP)*(1-FreqP) + (SNPmatrix==2)*-2*((1-FreqP)^2)
    Gmatrix <- tcrossprod(SNPmatrix,SNPmatrix)/sum(TwoPQ^2)
  }
  if (method == "Slater"){
    drop.alleles <- which(Frequency==0)
    if(length(drop.alleles)>0){
      Frequency <- Frequency[-drop.alleles]
      SNPmatrix <- SNPmatrix[,-drop.alleles, drop=FALSE]
      FreqP <- FreqP[,-drop.alleles, drop=FALSE]
    }
    FreqPQ <- matrix(rep(Frequency * (1-Frequency), each = nrow(SNPmatrix)), ncol = ncol(SNPmatrix))
    SNPmatrix[which(is.na(SNPmatrix))] <- 0
    G.ii <- (SNPmatrix^2 - (2 * FreqP) * SNPmatrix + FreqP^2)/FreqPQ
    G.ii <- as.matrix(colSums(t(G.ii), na.rm = TRUE))
    G.ii <- 1 + (G.ii)/NumberMarkers
    SNPmatrix <- (SNPmatrix - (FreqP))/sqrt(FreqPQ)
    SNPmatrix[is.na(SNPmatrix)] <- 0
    Gmatrix <- (tcrossprod(SNPmatrix, SNPmatrix))/NumberMarkers
    diag(Gmatrix) <- G.ii
  }
  if( method == "Endelman" ){
    if( ploidy != 4 ) stop("'Endelman' requires ploidy=4")
    Frequency <- colSums(SNPmatrix)/(nrow(SNPmatrix)*ploidy)
    Frequency <- cbind(Frequency,1-Frequency)
    SixPQ <- 6 * t((Frequency[, 1]^2)) %*% (Frequency[, 2]^2)
    SNPmatrix <- 6 * t((Frequency[, 1]^2)%*%t(rep(1,nrow(SNPmatrix)))) - 
      3*t((Frequency[, 1])%*%t(rep(1,nrow(SNPmatrix))))*SNPmatrix + 0.5 * SNPmatrix*(SNPmatrix-1)
    Gmatrix <- (tcrossprod(SNPmatrix, SNPmatrix))/as.numeric(SixPQ)
  }
  if(ASV){ Gmatrix <- get_ASV_l(Gmatrix) }
  Gmatrix
}

get_target_fun <- function(){ getOption("AGHmatrix_target_fun", Gmatrix) }

# ---------- Deterministic small data generators ----------
make_diploid <- function(){
  X <- cbind(
    c(0,1,2,0,1,2),     # M1
    c(0,0,1,1,2,2),     # M2
    c(2,1,0,2,1,0),     # M3
    c(1,1,1,1,1,1),     # M4 (monomorphic)
    c(0,-9,2,1,0,2),    # M5 (with missingValue=-9)
    c(2,2,1,1,0,0)      # M6
  )
  colnames(X) <- paste0("M", 1:ncol(X)); rownames(X) <- paste0("I", 1:nrow(X))
  storage.mode(X) <- "double"
  X
}

make_tetraploid <- function(){
  X <- cbind(
    c(0,1,2,3,4,2),   # T1
    c(4,3,2,1,0,2),   # T2
    c(0,0,2,2,4,2),   # T3 (even dosages)
    c(1,2,3,1,2,3),   # T4
    c(0,4,0,4,0,4),   # T5 (polarised)
    c(2,2,2,2,2,2)    # T6 (monomorphic)
  )
  colnames(X) <- paste0("T", 1:ncol(X)); rownames(X) <- paste0("J", 1:nrow(X))
  storage.mode(X) <- "double"
  X
}

make_ratio <- function(){
  X <- cbind(
    c(0.0, 0.2, 0.4, 0.6, 0.8),
    c(0.1, 0.1, 0.1, 0.9, 0.9),
    c(0.5, 0.5, 0.5, 0.5, 0.5),
    c(0.0, 0.3, 0.6, 0.3, 0.0)
  )
  colnames(X) <- paste0("R", 1:ncol(X)); rownames(X) <- paste0("K", 1:nrow(X))
  storage.mode(X) <- "double"
  X
}

strip_attrs <- function(x){
  at <- attributes(x)
  if (is.null(at)) return(x)
  keep <- c("dim","dimnames","class")
  attributes(x) <- at[names(at) %in% keep]
  x
}
compare_G <- function(X, args, method, tol=1e-8){
  legacy <- do.call(Gmatrix_legacy, c(list(SNPmatrix=X, method=method), args))
  target_fun <- get_target_fun()
  target <- do.call(target_fun, c(list(SNPmatrix=X, method=method), args))
  # drop non-structural attributes so we compare pure numerics
  legacy <- strip_attrs(legacy)
  target <- strip_attrs(target)
  expect_equal(target, legacy, tolerance = tol, scale = 1)
  expect_true(isSymmetric(target))
  expect_identical(dim(target), c(nrow(X), nrow(X)))
  invisible(list(legacy=legacy, target=target))
}

# Tests per method

test_that("VanRaden (diploid) parity",{
  X <- make_diploid()
  args <- list(missingValue=-9, ploidy=2, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1, verify.posdef=FALSE)
  compare_G(X, args, method="VanRaden")
})

test_that("Yang (diploid) parity",{
  X <- make_diploid()
  args <- list(missingValue=-9, ploidy=2, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing = 1,
               impute.method="mean")
  compare_G(X, args, method="Yang")
})

test_that("Su (diploid dominance) parity",{
  X <- make_diploid()
  args <- list(missingValue=-9, ploidy=2, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1)
  compare_G(X, args, method="Su")
})

test_that("Vitezica (diploid dominance) parity",{
  X <- make_diploid()
  args <- list(missingValue=-9, ploidy=2, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1)
  compare_G(X, args, method="Vitezica")
})

test_that("MarkersMatrix parity",{
  X <- make_diploid()
  args <- list(missingValue=-9, ploidy=2)
  compare_G(X, args, method="MarkersMatrix", tol=0) # exact integer equality expected
})

test_that("VanRaden (tetraploid) parity, ploidy.correction off/on",{
  X <- make_tetraploid()
  base_args <- list(missingValue=-9, ploidy=4, ratio=FALSE, integer=TRUE,
                    rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1)
  compare_G(X, base_args, method="VanRaden")
  compare_G(X, c(base_args, list(ploidy.correction=TRUE)), method="VanRaden")
})

test_that("Slater (autotetraploid full model) parity",{
  X <- make_tetraploid()
  args <- list(missingValue=-9, ploidy=4, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1)
  compare_G(X, args, method="Slater")
})

test_that("Endelman (autotetraploid digentic dominance) parity",{
  X <- make_tetraploid()
  args <- list(missingValue=-9, ploidy=4, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1)
  compare_G(X, args, method="Endelman")
})

test_that("Pseudo-diploid parametrisation parity (ploidy=4)",{
  X <- make_tetraploid()
  args <- list(missingValue=-9, ploidy=4, pseudo.diploid=TRUE, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1)
  compare_G(X, args, method="VanRaden")
})

test_that("Weights alignment remains correct after marker filtering (no filtering here)",{
  X <- make_diploid()
  w <- c(M1=0.5, M2=1.2, M3=0.7, M4=1.0, M5=0.8, M6=1.5)
  # permute column order to ensure matching is by name, not position
  X2 <- X[, c("M3","M1","M6","M2","M5","M4")]
  args <- list(missingValue=-9, ploidy=2, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1, weights=w)
  compare_G(X2, args, method="VanRaden")
})

test_that("ASV rescaling parity wraps any method",{
  X <- make_diploid()
  args <- list(missingValue=-9, ploidy=2, ratio=FALSE, integer=TRUE,
               rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1, ASV=TRUE)
  compare_G(X, args, method="Yang")
  compare_G(X, args, method="VanRaden")
})

test_that("Ratio pathway forces VanRaden and matches explicit VanRaden+ratio",{
  X <- make_ratio()
  args1 <- list(missingValue=-9, ploidy=2, ratio=TRUE, ratio.check=FALSE, integer=FALSE)
  # method argument ignored when ratio=TRUE; we still pass Yang to check override
  out1 <- compare_G(X, args1, method="Yang")
  args2 <- list(missingValue=-9, ploidy=2, ratio=TRUE, ratio.check=FALSE, integer=FALSE)
  out2 <- compare_G(X, args2, method="VanRaden")
  expect_equal(out1$target, out2$target, tolerance=0)
  expect_equal(out1$legacy, out2$legacy, tolerance=0)
})

prop_methods <- c("VanRaden","Yang","Su","Vitezica")

test_that("All diploid methods return symmetric matrices with correct size",{
  X <- make_diploid(); args <- list(missingValue=-9, ploidy=2, ratio=FALSE, integer=TRUE,
                                    rmv.mono=FALSE, thresh.htzy=0, thresh.missing=1)
  for(m in prop_methods){
    target <- do.call(get_target_fun(), c(list(SNPmatrix=X, method=m), args))
    expect_true(isSymmetric(target)); expect_identical(dim(target), c(nrow(X), nrow(X)))
  }
})

# Helper used across tests
delta_summary <- function(A, B) {
  D <- round(A, 4) - round(B, 4)
  c(max_abs = max(abs(D)), nnz = sum(D != 0))
}

test_that("new Gmatrix matches VanRaden legacy on snp.pine (diploid)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  # Load data
  data(snp.pine, package = "AGHmatrix")
  
  # Fake imputation
  snp.pine.fake <- ifelse(snp.pine == -9, 1, snp.pine)
  
  # New
  G_VanRadenPine  <- Gmatrix(SNPmatrix = snp.pine,      missingValue = -9,
                             maf = 0, method = "VanRaden")
  G_VanRadenPine2 <- Gmatrix(SNPmatrix = snp.pine.fake, missingValue = -9,
                             maf = 0, method = "VanRaden")
  
  # Legacy
  G_VanRadenPine_curr  <- Gmatrix_legacy(SNPmatrix = snp.pine,      missingValue = -9,
                                         maf = 0, method = "VanRaden")
  G_VanRadenPine2_curr <- Gmatrix_legacy(SNPmatrix = snp.pine.fake, missingValue = -9,
                                         maf = 0, method = "VanRaden")
  
  expect_false(any(round(G_VanRadenPine_curr,  4) != round(G_VanRadenPine,  4)),
               info = {
                 d <- round(G_VanRadenPine_curr,4) - round(G_VanRadenPine,4)
                 paste0("Rounded(4) mismatch in pine with missings. ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_false(any(round(G_VanRadenPine2_curr, 4) != round(G_VanRadenPine2, 4)),
               info = {
                 d <- round(G_VanRadenPine2_curr,4) - round(G_VanRadenPine2,4)
                 paste0("Rounded(4) mismatch in fake-imputed pine. ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  # Symmetry
  expect_true(isSymmetric(G_VanRadenPine))
  expect_true(isSymmetric(G_VanRadenPine2))
  expect_true(isSymmetric(G_VanRadenPine_curr))
  expect_true(isSymmetric(G_VanRadenPine2_curr))
})

test_that("new Gmatrix matches legacy (Yang) on snp.pine (diploid)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.pine, package = "AGHmatrix")
  snp.pine.fake <- ifelse(snp.pine == -9, 1, snp.pine)
  
  # New
  G_YangPine  <- Gmatrix(SNPmatrix = snp.pine,      missingValue = -9,
                         maf = 0.05, method = "Yang")
  G_YangPine2 <- Gmatrix(SNPmatrix = snp.pine.fake, missingValue = -9,
                         maf = 0.05, method = "Yang")
  
  # Legacy
  G_YangPine_curr  <- Gmatrix_legacy(SNPmatrix = snp.pine,      missingValue = -9,
                                     maf = 0.05, method = "Yang")
  G_YangPine2_curr <- Gmatrix_legacy(SNPmatrix = snp.pine.fake, missingValue = -9,
                                     maf = 0.05, method = "Yang")
  
  expect_false(any(round(G_YangPine_curr,  4) != round(G_YangPine,  4)),
               info = {
                 d <- round(G_YangPine_curr,4) - round(G_YangPine,4)
                 paste0("Rounded(4) mismatch in pine (Yang) with missings. ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_YangPine2_curr, 4) != round(G_YangPine2, 4)),
               info = {
                 d <- round(G_YangPine2_curr,4) - round(G_YangPine2,4)
                 paste0("Rounded(4) mismatch in fake-imputed pine (Yang). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_YangPine))
  expect_true(isSymmetric(G_YangPine2))
  expect_true(isSymmetric(G_YangPine_curr))
  expect_true(isSymmetric(G_YangPine2_curr))
})

test_that("new Gmatrix matches legacy (Su) on snp.pine (diploid)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.pine, package = "AGHmatrix")
  snp.pine.fake <- ifelse(snp.pine == -9, 1, snp.pine)
  
  # New
  G_SuPine  <- Gmatrix(SNPmatrix = snp.pine,      missingValue = -9,
                       maf = 0.05, method = "Su")
  G_SuPine2 <- Gmatrix(SNPmatrix = snp.pine.fake, missingValue = -9,
                       maf = 0.05, method = "Su")
  
  # Legacy
  G_SuPine_curr  <- Gmatrix_legacy(SNPmatrix = snp.pine,      missingValue = -9,
                                   maf = 0.05, method = "Su")
  G_SuPine2_curr <- Gmatrix_legacy(SNPmatrix = snp.pine.fake, missingValue = -9,
                                   maf = 0.05, method = "Su")
  
  expect_false(any(round(G_SuPine_curr,  4) != round(G_SuPine,  4)),
               info = {
                 d <- round(G_SuPine_curr,4) - round(G_SuPine,4)
                 paste0("Rounded(4) mismatch in pine (Su) with missings. ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_SuPine2_curr, 4) != round(G_SuPine2, 4)),
               info = {
                 d <- round(G_SuPine2_curr,4) - round(G_SuPine2,4)
                 paste0("Rounded(4) mismatch in fake-imputed pine (Su). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_SuPine))
  expect_true(isSymmetric(G_SuPine2))
  expect_true(isSymmetric(G_SuPine_curr))
  expect_true(isSymmetric(G_SuPine2_curr))
})

test_that("new Gmatrix matches legacy (Vitezica) on snp.pine (diploid)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.pine, package = "AGHmatrix")
  snp.pine.fake <- ifelse(snp.pine == -9, 1, snp.pine)
  
  # New
  G_VitPine  <- Gmatrix(SNPmatrix = snp.pine,      missingValue = -9,
                        maf = 0.05, method = "Vitezica")
  G_VitPine2 <- Gmatrix(SNPmatrix = snp.pine.fake, missingValue = -9,
                        maf = 0.05, method = "Vitezica")
  
  # Legacy
  G_VitPine_curr  <- Gmatrix_legacy(SNPmatrix = snp.pine,      missingValue = -9,
                                    maf = 0.05, method = "Vitezica")
  G_VitPine2_curr <- Gmatrix_legacy(SNPmatrix = snp.pine.fake, missingValue = -9,
                                    maf = 0.05, method = "Vitezica")
  
  expect_false(any(round(G_VitPine_curr,  4) != round(G_VitPine,  4)),
               info = {
                 d <- round(G_VitPine_curr,4) - round(G_VitPine,4)
                 paste0("Rounded(4) mismatch in pine (Vitezica) with missings. ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_VitPine2_curr, 4) != round(G_VitPine2, 4)),
               info = {
                 d <- round(G_VitPine2_curr,4) - round(G_VitPine2,4)
                 paste0("Rounded(4) mismatch in fake-imputed pine (Vitezica). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_VitPine))
  expect_true(isSymmetric(G_VitPine2))
  expect_true(isSymmetric(G_VitPine_curr))
  expect_true(isSymmetric(G_VitPine2_curr))
})

test_that("new MarkersMatrix matches legacy on snp.pine (diploid)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.pine, package = "AGHmatrix")
  snp.pine.fake <- ifelse(snp.pine == -9, 1, snp.pine)
  
  # New
  G_MM      <- Gmatrix(SNPmatrix = snp.pine,      missingValue = -9,
                       maf = 0.05, method = "MarkersMatrix")
  G_MM2     <- Gmatrix(SNPmatrix = snp.pine.fake, missingValue = -9,
                       maf = 0.05, method = "MarkersMatrix")
  
  # Legacy
  G_MM_curr  <- Gmatrix_legacy(SNPmatrix = snp.pine,      missingValue = -9,
                               maf = 0.05, method = "MarkersMatrix")
  G_MM2_curr <- Gmatrix_legacy(SNPmatrix = snp.pine.fake, missingValue = -9,
                               maf = 0.05, method = "MarkersMatrix")
  
  expect_false(any(round(G_MM_curr,  4) != round(G_MM,  4)),
               info = {
                 d <- round(G_MM_curr,4) - round(G_MM,4)
                 paste0("Rounded(4) mismatch in pine (MarkersMatrix). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_MM2_curr, 4) != round(G_MM2, 4)),
               info = {
                 d <- round(G_MM2_curr,4) - round(G_MM2,4)
                 paste0("Rounded(4) mismatch in fake-imputed pine (MarkersMatrix). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_MM))
  expect_true(isSymmetric(G_MM2))
  expect_true(isSymmetric(G_MM_curr))
  expect_true(isSymmetric(G_MM2_curr))
})

test_that("new Gmatrix matches VanRaden legacy on snp.sol (ploidy=4)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.sol, package = "AGHmatrix")
  snp.sol.fake <- ifelse(snp.sol == -9, 2, snp.sol)  # mid-dosage fake impute for tetraploids
  
  # New
  G_VR4   <- Gmatrix(SNPmatrix = snp.sol,      method = "VanRaden",
                     ploidy = 4, missingValue = -9, maf = 0.05)
  G_VR4_2 <- Gmatrix(SNPmatrix = snp.sol.fake, method = "VanRaden",
                     ploidy = 4, missingValue = -9, maf = 0.05)
  
  # Legacy
  G_VR4_curr   <- Gmatrix_legacy(SNPmatrix = snp.sol,      method = "VanRaden",
                                 ploidy = 4, missingValue = -9, maf = 0.05)
  G_VR4_2_curr <- Gmatrix_legacy(SNPmatrix = snp.sol.fake, method = "VanRaden",
                                 ploidy = 4, missingValue = -9, maf = 0.05)
  
  expect_false(any(round(G_VR4_curr,  4) != round(G_VR4,  4)),
               info = {
                 d <- round(G_VR4_curr,4) - round(G_VR4,4)
                 paste0("Rounded(4) mismatch (VanRaden, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_VR4_2_curr, 4) != round(G_VR4_2, 4)),
               info = {
                 d <- round(G_VR4_2_curr,4) - round(G_VR4_2,4)
                 paste0("Rounded(4) mismatch fake-imputed (VanRaden, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_VR4))
  expect_true(isSymmetric(G_VR4_2))
  expect_true(isSymmetric(G_VR4_curr))
  expect_true(isSymmetric(G_VR4_2_curr))
})

test_that("new Gmatrix matches legacy (Slater) on snp.sol (ploidy=4)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.sol, package = "AGHmatrix")
  snp.sol.fake <- ifelse(snp.sol == -9, 2, snp.sol)
  
  # New
  G_Sl4   <- Gmatrix(SNPmatrix = snp.sol,      method = "Slater",
                     ploidy = 4, missingValue = -9, maf = 0.05)
  G_Sl4_2 <- Gmatrix(SNPmatrix = snp.sol.fake, method = "Slater",
                     ploidy = 4, missingValue = -9, maf = 0.05)
  
  # Legacy
  G_Sl4_curr   <- Gmatrix_legacy(SNPmatrix = snp.sol,      method = "Slater",
                                 ploidy = 4, missingValue = -9, maf = 0.05)
  G_Sl4_2_curr <- Gmatrix_legacy(SNPmatrix = snp.sol.fake, method = "Slater",
                                 ploidy = 4, missingValue = -9, maf = 0.05)
  
  expect_false(any(round(G_Sl4_curr,  4) != round(G_Sl4,  4)),
               info = {
                 d <- round(G_Sl4_curr,4) - round(G_Sl4,4)
                 paste0("Rounded(4) mismatch (Slater, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_Sl4_2_curr, 4) != round(G_Sl4_2, 4)),
               info = {
                 d <- round(G_Sl4_2_curr,4) - round(G_Sl4_2,4)
                 paste0("Rounded(4) mismatch fake-imputed (Slater, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_Sl4))
  expect_true(isSymmetric(G_Sl4_2))
  expect_true(isSymmetric(G_Sl4_curr))
  expect_true(isSymmetric(G_Sl4_2_curr))
})

test_that("new Gmatrix matches legacy (Endelman) on snp.sol (ploidy=4)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.sol, package = "AGHmatrix")
  snp.sol.fake <- ifelse(snp.sol == -9, 2, snp.sol)
  
  # New
  G_End4   <- Gmatrix(SNPmatrix = snp.sol,      method = "Endelman",
                      ploidy = 4, missingValue = -9, maf = 0.05)
  G_End4_2 <- Gmatrix(SNPmatrix = snp.sol.fake, method = "Endelman",
                      ploidy = 4, missingValue = -9, maf = 0.05)
  
  # Legacy
  G_End4_curr   <- Gmatrix_legacy(SNPmatrix = snp.sol,      method = "Endelman",
                                  ploidy = 4, missingValue = -9, maf = 0.05)
  G_End4_2_curr <- Gmatrix_legacy(SNPmatrix = snp.sol.fake, method = "Endelman",
                                  ploidy = 4, missingValue = -9, maf = 0.05)
  
  expect_false(any(round(G_End4_curr,  4) != round(G_End4,  4)),
               info = {
                 d <- round(G_End4_curr,4) - round(G_End4,4)
                 paste0("Rounded(4) mismatch (Endelman, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_End4_2_curr, 4) != round(G_End4_2, 4)),
               info = {
                 d <- round(G_End4_2_curr,4) - round(G_End4_2,4)
                 paste0("Rounded(4) mismatch fake-imputed (Endelman, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_End4))
  expect_true(isSymmetric(G_End4_2))
  expect_true(isSymmetric(G_End4_curr))
  expect_true(isSymmetric(G_End4_2_curr))
})

test_that("new Gmatrix matches VanRaden legacy (pseudo-diploid) on snp.sol (ploidy=4, pseudo=TRUE)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.sol, package = "AGHmatrix")
  snp.sol.fake <- ifelse(snp.sol == -9, 2, snp.sol)
  
  # New
  G_VRpd   <- Gmatrix(SNPmatrix = snp.sol,      method = "VanRaden",
                      ploidy = 4, pseudo.diploid = TRUE,
                      missingValue = -9, maf = 0.05)
  G_VRpd_2 <- Gmatrix(SNPmatrix = snp.sol.fake, method = "VanRaden",
                      ploidy = 4, pseudo.diploid = TRUE,
                      missingValue = -9, maf = 0.05)
  
  # Legacy
  G_VRpd_curr   <- Gmatrix_legacy(SNPmatrix = snp.sol,      method = "VanRaden",
                                  ploidy = 4, pseudo.diploid = TRUE,
                                  missingValue = -9, maf = 0.05)
  G_VRpd_2_curr <- Gmatrix_legacy(SNPmatrix = snp.sol.fake, method = "VanRaden",
                                  ploidy = 4, pseudo.diploid = TRUE,
                                  missingValue = -9, maf = 0.05)
  
  expect_false(any(round(G_VRpd_curr,  4) != round(G_VRpd,  4)),
               info = {
                 d <- round(G_VRpd_curr,4) - round(G_VRpd,4)
                 paste0("Rounded(4) mismatch (VanRaden pseudo-diploid, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  expect_false(any(round(G_VRpd_2_curr, 4) != round(G_VRpd_2, 4)),
               info = {
                 d <- round(G_VRpd_2_curr,4) - round(G_VRpd_2,4)
                 paste0("Rounded(4) mismatch fake-imputed (VanRaden pseudo-diploid, ploidy=4). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_VRpd))
  expect_true(isSymmetric(G_VRpd_2))
  expect_true(isSymmetric(G_VRpd_curr))
  expect_true(isSymmetric(G_VRpd_2_curr))
})

test_that("new Gmatrix matches VanRaden legacy on snp.pine with weights (diploid)", {
  Sys.setenv(OPENBLAS_NUM_THREADS = "1",
             MKL_NUM_THREADS      = "1",
             OMP_NUM_THREADS      = "1")
  
  data(snp.pine, package = "AGHmatrix")
  # Weights matched by column name (legacy/new both re-align by names)
  set.seed(123)
  w <- runif(ncol(snp.pine), min = 0.001, max = 0.1)
  names(w) <- colnames(snp.pine)
  
  G_w   <- Gmatrix(SNPmatrix = snp.pine, missingValue = -9, maf = 0.05,
                   method = "VanRaden", weights = w)
  G_w_c <- Gmatrix_legacy(SNPmatrix = snp.pine, missingValue = -9, maf = 0.05,
                          method = "VanRaden", weights = w)
  
  expect_false(any(round(G_w_c,  4) != round(G_w,  4)),
               info = {
                 d <- round(G_w_c,4) - round(G_w,4)
                 paste0("Rounded(4) mismatch (VanRaden weighted). ",
                        "max|delta|=", max(abs(d)),
                        ", nnz(delta)=", sum(d != 0))
               })
  
  expect_true(isSymmetric(G_w))
  expect_true(isSymmetric(G_w_c))
})

#########################################################################
# 									
# Package: AGHmatrix 							
# 									
# File: Amatrix.R
# Contains: Amatrix 							
# 									
# Written by Rodrigo Rampazo Amadeu and Thiago de Paula Oliveira
# 									
# First version: Feb-2014 
# Last update: 21-July-2026 						
# License: GPL-3
# 								
#########################################################################

#' @title Construction of Relationship Matrix A
#'
#' @description
#' Constructs an additive relationship matrix (A) from pedigree data in a 
#' three-column format. The calculation depends on the specified ploidy level 
#' (must be an even  number). If \code{ploidy = 4}, the matrix can incorporate a 
#' user-defined proportion (\code{w}) of parental gametes that are IBD 
#' (Identical by Descent) due to double reduction.
#'
#' When \code{ploidy = 2} and \code{dominance = FALSE}, the diploid additive 
#' numerator relationship matrix is built using Henderson's (1976) method. If 
#' \code{dominance = TRUE}, a diploid dominance numerator relationship matrix 
#' is computed following Cockerham (1954). The recursive approach is detailed 
#' in Mrode (2005).
#'
#' For \code{ploidy > 2} and \code{dominance = FALSE}, an autopolyploid 
#' additive relationship matrix is computed based on Kerr et al. (2012). 
#' If \code{slater = TRUE} and \code{ploidy = 4}, Slater's (2013) method is 
#' used instead.
#'
#' @param data Pedigree data frame in a 3-column format. Unknown values should 
#' be set to 0.
#' @param ploidy An even number representing the ploidy level (default is 2).
#' @param w Proportion of parental gametes that are IBD due to double 
#' reduction (default is 0). Applicable only when \code{ploidy = 4}.
#' @param verify Logical; if TRUE (default), the pedigree file is checked for 
#' conflicting entries.
#' @param dominance Logical; if TRUE, computes the dominance relationship 
#' matrix (valid only for \code{ploidy = 2}).
#' @param slater Logical; if TRUE and \code{ploidy = 4}, computes the additive 
#' autotetraploid relationship matrix using Slater (2013).
#' @param ASV Logical; if TRUE, transforms the matrix into the average 
#' semivariance (ASV) defined as 
#' \eqn{K = K / (\text{trace}(K) / (\text{nrow}(K) - 1))}. See Feldmann et al. 
#' (2022), Formula 2.
#' @param ... Additional arguments passed to \code{datatreat()}.
#' 
#' @return A square matrix representing the relationship between individuals 
#' in the pedigree.
#'
#' @examples
#' data(ped.mrode)
#' # Additive relationship matrix for diploids (Henderson 1976):
#' Amatrix(ped.mrode, ploidy = 2)
#'
#' # Dominance relationship matrix for diploids (Cockerham 1954):
#' Amatrix(ped.mrode, ploidy = 2, dominance = TRUE)
#'
#' # Additive relationship matrix for autotetraploids (Kerr 2012):
#' Amatrix(ped.mrode, ploidy = 4)
#'
#' # Additive relationship matrix for autooctaploids (Kerr 2012):
#' Amatrix(ped.mrode, ploidy = 8)
#'
#' # Additive relationship matrix for autotetraploids with double reduction 
#' # (Kerr 2012):
#' Amatrix(ped.mrode, ploidy = 4, w = 0.1)
#'
#' # Additive relationship matrix for autotetraploids with double reduction 
#' # using Slater (2013):
#' Amatrix(ped.mrode, ploidy = 4, w = 0.1, slater = TRUE)
#'
#' # Additive relationship matrix for autohexaploids with double reduction 
#' # (Kerr 2012):
#' Amatrix(ped.mrode, ploidy = 6, w = 0.1)
#'
#' @useDynLib AGHmatrix, .registration = TRUE
#' @importFrom Rcpp evalCpp
#' @note This function uses compiled C++ code via Rcpp for improved performance.
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#' @author Thiago de Paula Oliveira, \email{toliveira@@abacusbio.com}
#'
#' @references 
#'   \emph{Cockerham, C.C. (1954). An extension of the concept of partitioning 
#'   hereditary variance for analysis of covariances among relatives when 
#'   epistasis is present. Genetics, 39, 859–882.}
#' @references 
#'   \emph{Feldmann, M.J., et al. (2022). Average semivariance directly yields 
#'   accurate estimates of the genomic variance in complex trait analyses. 
#'   G3 (Bethesda), 12(6).}
#' @references 
#'   \emph{Henderson, C.R. (1976). A simple method for computing the inverse 
#'   of a numerator relationship matrix used in prediction of breeding values. 
#'   Biometrics, 32, 69–83.}
#' @references 
#'   \emph{Kerr, R.J., et al. (2012). Use of the numerator relationship matrix 
#'   in genetic analysis of autopolyploid species. Theoretical and Applied 
#'   Genetics, 124, 1271–1282.}
#' @references 
#'   \emph{Mrode, R.A. (2014). Linear Models for the Prediction of Animal 
#'   Breeding Values (3rd ed.). CABI.}
#' @references 
#'   \emph{Slater, A.T., et al. (2013). Improving the analysis of low 
#'   heritability complex traits for enhanced genetic gain in potato. 
#'   Theoretical and Applied Genetics, 127, 809–820.}
#' @export
Amatrix <- function(data = NULL,
                    ploidy = 2,
                    w = 0,
                    verify = TRUE,
                    dominance = FALSE,
                    slater = FALSE,
                    ASV = FALSE,
                    ...) {
  #-------------------------------------------------------------------------
  # 1. Validate Inputs
  #-------------------------------------------------------------------------
  if (is.null(data) || !is.data.frame(data) || ncol(data) < 3)
    stop("Input 'data' must be a non-null data frame with at least 3 columns.")
  
  if (nrow(data) == 0)
    stop("Pedigree data is empty.")
  
  if (ploidy %% 2 != 0)
    stop("Ploidy should be an even number.")
  
  if (ploidy < 2)
    stop("Ploidy must be at least 2.")
  
  if (ploidy != 2 && dominance)
    stop("Dominance matrix only implemented for ploidy = 2.")
  
  if (slater && ploidy != 4) {
    warning(paste("Slater method is only valid for ploidy = 4.",
                  "Proceeding with default method."))
    slater <- FALSE
  }
  
  #-------------------------------------------------------------------------
  # 2. Pedigree Preprocessing and Verification
  #-------------------------------------------------------------------------
  orig.order <- as.character(data[[1]])
  
  if (verify) {
    cat("Verifying conflicting data...\n")
    if (verifyped(data)) stop("Please double-check your data and try again.")
  }

  cat("Organizing data with fast method...\n")
  data.after.treat <- try(datatreat(data = data, unk = 0, ...), silent = TRUE)
  
  if (inherits(data.after.treat, "try-error") ||
      !is.list(data.after.treat) ||
      is.null(data.after.treat$ind_data) ||
      length(unique(data.after.treat$ind_data)) != nrow(data)) {
    stop(paste("It wasn't possible to organize your data chronologically.",
               "Check for conflicting pedigree entries, duplicate IDs, or circular references."))
  }
  
  #-------------------------------------------------------------------------
  # 3. Matrix Construction (via Rcpp)
  #-------------------------------------------------------------------------
  data <- data.after.treat
  s <- as.integer(data$sire)
  d <- as.integer(data$dire)
  n <- length(s)
  
  if (!is.numeric(s) || !is.numeric(d) || length(s) != length(d))
    stop("Sire and dam columns must be numeric vectors of equal length.")
  
  if (n > 1000)
    cat("Processing large pedigree data. It may take a couple of minutes...\n")

  start_time <- Sys.time()

  if (ploidy == 2) {
    cat("Constructing matrix A using ploidy = 2\n")
    A <- buildA_ploidy2_cpp(s, d, n)
    if (dominance) {
      cat("Constructing dominance relationship matrix\n")
      A <- buildDominanceMatrix_cpp(A, s, d)
    }

  } else if (slater && ploidy == 4) {
    cat(sprintf(
      "Constructing matrix A using Slater et al. (2014), ploidy = 4, w = %.2f\n",
      w))
    A <- buildA_slater_cpp(s, d, w)

  } else {
    cat(sprintf(
      "Constructing matrix A using Kerr et al. (2012), ploidy = %d, w = %.2f\n",
      ploidy, w))
    v <- ploidy / 2
    A <- buildA_kerr_cpp(s, d, w, v)
  }
  
  #-------------------------------------------------------------------------
  # 4. Post-processing
  #-------------------------------------------------------------------------
  if (anyNA(A))
    cat(paste("Warning: Matrix contains NA values. Use",
              "'verifyped()' to check data.\n"))
  
  dimnames(A) <- list(data$ind_data, data$ind_data)
  
  # Only reorder if necessary
  if (!all(orig.order == data$ind_data)) {
    idx <- match(orig.order, data$ind_data)
    A <- A[idx, idx, drop = FALSE]
  }
  
  if (ASV) {
    A <- get_ASV(A)
  }
  
  attr(A, "ploidy") <- ploidy
  attr(A, "method") <- if (dominance) {
    "dominance"
  } else if (slater) {
    "slater"
  } else {
    "default"
  }
  
  elapsed <- difftime(Sys.time(), start_time, units = "mins")
  cat("Completed! Time =", round(elapsed, 2), "minutes\n")
  
  return(A)
}

# Parity tests between new Amatrix() and Amatrix_legacy()
Amatrix_legacy <- function(data = NULL,
                    ploidy=2,
                    w=0,
                    verify=TRUE,
                    dominance=FALSE,
                    slater=FALSE,
                    ASV=FALSE,
                    ...){
  if(ploidy%%2!=0)
    stop(deparse("Ploidy should be an even number"))
  
  if(ploidy!=2 & dominance)
    stop(deparse("Dominance relationship matrix is implemented only for ploidy=2"))
  
  if( is.null(data))
    stop(deparse("Please define the variable data"))
  
  unk=0
  cat("Verifying conflicting data... \n")
  flag<-verifyped_legacy(data)
  if(flag) 
    stop(deparse("Please double-check your data and try again"))
  
  cat("Organizing data... \n")
  orig.order <- as.character(data[,1])
  data.after.treat <- try(datatreat_legacy(data=data,unk=unk,...),silent=TRUE)
  
  # checking if order was fixed
  flag = FALSE
  flag = inherits(data.after.treat,"try-error")
  if(!flag)
    flag = (length(unique(data.after.treat$ind.data))!=nrow(data))
  
  if(flag){
    cat("To organize the data in a fast way wasn't possible... \n")
    cat("Trying to organize in a slow (naive) way... \n")
    data.sorted <- sortped_legacy(data)
    data.after.treat <- try(datatreat_legacy(data=data.sorted,unk=unk,...))
    
    # checking if order was fixed
    flag = FALSE
    flag = inherits(data.after.treat,"try-error")
    if(!flag)
      flag = (length(unique(data.after.treat$ind.data))!=nrow(data))
    
    if(flag){
      cat("It wasn't possible to organize your data chronologically. We recommend you to do it by hand or use the flag 'naive_sort=TRUE'. If the problem persists, please contact this package mainteiner \n")
      return()
    }
  }
  
  data <- data.after.treat
  
  s <- data$sire
  d <- data$dire
  
  if( is.null(s) || is.null(d) )
    stop(deparse("Please define the variable s (sire) and/or d (dire)"))
  
  if( length(s) != length(d) )
    stop(deparse("Please verify the variable s (sire) and/or d (dire), they don't have the same length"))
  
  if( !is.numeric(s) || !is.numeric(d) )
    stop(deparse("Pleasy verify your data, it has to be 2 numeric vectors"))
  
  if( length(data$sire) > 1000 )
    cat("Processing a large pedigree data... It may take a couple of minutes... \n")
  
  
  n <- length(s)
  A <- matrix(NA,ncol=n,nrow=n)
  
  Time = proc.time()
  
  #### For ploidy 2 ####
  if(ploidy == 2){
    w <- NA
    cat("Constructing matrix A using ploidy = 2 \n")
    A[1,1] <- 1
    for( i in 2:n){
      
      ## Both are unknown
      if( s[i] == 0 && d[i] == 0 ){
        A[i,i] <- 1
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0
      }
      
      ## Sire is unknown
      if( s[i] == 0 && d[i] != 0 ){
        A[i,i] <- 1
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,d[i]])
      }
      
      ## Dire is unknown
      if( d[i] == 0 && s[i] != 0 ){
        A[i,i] <- 1
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,s[i]])
      }
      
      ## Both are known
      if( d[i] != 0 && s[i] != 0 ){
        A[i,i] <- 1+0.5*(A[d[i],s[i]])
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,s[i]]+A[j,d[i]])
      }
    }
  }
  
  if(dominance){
    cat("Constructing dominance relationship matrix \n")
    D <- matrix(NA,ncol=n,nrow=n)
    for(i in 1:n){
      for(j in 1:n){
        u1 <- ifelse(length(A[s[i],s[j]])>0,A[s[i],s[j]],0)
        u2 <- ifelse(length(A[d[i],d[j]])>0,A[d[i],d[j]],0)
        u3 <- ifelse(length(A[s[i],d[j]])>0,A[s[i],d[j]],0)
        u4 <- ifelse(length(A[s[j],d[i]])>0,A[s[j],d[i]],0)
        D[i,j] <- D[j,i] <- 0.25*(u1*u2+u3*u4)
      }
    }
    diag(D)<-1
    A<-D
    D<-NULL
  }
  
  #### For ploidy 4 ####
  if(slater==TRUE){
    listA <- list()
    
    cat(paste("Constructing matrix A using ploidy = 4 and proportion of double reduction =",w,";as in Slater et al. (2014) \n"))
    start.time <- Sys.time()
    
    A[1,1] <- (1+w)/4
    for( i in 2:n){
      
      ## Both are unknown
      if( s[i] == 0 && d[i] == 0 ){
        A[i,i] <- (1+w)/4
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0
      }
      
      ## Sire is unknown
      if( s[i] == 0 && d[i] != 0 ){
        A[i,i] <- (5 + 7*w + 4*A[d[i],d[i]]*(1-w) ) / 24
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,d[i]])
      }
      
      ## Dire is unknown
      if( d[i] == 0 && s[i] != 0 ){
        A[i,i] <- (5 + 8*w + 4*A[s[i],s[i]]*(1-w) ) / 24 ##On Slater in 7w, deriving from hand based on Kerr is 8w
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,s[i]])
      }
      
      ## Both are known
      if( d[i] != 0 && s[i] != 0 ){
        A[i,i] <- (1 + 2*w + (1-w)*(A[s[i],s[i]]) + (1-w)*(A[d[i],d[i]]) + 3*A[s[i],d[i]] ) / 6
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,s[i]]+A[j,d[i]])
      }
    }
    
    A <- 4*A
  }
  
  if(slater==FALSE && ploidy>2){ ## It does not use double-reduction proportion, need to double-check formula on kerr 2012 for higher ploidies...
    listA <- list()
    cat(paste("Constructing matrix A using ploidy =",ploidy,"and proportion of double reduction =",w,";as in Kerr et al. (2012) \n"))
    start.time <- Sys.time()
    v = ploidy/2
    A[1,1] <- (1)/(2*v)
    for( i in 2:n){
      ## Both are unknown
      if( s[i] == 0 && d[i] == 0 ){
        A[i,i] <- (1)/(2*v)
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0
      }
      
      ## Sire is unknown
      if( s[i] == 0 && d[i] != 0 ){
        A[i,i] <- (1 + (v-1)*w + ((v-1)*(1-w)*(v*A[d[i],d[i]] + 1/2 - 1))/(2*v-1))/(2*v)
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,d[i]])
      }
      
      ## Dire is unknown
      if( d[i] == 0 && s[i] != 0 ){
        A[i,i] <- (1 + (v-1)*w + ((v-1)*(1-w)*(v*A[s[i],s[i]] + 1/2 - 1))/(2*v-1))/(2*v)
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,s[i]])
      }
      
      ## Both are known
      if( d[i] != 0 && s[i] != 0 ){
        A[i,i] <- (1  + (v-1)*w + ((v-1)*(1-w)*(v*A[d[i],d[i]] + v*A[s[i],s[i]] - 1)/(2*v-1)))/(2*v) + A[d[i],s[i]]/2
        for( j in 1:(i-1))
          A[j,i] <- A[i,j] <- 0.5*(A[j,s[i]]+A[j,d[i]])
      }
    }
    
    A <- 2*v*A
  }
  
  NA.errors <- which(is.na(A))
  if( length(NA.errors) > 0 )
    cat("Please verify your original data with the function 'verifyped', there are some data missing/conflicting data \n")
  
  Time = as.matrix(proc.time()-Time)
  cat("Completed! Time =", Time[3]/60," minutes \n")
  #      "Visualization options: (matrix, w) \n ")
  rownames(A) <- colnames(A) <- data$ind.data
  
  A <- A[orig.order,orig.order]
  
  
  if (ASV) {
    A = get_ASV_legacy(A)
  }
  
  return(A)
}

# Helpers
verifyped_legacy <- function(data) FALSE
sortped_legacy <- function(data) data
datatreat_legacy <- function(data, unk = 0, ...) {
  stopifnot(is.data.frame(data), ncol(data) >= 3)
  colnames(data)[1:3] <- c("ind", "sire", "dam")
  ids <- as.character(data$ind)
  si  <- as.character(data$sire)
  da  <- as.character(data$dam)
  # Unknowns indicated by "0" or 0 -> keep as 0
  map <- setNames(seq_along(ids), ids)
  sire_idx <- ifelse(si == "0" | si == 0, 0L, as.integer(map[si]))
  dam_idx  <- ifelse(da == "0" | da == 0, 0L, as.integer(map[da]))
  out <- data.frame(
    ind.data = ids,
    sire     = sire_idx,
    dire     = dam_idx,
    stringsAsFactors = FALSE
  )
  out
}
get_ASV_legacy <- function(x) x / (sum(diag(x)) / (nrow(x) - 1))

# Pedigree
# Founders: A,B,C,D ; Progeny: E=A×B, F=C×D ; G=E×F ; H=E×F
make_ped <- function() {
  data.frame(
    id   = c("A","B","C","D","E","F","G","H"),
    sire = c("0","0","0","0","A","C","E","E"),
    dam  = c("0","0","0","0","B","D","F","F"),
    stringsAsFactors = FALSE
  )
}

make_ped2 <- function() {
  base <- make_ped()
  extra <- data.frame(
    id   = c("I","J"),
    sire = c("G","G"),
    dam  = c("H","E"),
    stringsAsFactors = FALSE
  )
  rbind(base, extra)
}

# ---- Utility: strip non-structural attrs and compare matrices ----
strip_attrs <- function(x) {
  at <- attributes(x)
  if (is.null(at)) return(x)
  keep <- c("dim", "dimnames", "class")
  attributes(x) <- at[names(at) %in% keep]
  x
}

compare_A <- function(ped, ..., tol = 1e-10) {
  legacy <- Amatrix_legacy(ped, ...)
  target <- Amatrix(ped, ...)
  legacy <- strip_attrs(legacy)
  target <- strip_attrs(target)
  expect_equal(target, legacy, tolerance = tol, scale = 1)
  expect_true(isSymmetric(target))
  expect_identical(dim(target), c(nrow(ped), nrow(ped)))
  invisible(list(legacy = legacy, target = target))
}

# Tests

test_that("Diploid additive (Henderson 1976) parity", {
  ped <- make_ped()
  compare_A(ped, ploidy = 2, dominance = FALSE, verify = FALSE)
})

test_that("Diploid dominance (Cockerham 1954) parity", {
  ped <- make_ped()
  compare_A(ped, ploidy = 2, dominance = TRUE, verify = FALSE)
})

test_that("ASV rescaling parity (diploid additive)", {
  ped <- make_ped()
  compare_A(ped, ploidy = 2, dominance = FALSE, ASV = TRUE, verify = FALSE)
})

test_that("Autotetraploid additive (Kerr 2012) parity, w = 0", {
  ped <- make_ped()
  compare_A(ped, ploidy = 4, w = 0, slater = FALSE, verify = FALSE)
})

test_that("Autotetraploid additive (Kerr 2012) parity, w > 0", {
  ped <- make_ped()
  compare_A(ped, ploidy = 4, w = 0.1, slater = FALSE, verify = FALSE)
})

test_that("Autotetraploid additive (Slater 2013) parity, w > 0", {
  ped <- make_ped()
  compare_A(ped, ploidy = 4, w = 0.1, slater = TRUE, verify = FALSE)
})

test_that("Higher ploidy (hexaploid, Kerr 2012) parity, w = 0", {
  ped <- make_ped2()
  compare_A(ped, ploidy = 6, w = 0, slater = FALSE, verify = FALSE)
})

test_that("Higher ploidy (hexaploid, Kerr 2012) parity, w > 0", {
  ped <- make_ped2()
  compare_A(ped, ploidy = 6, w = 0.05, slater = FALSE, verify = FALSE)
})

test_that("Row/column names and order preserved", {
  ped <- make_ped2()
  out1 <- Amatrix_legacy(ped, ploidy = 2, verify = FALSE)
  out2 <- Amatrix(ped, ploidy = 2, verify = FALSE)
  expect_identical(rownames(out2), rownames(out1))
  expect_identical(colnames(out2), colnames(out1))
})

test_that("Dominance requested with ploidy != 2", {
  ped <- make_ped()
  expect_error(Amatrix_legacy(ped, ploidy = 4, dominance = TRUE, verify = FALSE))
  expect_error(Amatrix(ped, ploidy = 4, dominance = TRUE, verify = FALSE))
})

#########################################
# 									
# Package: AGHmatrix 							
# 									
# File: missingdata.R							
# Contains: missingdata							
# 									
# Written by Rodrigo Rampazo Amadeu 					
# 									
# First version: Feb-2014 						
# Last update: 14-Apr-2015 						
# License: GPL-3
# 									
#########################################

#' Survying on missing data
#'
#' This function verify which rows in a pedigree data has missing parental or conflictuos data
#'
#' @param data data name from a pedigree list
#' @param unk unknown value of your data
#'
#' @return list with $conflict: rows of the data which are at least one parental name equal to the individual. $missing.sire: rows of the data which arie missing data sire (Parental 1) information. $missing.dire: same as above for dire (Parental 2). $summary.missing: summary of the missing data. 2 columns, 1st for the name of the parental listed, 2nd for the how many times appeared in the data.
#'
#' @examples 
#' data(ped.mrode)
#' missingdata(ped.mrode)
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#'
#' @export

missingdata <- function(data,
			unk=0
                        ){
    pedigree.data<-data
    data <- c()

    if( ncol(pedigree.data) != 3 ){
      print("Data with more than 3 columns, please verify")
      return()
    }

    #Treating all as numeric
    ind.data <- as.vector(c(unk,as.character(pedigree.data[,1])))
    sire.data <- as.vector(pedigree.data[,2])
    dire.data <- as.vector(pedigree.data[,3])
    sire <- match(sire.data, ind.data)
    dire <- match(dire.data, ind.data)
    ind <- as.vector(c(1:length(ind.data)))
    missing <- c()

    #Verify the individual w/ same name in sire/dire
    missing$conflict <- c(which( sire == ind[-1]),which( dire == ind[-1] ))

    #Verify the missing sire (Parent 1)
    missing$sire.na <- c( which(is.na(sire)))

    #Verify the missing dire (Parent 2)
    missing$dire.na <- c( which(is.na(dire)))

    #Making a summary of the missing data
    missing$sire <- as.matrix(summary(as.factor(pedigree.data[which(is.na(sire)),2])))
    missing$dire <- as.matrix(summary(as.factor(pedigree.data[which(is.na(dire)),3])))
    names <- unique(c(rownames(missing$sire),rownames(missing$dire)))
    pos.sire <- match(rownames(missing$sire),names)
    pos.dire <- match(rownames(missing$dire),names)
    missing$parent <- rep(0,length(names))
    missing$parent[pos.dire] <- missing$dire
    missing$parent[pos.sire] <- missing$parent[pos.sire]+missing$sire
    missing$parent <- as.matrix(missing$parent)
    rownames(missing$parent) <- names
    #Final list
    missing <- list(conflict=missing$conflict,missing.sire=missing$sire.na,missing.dire=missing$dire.na,summary.missing=missing$parent)

    ## Improve this part :)
    #if(molecular){
    # if( csv )
    #    data <- read.csv("molecular_diploid.csv")
    #  mol.data <- data[,-1]
    #  row.names(mol.data) <- data[,1]
    #  mol.data <- replace(mol.data, mol.data == unk, NA)

    #  missing.per.ind <- apply(mol.data,1, function(x) sum(is.na(x)))# / ncol(example) * 100
    #  missing.per.marker  <- apply(t(mol.data),1,function(x) sum(is.na(x)))
    #  summary(missing.per.ind)/length(missing.per.marker)
    #  summary(missing.per.marker)/length(missing.per.ind)
    #}
  return(missing)
}

#########################################################################
# 									
# Package: AGHmatrix 							
# 									
# File: formatmatrix.R 						
# Contains: formatmatrix						
# 									
# Written by Rodrigo Rampazo Amadeu 					
# 									
# First version: Feb-2014 						
# Last update: 20-May-2015 						
# License: GPL-3
# 								
#######################################

#' Transform a matrix in 3 columns
#'
#' Given any square matrix transform it in a 3 columns way (row, column, value) mainly to be used in outsourcing data processing (as ASREML-standalone)
#'
#' @param data matrix (nxn).
#' @param save if TRUE save the output in a file. Default=TRUE.
#' @param return if TRUE return the output in a object. Default=FALSE.
#' @param name name of the csv file to be saved. Default=data name.
#' @param round.by select the number of digits after 0 you want in your data. Default = 12
#' @param exclude.0 if TRUE, remove all lines equal to zero (ASREML option). Default = TRUE
#' 
#' @return a object or a csv file with a table with 3 columns representing the matrix.
#'
#' @examples
#' #Example with random matrix
#' data<-matrix(c(1,0.1,0,0.1,1,0,0,0,1.1),3)
#' formatmatrix(data=data,save=FALSE,return=TRUE,exclude.0=TRUE)
#'
#' #Example with pedigree matrix
#' #Reading the example data
#' data(ped.mrode)
#' #Making Relationship Matrix
#' Amrode<-Amatrix(ped.mrode)
#' #Inverting the Matrix
#' Amrode.inv<-solve(Amrode)
#' #Making the 3 columns format
#' Amrode.inv.ASREML<-formatmatrix(Amrode,save=FALSE,return=TRUE,exclude.0=TRUE)
#' #Printing it
#' Amrode.inv.ASREML 
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#'
#' @export

formatmatrix <- function(
    data = NULL,
    save = TRUE,
    return = FALSE,
    name = deparse(substitute(data)),
    round.by = 12,
    exclude.0 = TRUE)
    {
  if( is.null(data))
    stop(deparse("Define a matrix data"))
  
  cat("Converting to column format... \n")
  
  Time <- proc.time()
  n <- nrow(data)
  first <- second <- third <- c()
  for( i in 1:n ){
    first <- c(first,rep(i,i))
    second <- c(second,c(1:i))
  }
  third <- data[upper.tri(data,diag=TRUE)]
  
  third<-round(third,round.by)
  
  columns <- cbind(first, second, third)
  
  if(exclude.0)
    columns<-columns[third!=0,]
  
  if(save){
    write.table(columns,file=paste(name,".csv",sep=""),sep=" ",row.names=FALSE,col.names=FALSE)
    cat(paste("Saved as ",name,".csv"," \n",sep=""))
  }
  
  Time = as.matrix(proc.time()-Time[3])
    
  attr(columns, "rownames") <- rownames(data) #necessary to ingest in asremlr as an R object
  
  cat("Completed! Time =", Time[3]/60," minutes \n")
  if(return)
    return( columns )
}

#' Molecular data for diploid examples
#'
#' Dataset extract from  supplementary material from Resende et al. (2012). 
#' SNP marker matrix from Pine tree coded as 0,1, and 2, and missing value as -9.
#'
#' @docType data
#'
#' @usage data(snp.pine)
#'
#' @format matrix
#'
#' @keywords datasets
#'
#' @references Resende, MF, et al., 2012 Accuracy of genomic selection methods in a standard data set of loblolly pine (Pinus taeda l.). Genetics 190: 1503–1510.
#'
#' @examples data(snp.pine)
#'
#' @name snp.pine
NULL

#########################################################################
# 									
# Package: AGHmatrix 							
# 									
# File: AmatrixPolyCross.R
# Contains: AmatrixPolyCross 							
# 									
# Written by Rodrigo Rampazo Amadeu 			
# Contributors: Leticia AC Lara
# 									
# First version: 20-Nov-2020 
# Last update: 05-Aug-2021 						
# License: GPL-3
# 								
#########################################################################

#' Construction of pedigree-based relationship matrix with parental guessing possibility
#'
#' Creates an additive relationship matrix A based on a non-deterministic pedigree with 4+ columns where each column represents a possible parent. This function was built with the following designs in mind. 
#' 1) A mating design where you have equally possible parents. For example, a generation of insects derived from the mating of three insects in a cage. All the insects in this generation will have the same expected relatedness with all the possible parents (1/3). If there are only two parents in the cage, the function assumes no-inbreeding and the pedigree is deterministic (the individual is offspring of the cross between the two parents). Another example, a population of 10 open-pollinated plants where you harvest the seeds without tracking the mother. 
#' 2) When fixedParent is TRUE: a mating design where you know one parent and might know the other possible parents. For example, a polycross design where you have seeds harvested from a mother plant and possible polen donors.
#'
#' @param data pedigree data name. Unknown value should be equal 0. See example for construction.
#' @param fixedParent if false, assumes that all the parents are equally possible parents. If true, assumes that the first parental is known and the others are equally possible parents. Default = FALSE.
#' 
#' @return Matrix with the relationship between the individuals.
#'
#' @examples
#' #the following pedigree has the id of the individual followed by possible parents
#' #if 0 is unknown
#' #the possible parents are filled from left to right
#' #in the pedigree data frame examples:
#' #id 1,2,3,4 have unknown parents and are assumed unrelated
#' #id 5 has three possible parents (1,2,3)
#' #id 6 has three possible parents (2,3,4)
#' #id 7 has two parents (deterministic case here, the parents are 3 and 4)
#' #id 8 has four possible parents (5,6,7,1)
#' 
#' pedigree = data.frame(id=1:8,
#'                       parent1 = c(0,0,0,0,1,2,3,5),
#'                       parent2 = c(0,0,0,0,2,3,4,6),
#'                       parent3 = c(0,0,0,0,3,4,0,7),
#'                       parent4 = c(0,0,0,0,0,0,0,1),
#'                       parent5 = 0)
#'
#' print(pedigree)
#'
#' AmatrixPolyCross(pedigree)
#' 
#' #when polyCross is set to be true:
#' #id 5 is offspring of parent 1 in a deterministic way and two other possible parents (2,3)
#' #id 6 is offspring of parent 2 in a deterministic way and two other possible parents (3,4)
#' #id 7 has two parents (deterministic case here, the parents are 3 and 4); as before
#' #id 8 is offspring of parent 5 in a deterministic way and has three other possible parents (6,7,1)
#' 
#' AmatrixPolyCross(pedigree,fixedParent=TRUE)
#'
#' @author Rodrigo R Amadeu, \email{rramadeu@@gmail.com}
#' @importFrom utils combn write.table
#'
#' @export
#' 
AmatrixPolyCross = function(data = NULL, fixedParent=FALSE){
  unk = 0
  orig.order <- as.character(data[, 1])
  
  data1 <- AGHmatrix::datatreat(data = data[,c(1,2,3)], unk = unk)
  Parents <- data1$sire
  Parents <- rbind(Parents,data1$dire)
  
  for(i in 4:ncol(data))
    Parents <- rbind(Parents, AGHmatrix::datatreat(data = data[,c(1,2,i)], unk = unk)$dire)
  
  for(i in 1:ncol(Parents)){
    tst = which(Parents[,i]!=0)
    if(length(tst)>0)
      if(min(which(Parents[,i]==0)) < max(tst))
        stop(deparse(paste0("Check parent order of line ",i,
                            ", missing (or non-used) values should be located at the right!")))
  }
  
  if (length(data$sire) > 1000) 
    cat("Processing a large pedigree data... It may take a couple of minutes... \n")
  
  if(fixedParent==FALSE){
    n <- ncol(Parents)
    Time = proc.time()
    
    combs = NULL
    for(i in 2:nrow(Parents)){
      combs = cbind(combs, combn(i,2))
    }
    
    combs = t(unique(t(combs))) #fixing order by number of max parents
    
    cat("Constructing matrix A using ploidy = 2 \n")
    Afinal = matrix(NA,n,n)
    Afinal[1, 1] <- 1
    Acombs <- array(NA, dim=c(n,n,ncol(combs)))
    
    for (i in 2:n) {
      for(combsIndex in 1:ncol(combs)){
        A <- Afinal
        s <- Parents[combs[1,combsIndex],]
        d <- Parents[combs[2,combsIndex],]
        if (s[i] == 0 && d[i] == 0) {
          A[i, i] <- 1
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0
        }
        if (s[i] == 0 && d[i] != 0) {
          A[i, i] <- 1
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0.5 * 
              (A[j, d[i]])
        }
        if (d[i] == 0 && s[i] != 0) {
          A[i, i] <- 1
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0.5 * 
              (A[j, s[i]])
        }
        if (d[i] != 0 && s[i] != 0) {
          A[i, i] <- 1 + 0.5 * (A[d[i], s[i]])
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0.5 * 
              (A[j, s[i]] + A[j, d[i]])
        }
        Acombs[1:i,1:i,combsIndex] = A[1:i,1:i]
      }
      
      ## Total parents
      totalParents = length(which(Parents[,i]!=0))
      if(totalParents < 3){
        Afinal[i,] = Afinal[,i] = Acombs[i,,1]
      }else{
        Afinal[i,] = Afinal[,i] = apply(Acombs[i,,(1:choose(totalParents,2))],1,mean)
      }
    }
    A = Afinal
  }
  
  if(fixedParent==TRUE){
    n <- ncol(Parents)
    Time = proc.time()
    Afinal = matrix(NA,n,n)
    Afinal[1, 1] <- 1
    Acombs <- array(NA, dim=c(n,n,(nrow(Parents)-1)))
    s <- Parents[1,]   
    for (i in 2:n) {
      for(combsIndex in 1:(nrow(Parents)-1)){
        A <- Afinal
        d <- Parents[combsIndex+1,]
        if (s[i] == 0 && d[i] == 0) {
          A[i, i] <- 1
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0
        }
        if (s[i] == 0 && d[i] != 0) {
          A[i, i] <- 1
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0.5 * 
              (A[j, d[i]])
        }
        if (d[i] == 0 && s[i] != 0) {
          A[i, i] <- 1
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0.5 * 
              (A[j, s[i]])
        }
        if (d[i] != 0 && s[i] != 0) {
          A[i, i] <- 1 + 0.5 * (A[d[i], s[i]])
          for (j in 1:(i - 1)) A[j, i] <- A[i, j] <- 0.5 * 
              (A[j, s[i]] + A[j, d[i]])
        }
        Acombs[1:i,1:i,combsIndex] = A[1:i,1:i]
      }
      
      ## Total parents
      totalParents = length(which(Parents[,i]!=0))
      if(totalParents < 3){
        Afinal[i,] = Afinal[,i] = Acombs[i,,1]
      }else{
        Afinal[i,] = Afinal[,i] = apply(Acombs[i,,1:(totalParents-1)],1,mean)
      }
    }
    A = Afinal
  }
  
  Time = as.matrix(proc.time() - Time)
  cat("Completed! Time =", Time[3]/60, " minutes \n")
  rownames(A) <- colnames(A) <- data1$ind_data
  A <- A[orig.order, orig.order]
  return(A)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

buildA_kerr_cpp <- function(s, d, w, v) {
    .Call(`_AGHmatrix_buildA_kerr_cpp`, s, d, w, v)
}

buildA_ploidy2_cpp <- function(sire, dire, n) {
    .Call(`_AGHmatrix_buildA_ploidy2_cpp`, sire, dire, n)
}

buildA_slater_cpp <- function(s, d, w) {
    .Call(`_AGHmatrix_buildA_slater_cpp`, s, d, w)
}

ascii_to_number <- function(pedigree_data, unk = "0") {
    .Call(`_AGHmatrix_ascii_to_number`, pedigree_data, unk)
}

check_matrix <- function(obj) {
    invisible(.Call(`_AGHmatrix_check_matrix`, obj))
}

datatreat_cpp <- function(data, n_max = 50L, unk = "0", save = FALSE) {
    .Call(`_AGHmatrix_datatreat_cpp`, data, n_max, unk, save)
}

diploid_p0p2_TwoPQ_cpp <- function(M) {
    .Call(`_AGHmatrix_diploid_p0p2_TwoPQ_cpp`, M)
}

buildDominanceMatrix_cpp <- function(A, s, d) {
    .Call(`_AGHmatrix_buildDominanceMatrix_cpp`, A, s, d)
}

Gmatrix_Endelman <- function(M, ploidy) {
    .Call(`_AGHmatrix_Gmatrix_Endelman`, M, ploidy)
}

Gmatrix_MarkersMask <- function(M) {
    .Call(`_AGHmatrix_Gmatrix_MarkersMask`, M)
}

Gmatrix_Slater <- function(M, Frequency, FreqP, NumberMarkers) {
    .Call(`_AGHmatrix_Gmatrix_Slater`, M, Frequency, FreqP, NumberMarkers)
}

Gmatrix_Su <- function(M) {
    .Call(`_AGHmatrix_Gmatrix_Su`, M)
}

Gmatrix_vanraden <- function(M, FreqP, denom) {
    .Call(`_AGHmatrix_Gmatrix_vanraden`, M, FreqP, denom)
}

Gmatrix_VanRaden_weighted <- function(M, weights, FreqP, denom) {
    .Call(`_AGHmatrix_Gmatrix_VanRaden_weighted`, M, weights, FreqP, denom)
}

Gmatrix_vanraden_poly_unweighted <- function(X, ploidy, ratio, ploidy_correction) {
    .Call(`_AGHmatrix_Gmatrix_vanraden_poly_unweighted`, X, ploidy, ratio, ploidy_correction)
}

Gmatrix_vanraden_poly_weighted <- function(X, w, ploidy, ratio, ploidy_correction) {
    .Call(`_AGHmatrix_Gmatrix_vanraden_poly_weighted`, X, w, ploidy, ratio, ploidy_correction)
}

Gmatrix_Vitezica <- function(M, FreqP) {
    .Call(`_AGHmatrix_Gmatrix_Vitezica`, M, FreqP)
}

#' Compute Martini block corrections (H11, H12, H21, H22 - A22)
#'
#' @param A12 n1 x n2 block from A
#' @param A22 n2 x n2 block from A (genotyped)
#' @param G22 n2 x n2 block from G (genotyped)
#' @param tau scalar
#' @param omega scalar
#' @return list(H11, H12, H21, H22corr) where H22corr = H22 - A22
H_martini_blocks <- function(A12, A22, G22, tau, omega) {
    .Call(`_AGHmatrix_H_martini_blocks`, A12, A22, G22, tau, omega)
}

#' Compute var/mean of G by classes of rounded A, and fill varA/meanG
#'
#' @param G square matrix
#' @param A square matrix (same dim as G)
#' @param round_digits integer number of decimal places to round A
#' @return list(varA, meanG, classes, var_by_class, mean_by_class)
munoz_var_mean_by_Aclass <- function(G, A, round_digits) {
    .Call(`_AGHmatrix_munoz_var_mean_by_Aclass`, G, A, round_digits)
}

slater_par_cpp <- function(X, ploidy) {
    .Call(`_AGHmatrix_slater_par_cpp`, X, ploidy)
}

datatreat_legacy <- function(data=NULL,
                      n.max=50,
                      unk=0,
                      save=FALSE
){
  indicator <- k <-  0
  if(is.null(data))
    stop(deparse("Select a data name"))
  
  
  for( i in 1:n.max){
    
    #Data Treatment
    if( i == 1){
      data <- as.matrix(data)
      k <- rep(NA,2) #only for don't stop the loop on the first time
    }
    if( i > 1 )
      data <- new.data
    pedigree <- asciitonumber(data,unk=unk)
    ind.data <- pedigree$ind.data
    sire <- pedigree$sire
    dire <- pedigree$dire
    ind <- c(1:length(sire))
    
    right.pos <- rep(NA, length=length(sire))
    
    #Verify alternatively sire and dire each iteraction+1
    parent <- sire
    parent.ind <- "sire"
    
    if( indicator%%2 == 1 ){
      parent <- dire
      parent.ind <- "dire"
    }
    
    
    for ( j in 1:length(parent)){
      if( is.na(match(parent[j], ind[1:j])) && parent[j] != 0){
        right.pos[j] <- which(ind == parent[j])
      }
    }
    
    error <- c()
    for( j in 1:length(parent))
      if( parent[j] > j)
        error <- c(error,j)
    
    #Print the step point
    if( save ){
      cat( paste("iteraction #",i,parent.ind,"\n",sep=""))
      cat( paste(error,"\n"))
    }
    
    
    #Right positions of the new data
    if( length(error) > 0 ){
      after <- which( !is.na(right.pos))
      before <- right.pos[after]
      for( j in 1:length(before)){
        if( j == 1 || before[j] != before[j-1] ){
          ind[after[j]] <- before[j]
          ind[before[j]] <- after[j]
        }
      }
    }
    new.data <- data[ind,]
    
    
    # Verify changes in the loop
    lastk <- k #indicator that there is no more changes in the last parent
    k <- length(error) #indicator that no changes
    
    
    if (  k == 0 )
      indicator = indicator+1
    
    if ( i != 1 && k == 0 && lastk == 0 ){
      cat("Your data was chronologically organized with success. \n")
      if(save){
        cat(paste("orgnew.txt",sep=""))
        write.table(new.data,  file=paste("orgped.txt",sep=""), quote=FALSE, row.names=FALSE, col.names=FALSE)
      }
      return(pedigree)
    }
    
    if ( i == n.max ){
      cat("Your data was not chronologically organized with sucess. Check your data with missing.data function and/or verify the individuals in the 2 last iteractions above descripted (the number is the row in the file: \n")
      cat(paste("orgped.txt",sep=""))
      write.table(new.data, file=paste("orgped.txt",sep=""), quote=FALSE, row.names=FALSE, col.names=FALSE)
      return(pedigree)
    }
  }
}

asciitonumber_legacy <- function(
    pedigree.data,
    unk=0
){
  if( ncol(pedigree.data) != 3 ){
    print("Data with more than 3 columns, please verify")
    return()
  }
  ind.data <- as.vector(c(unk,pedigree.data[,1]))
  sire.data <- as.vector(pedigree.data[,2])
  dire.data <- as.vector(pedigree.data[,3])
  sire <- match(sire.data, ind.data)
  dire <- match(dire.data, ind.data)
  ind <- as.vector(c(1:length(ind.data)))
  sire <- sire-1
  dire <- dire-1
  ind <- ind[-length(ind)]
  ind.data <- ind.data[-1]
  pedigree <- list(sire=sire,dire=dire,ind.data=pedigree.data[ind,1])
  return(pedigree)
}


sortped<-function(data = NULL, loop.in = 1000, loop.between = 100, print = FALSE)
{   
  if (is.null(data)) 
    stop(deparse("Please define the variable data"))
  
  stop.loop.1 <- stop.loop.2 <- FALSE
  
  for(j in 1:loop.between){
    if(print) cat(paste("looping between...",print(j)))
    
    for(i in 1:loop.in){
      if(print) cat(paste("looping in first parent...",print(i)))
      ind<-data[,1]
      sire<-data[,2]
      dire<-data[,3]
      index<-1:length(ind)
      compare<-match(dire,ind)
      compare[which(is.na(compare))]<-0
      loop<-which(compare>index)
      newindex<-index
      newindex[loop[1]]<-compare[loop[1]]
      newindex[compare[loop[1]]]<-loop[1]
      data<-data[newindex,]
      if(print) print(length(loop))
      if( length(loop) == 0 && i == 1)
        stop.loop.1 <- TRUE
      if(length(loop)==0) break
    }
    
    for(i in 1:loop.in){
      if(print) cat(paste("looping in second parent...",print(i))) 
      ind<-data[,1]
      sire<-data[,2]
      dire<-data[,3]
      index<-1:length(ind)
      compare<-match(sire,ind)
      compare[which(is.na(compare))]<-0
      loop<-which(compare>index)
      newindex<-index
      newindex[loop[1]]<-compare[loop[1]]
      newindex[compare[loop[1]]]<-loop[1]
      data<-data[newindex,]
      if(print) print(length(loop))
      if( length(loop) == 0 && i == 1)
        stop.loop.2 <- TRUE
      if( length(loop) == 0 ) break
    }
    
    if( stop.loop.1 && stop.loop.2) break
  }
  return(data)
}


test_that("Both versions of asciitonumber and datatreat give same output", {
  # Test data
  pedigree <- matrix(c(
    "A", "B", "C",
    "B", "D", "E",
    "C", "F", "G",
    "D", "0", "0",
    "E", "0", "0",
    "F", "0", "0",
    "G", "0", "0"
  ), ncol = 3, byrow = TRUE)
  
  # Set colnames (optional, for clarity)
  colnames(pedigree) <- c("id", "sire", "dam")
  
  # Run legacy R version
  ped_sorted_legacy <- sortped(pedigree)
  res_legacy <- asciitonumber_legacy(ped_sorted_legacy)
  
  # Run C++ version
  ped_sorted_cpp <- datatreat_cpp(pedigree, save = FALSE)
  res_cpp <- ascii_to_number(ped_sorted_legacy)
  
  # Compare sire
  expect_equal(res_cpp$sire, res_legacy$sire)
  
  # Compare dam
  expect_equal(res_cpp$dire, res_legacy$dire)
  
  # Compare individual names
  expect_equal(as.character(res_cpp$ind_data), as.character(res_legacy$ind.data))
})

test_that("Second test for pedigree", {
  pedigree <- matrix(c(
    "A", "B", "C",
    "B", "D", "E",
    "C", "F", "G",
    "D", "0", "0",
    "E", "0", "0",
    "F", "0", "0",
    "G", "0", "0"
  ), ncol = 3, byrow = TRUE)
  colnames(pedigree) <- c("id", "sire", "dam")
  
  # Legacy
  ped_sorted_legacy <- sortped(pedigree)
  res_legacy <- asciitonumber_legacy(ped_sorted_legacy)
  
  # C++
  ped_sorted_cpp <- datatreat_cpp(pedigree, save = FALSE)
  res_cpp <- ascii_to_number(ped_sorted_legacy)
  
  expect_equal(res_cpp$sire, res_legacy$sire)
  expect_equal(res_cpp$dire, res_legacy$dire)
  expect_equal(as.character(res_cpp$ind_data), as.character(res_legacy$ind.data))
})


test_that("Handles missing parents as '0' or NA", {
  pedigree <- matrix(c(
    "A", "B", "C",
    "B", "D", NA,
    "C", "0", "G",
    "D", NA, "0",
    "E", "0", "0",
    "F", NA, NA,
    "G", "0", "0"
  ), ncol = 3, byrow = TRUE)
  colnames(pedigree) <- c("id", "sire", "dam")
  
  # Convert NAs to "0" to match unknown handling
  pedigree[is.na(pedigree)] <- "0"
  
  ped_sorted_legacy <- sortped(pedigree)
  res_legacy <- asciitonumber_legacy(ped_sorted_legacy)
  ped_sorted_cpp <- datatreat_cpp(pedigree, save = FALSE)
  res_cpp <- ascii_to_number(ped_sorted_legacy)
  
  expect_equal(res_cpp$sire, res_legacy$sire)
  expect_equal(res_cpp$dire, res_legacy$dire)
  expect_equal(as.character(res_cpp$ind_data), as.character(res_legacy$ind.data))
})


test_that("Unordered pedigree gets sorted correctly", {
  pedigree <- matrix(c(
    "E", "F", "G",
    "A", "B", "C",
    "B", "0", "0",
    "C", "0", "0",
    "F", "0", "0",
    "D", "E", "0",
    "G", "0", "0"
  ), ncol = 3, byrow = TRUE)
  colnames(pedigree) <- c("id", "sire", "dam")
  
  ped_sorted_legacy <- sortped(pedigree)
  res_legacy <- asciitonumber_legacy(ped_sorted_legacy)
  ped_sorted_cpp <- datatreat_cpp(pedigree, save = FALSE)
  res_cpp <- ascii_to_number(ped_sorted_legacy)
  
  expect_equal(res_cpp$sire, res_legacy$sire)
  expect_equal(res_cpp$dire, res_legacy$dire)
  expect_equal(as.character(res_cpp$ind_data), as.character(res_legacy$ind.data))
})


test_that("Large pedigree works the same", {
  set.seed(42)
  ids <- paste0("ID", 1:100)
  parents <- sample(c(ids, "0"), size = 200, replace = TRUE)
  pedigree <- matrix(cbind(ids, parents[1:100], parents[101:200]), ncol = 3)
  colnames(pedigree) <- c("id", "sire", "dam")
  
  # Clean unknowns if needed
  pedigree[is.na(pedigree)] <- "0"
  
  ped_sorted_legacy <- sortped(pedigree)
  res_legacy <- asciitonumber_legacy(ped_sorted_legacy)
  ped_sorted_cpp <- datatreat_cpp(pedigree, save = FALSE)
  res_cpp <- ascii_to_number(ped_sorted_legacy)
  
  expect_equal(res_cpp$sire, res_legacy$sire)
  expect_equal(res_cpp$dire, res_legacy$dire)
  expect_equal(as.character(res_cpp$ind_data), as.character(res_legacy$ind.data))
})


test_that("Missing parent not in ID column still handled", {
  pedigree <- matrix(c(
    "A", "X", "Y",
    "B", "A", "Z",
    "C", "B", "0"
  ), ncol = 3, byrow = TRUE)
  colnames(pedigree) <- c("id", "sire", "dam")
  pedigree[is.na(pedigree)] <- "0"
  
  ped_sorted_legacy <- sortped(pedigree)
  res_legacy <- asciitonumber_legacy(ped_sorted_legacy)
  ped_sorted_cpp <- datatreat_cpp(pedigree, save = FALSE)
  res_cpp <- ascii_to_number(ped_sorted_legacy)
  
  expect_equal(res_cpp$sire, res_legacy$sire)
  expect_equal(res_cpp$dire, res_legacy$dire)
  expect_equal(as.character(res_cpp$ind_data), as.character(res_legacy$ind.data))
})

test_that("Duplicated IDs are caught or warned in the new C++", {
  pedigree <- matrix(c(
    "A", "B", "C",
    "B", "D", "E",
    "A", "F", "G"  # duplicate "A"
  ), ncol = 3, byrow = TRUE)
  colnames(pedigree) <- c("id", "sire", "dam")
  
  # C++ version should stop with an error
  expect_error({
    datatreat_cpp(pedigree)
  }, regexp = "Duplicate individual ID found", ignore.case = TRUE)
})

#########################################
# 									
# Package: AGHmatrix 							
# 									
# File: Mcheck.R							
# Contains: Mcheck							
# 									
# Written by Luis F V Ferrao and Rodrigo Amadeu		
# 									
# First version: Feb-2014 						
# Last update: 30-Mar-2023 						
# License: GPL-3
# 									
#########################################

#' Check and filter markers
#'
#' This function does different filtering on the marker matrix
#'
#' @param SNPmatrix matrix (n x m), where n is is individual names and m is marker names (coded inside the matrix as 0, 1, 2, ..., ploidy, and, missingValue). 
#' @param ploidy data ploidy (an even number between 2 and 20). Default=2.
#' @param missingValue missing value in data. Default=-9.
#' @param thresh.missing threshold on missing data, SNPs below of this frequency value will be maintained, if equal to 1, no threshold and imputation is considered. Default = 0.50.
#' @param thresh.maf minimum allele frequency accepted to each marker. Default=0.05.
#' @param thresh.htzy threshold heterozigosity, remove SNPs below this threshold. Default=0.
#' @param impute.method "mean" to impute the missing data by the mean per marker, "mode" to impute the missing data by the mode per marker, "global.mean" to impute the missing data by the mean across all markers, "global.mode" to impute the missing data my the mode across all marker. Default = "mean".
#' @param rmv.mono if monomorphic markers should be removed. Default=TRUE.
#'
#' @return SNPmatrix after filtering steps.
#'
#' @importFrom stats var
#' @importFrom matrixStats colVars
#' @examples 
#' data(snp.pine)
#' M = Mcheck(snp.pine)
#'
#' @author Luis F V Ferrao
#' @author Rodrigo Amadeu, \email{rramadeu@@gmail.com}
#' @author Thiago de Paula Oliveira \email{toliveira@abacusbio.com}
#'
#' @export
#' 
Mcheck <- function(SNPmatrix = NULL,
                   ploidy = 2,
                   missingValue = -9,
                   thresh.maf = 0.05,
                   thresh.missing = 0.9,
                   thresh.htzy = 0, 
                   impute.method = "mean",
                   rmv.mono = TRUE) {
  
  #-----------------------------------------------------------------------------
  # SNP missing data
  #-----------------------------------------------------------------------------
  ncol.init <- ncol(SNPmatrix)
  
  if (!is.na(missingValue)) {
    m <- match(SNPmatrix, missingValue, 0)
    SNPmatrix[m > 0] <- NA
  }
  
  #-----------------------------------------------------------------------------
  # Faster missing rate calculation
  #-----------------------------------------------------------------------------
  missing <- colMeans(is.na(SNPmatrix))
  missing.low <- missing <= thresh.missing
  cat("\nMissing data check: \n")
  if (any(missing.low)) {
    cat("\tTotal SNPs:", ncol(SNPmatrix), "\n")
    cat("\t", ncol(SNPmatrix) - sum(missing.low), 
        "SNPs dropped due to missing data threshold of", 
        thresh.missing, "\n")
    cat("\tTotal of:", sum(missing.low), " SNPs \n")
    idx.rm <- which(missing.low)
    SNPmatrix <- SNPmatrix[, idx.rm, drop = FALSE]
  } else {
    cat("\tNo SNPs with missing data, missing threshold of =", 
        thresh.missing, "\n")
  }
  
  #-----------------------------------------------------------------------------
  # Minor allele frequency
  #-----------------------------------------------------------------------------
  AF <- colMeans(SNPmatrix, na.rm = TRUE) / ploidy
  MAF <- ifelse(AF > 0.5, 1 - AF, AF)
  snps.low <- MAF < thresh.maf
  cat("\nMAF check: \n")
  if (any(snps.low)) {
    cat("\t", sum(snps.low), "SNPs dropped with MAF below", thresh.maf, "\n")
    cat("\tTotal:", ncol(SNPmatrix) - sum(snps.low), " SNPs \n")
    idx.rm <- which(snps.low)
    SNPmatrix <- SNPmatrix[, -idx.rm, drop = FALSE]
  } else {
    cat("\tNo SNPs with MAF below", thresh.maf, "\n")
  }
  
  #-----------------------------------------------------------------------------
  # Monomorphic SNPs
  #-----------------------------------------------------------------------------
  if (rmv.mono) {
    mono <- matrixStats::colVars(SNPmatrix, na.rm = TRUE) == 0
    mono[is.na(mono)] <- TRUE
    cat("\nMonomorphic check: \n")
    if (any(mono)) {
      cat("\t", sum(mono), "monomorphic SNPs \n")
      cat("\tTotal:", ncol(SNPmatrix) - sum(mono), "SNPs \n")
      idx.rm <- which(mono)
      SNPmatrix <- SNPmatrix[, -idx.rm, drop = FALSE]
    } else {
      cat("\tNo monomorphic SNPs \n")
    }
  }
  
  #-----------------------------------------------------------------------------
  # Imputation
  #-----------------------------------------------------------------------------
  if (impute.method == "global.mean") {
    ix <- which(is.na(SNPmatrix))
    if (length(ix) > 0) {
      SNPmatrix[ix] <- mean(SNPmatrix, na.rm = TRUE)
    }
  }
  
  if (impute.method == "global.mode") {
    ix <- which(is.na(SNPmatrix))
    if (length(ix) > 0) {
      vals <- SNPmatrix[!is.na(SNPmatrix)]
      mode_val <- as.integer(names(which.max(table(vals))))
      SNPmatrix[ix] <- mode_val
    }
  }
  
  if (impute.method == "mean") {
    imp_vals <- colMeans(SNPmatrix, na.rm = TRUE)
    ix <- which(is.na(SNPmatrix), arr.ind = TRUE)
    SNPmatrix[ix] <- imp_vals[ix[, 2]]
  }
  
  if (impute.method == "mode") {
    mode_fun <- function(x) {
      x <- x[!is.na(x)]
      as.integer(names(which.max(table(x))))
    }
    imp_vals <- apply(SNPmatrix, 2, mode_fun)
    ix <- which(is.na(SNPmatrix), arr.ind = TRUE)
    SNPmatrix[ix] <- imp_vals[ix[, 2]]
  }
  
  #-----------------------------------------------------------------------------
  # Heterozygosity
  #-----------------------------------------------------------------------------
  htrz <- 
    colSums(SNPmatrix != 0 & SNPmatrix != ploidy, na.rm = TRUE) / 
    nrow(SNPmatrix)
  htrz.low <- htrz < thresh.htzy
  cat("\nHeterozigosity data check: \n")
  if (any(htrz.low)) {
    cat("\tTotal SNPs:", ncol(SNPmatrix), "\n")
    cat("\t", ncol(SNPmatrix) - sum(htrz.low), 
        "SNPs dropped due to heterozygosity threshold of", thresh.htzy, "\n")
    cat("\tTotal of:", sum(htrz.low), " SNPs \n")
    idx.rm <- which(htrz.low)
    SNPmatrix <- SNPmatrix[, idx.rm, drop = FALSE]
  } else {
    cat("\tNo SNPs with heterozygosity, missing threshold of =", 
        thresh.htzy, "\n")
  }
  
  #-----------------------------------------------------------------------------
  # Summary
  #-----------------------------------------------------------------------------
  cat("\nSummary check: \n")
  cat("\tInitial: ", ncol.init, "SNPs \n")
  cat("\tFinal: ", ncol(SNPmatrix), " SNPs (", ncol.init - ncol(SNPmatrix), 
      " SNPs removed) \n\n")
  
  return(SNPmatrix)
}

## ----knitr_init, echo=FALSE, cache=FALSE--------------------------------------
library(knitr)
library(rmarkdown)

knitr::opts_chunk$set(collapse = TRUE,
                      comment = "#>",
                      fig.width = 7,
                      fig.height = 8,
                      fig.align = "center",
                      dev = "png",
                      dpi = 72,
                      cache = TRUE)


## ----echo=FALSE, results='hide'-----------------------------------------------
library(AGHmatrix)

## ----eval=FALSE---------------------------------------------------------------
# ## Install stable version
# install.packages("AGHmatrix")
# 
# ## Install development version
# install.packages("devtools")
# devtools::install_github("rramadeu/AGHmatrix")
# 
# ## Load
# library(AGHmatrix)

## -----------------------------------------------------------------------------
data(ped.mrode)
ped.mrode
str(ped.mrode) #check the structure

## ----eval=FALSE---------------------------------------------------------------
# #Computing additive relationship matrix for diploids (Henderson 1976):
# Amatrix(ped.mrode, ploidy=2)
# 
# #Computing dominant relationship matrix for diploids (Cockerham 1954):
# Amatrix(ped.mrode, ploidy=2, dominance=TRUE)
# 
# #Computing additive relationship matrix for autotetraploids (Kerr 2012):
# Amatrix(ped.mrode, ploidy=4)
# 
# #Computing additive relationship matrix for autooctaploids (Kerr 2012):
# Amatrix(ped.mrode, ploidy=8)
# 
# #Computing additive relationship matrix for autotetraploids
# # and double-reduction of 0.1 (Kerr 2012):
# Amatrix(ped.mrode, ploidy=4, w=0.1)
# 
# #Computing additive relationship matrix for autotetraploids
# # and double-reduction of 0.1 as in Slater et al. (2014):
# Amatrix(ped.mrode, ploidy=4, w=0.1, slater = TRUE)
# #not recommended, but kept in the package to reproduce some former analysis
# 
# #Computing additive relationship matrix for autohexaploids
# # and double-reduction of 0.1 (Kerr 2012):
# Amatrix(ped.mrode, ploidy=6, w=0.1)

## ----eval=FALSE---------------------------------------------------------------
# ?Amatrix

## -----------------------------------------------------------------------------
data(snp.pine)
snp.pine[1:5,1:5]
str(snp.pine)

## ----eval=FALSE---------------------------------------------------------------
# #Computing the additive relationship matrix based on VanRaden 2008
# G_VanRadenPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                           maf=0.05, method="VanRaden")
# 
# #Computing the additive relationship matrix based on Yang 2010
# G_YangPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                       maf=0.05, method="Yang")
# 
# #Computing the dominance relationship matrix based on Su 2012
# G_SuPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                     maf=0.05, method="Su")
# 
# #Computing the dominance relationship matrix based on Vitezica 2013
# G_VitezicaPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9,
#                           maf=0.05, method="Vitezica")

## ----eval=FALSE---------------------------------------------------------------
# ?Gmatrix

## ----eval=FALSE---------------------------------------------------------------
# #Loading the data
# data(snp.sol)
# str(snp.sol)
# 
# #Computing the additive relationship matrix based on VanRaden 2008
# # adapted by Ashraf 2016
# G_VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)
# 
# #Computing the dominance (digenic) matrix based on Endelman 2018 (Eq. 19)
# G_Dominance <- Gmatrix(snp.sol, method="Endelman", ploidy=4)
# 
# #Computing the full-autopolyploid matrix based on Slater 2016 (Eq. 8
# #and 9)
# G_FullAutopolyploid <- Gmatrix(snp.sol, method="Slater", ploidy=4)
# 
# #Computing the pseudodiploid matrix based on Slater 2016 (Eq. 5, 6,
# #and 7)
# G_Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE)
# 
# #Computing G matrix with specific weight for each marker as
# # in Liu et al. (2020).
# Gmatrix_weighted <- Gmatrix(snp.sol, method="VanRaden", weights = runif(3895,0.001,0.1), ploidy=4)

## ----eval=FALSE---------------------------------------------------------------
# ?Gmatrix

## ----eval=FALSE---------------------------------------------------------------
# #Loading the data
# library(AGHmatrix)
# data(snp.sol)
# snp.sol.ratio = snp.sol/4 #transforming it in a ratio of the minor allele frequency
# Gmatrix <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, ratio=FALSE)
# Gmatrix.ratio <- Gmatrix(snp.sol.ratio, method="VanRaden", ploidy=4, ratio=TRUE)
# Gmatrix[1:5,1:5]==Gmatrix.ratio[1:5,1:5]
# 
# ## it also has the ploidy.correction option
# Gmatrix.alternative <- Gmatrix(snp.sol,
#                                method="VanRaden",
#                                ploidy=4,
#                                ratio=FALSE,
#                                ploidy.correction=TRUE)
# 
# Gmatrix.ratio.alternative <- Gmatrix(snp.sol.ratio,
#                                      method="VanRaden",
#                                      ploidy=4,
#                                      ratio=TRUE,
#                                      ploidy.correction=TRUE)
# Gmatrix[1:5,1:5]==Gmatrix.alternative[1:5,1:5]
# Gmatrix.alternative[1:5,1:5]==Gmatrix.ratio.alternative[1:5,1:5]

## ----eval=FALSE---------------------------------------------------------------
# data(ped.sol)
# data(snp.sol)
# 
# #Computing the numerator relationship matrix 10% of double-reduction
# Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)
# Gmat <- Gmatrix(snp.sol, ploidy=4,
#                 maf=0.05, method="VanRaden")
# Gmat <- round(Gmat,3) #see appendix
# 
# #Computing H matrix (Martini)
# Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini",
#                         ploidy=4, missingValue=-9, maf=0.05)
# 
# #Computing H matrix (Munoz)
# Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol,
#                       ploidy=4, method="Munoz",
#                       missingValue=-9, maf=0.05)

## ----eval=FALSE---------------------------------------------------------------
# data(snp.pine)
# A <- Gmatrix(SNPmatrix=snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
# D <- Gmatrix(SNPmatrix=snp.pine, method="Vitezica", missingValue=-9,maf=0.05)

## ----eval=FALSE---------------------------------------------------------------
# #Additive-by-Additive Interactions
# A_A <- A*A
# #Dominance-by-Additive Interactions
# D_A <- D*A
# #Dominance-by-Dominance Interactions
# D_D <- D*D

## ----eval=FALSE---------------------------------------------------------------
# #Additive-by-Additive-by-Additive Interactions
# A_A_A <- A*A*A
# #Additive-by-Additive-by-Dominance Interactions
# A_A_D <- A*A*D
# #Additive-by-Dominance-by-Dominance Interactions
# A_D_D <- A*D*D
# #Dominance-by-Dominance-by-Dominance Interactions
# D_D_D <- D*D*D

## ----eval=FALSE---------------------------------------------------------------
# #Loading the data example
# data(ped.mrode)
# 
# #Computing the matrix
# A <- Amatrix(data=ped.mrode, ploidy=4, w=0.1)
# 
# #Building its inverse
# Ainv <- solve(A)
# 
# #Exporting it. The function "formatmatrix"
# # will convert it and save in your working directory
# formatmatrix(Ainv, round.by=12, exclude.0=TRUE, name="Ainv")

## -----------------------------------------------------------------------------
pedigree = data.frame(id=1:8,
                      parent1 = c(0,0,0,0,1,2,3,5),
                      parent2 = c(0,0,0,0,2,3,4,6),
                      parent3 = c(0,0,0,0,3,4,0,7),
                      parent4 = c(0,0,0,0,0,0,0,1),
                      parent5 = 0)

print(pedigree)
AmatrixPolyCross(pedigree)

## -----------------------------------------------------------------------------
AmatrixPolyCross(pedigree,fixedParent=TRUE)

## ----eval=TRUE,echo=FALSE-----------------------------------------------------
x = c(1000,5000,10000,20000,30000,40000,50000,60000,70000,80000,90000,100000)/1000 #Pedigree Size
y = c(252156,622500,1795260,6481064,14313448,25227680,49081224,70622336,96017144,125320048,158444856,194731908)/1e+6 #RAM GB 
ytime = c(0.0025, 0.080, 0.2, 0.89, 1.62,3.01,4.52,7.12,9.15,13.13,15.13,20) #minutes
df = data.frame(size=x,ram=y,time=ytime)

plot(x=df$size,y=df$ram, 
     ylab = "RAM (GB) at the peak of Amatrix() function",
     xlab = "Pedigree size (in 1,000 rows)",
     type="b",
     axes=FALSE)
axis(side = 2, at = c(0,4,8,16,32,48,64,96,144,192),cex.axis=.75)
axis(side = 1, at = c(1,5,10,20,30,40,50,60,70,80,90,100),cex.axis=.75)

plot(x=df$size,y=df$time, type="b",
     ylab = "Time to run (minutes) the Amatrix() function",
     xlab = "Pedigree size (in 1,000 rows)",
     axes=FALSE)
axis(side = 2, at = seq(0,20,2),cex.axis=.75)
axis(side = 1, at = c(1,5,10,20,30,40,50,60,70,80,90,100),cex.axis=.75)

## ----eval=FALSE,echo=FALSE----------------------------------------------------
# #To knit an this vignette into an .R file
# knitr::purl("vignettes/Tutorial_AGHmatrix.Rmd")

## -----------------------------------------------------------------------------
sessionInfo()

---
title: "AGHmatrix: An R package to compute relationship matrices for diploid and autopolyploid species"
author: "Rodrigo Amadeu"
date: "`r Sys.Date()`"
output:
   pdf_document:
     latex_engine: xelatex
     toc: TRUE
vignette: >
  %\VignetteIndexEntry{AGHmatrix Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r knitr_init, echo=FALSE, cache=FALSE}
library(knitr)
library(rmarkdown)

knitr::opts_chunk$set(collapse = TRUE,
                      comment = "#>",
                      fig.width = 6,
                      fig.height = 6,
                      fig.align = "center",
                      dev = "png",
                      dpi = 36,
                      cache = TRUE)

```

```{r, echo=FALSE, results='hide'}
library(AGHmatrix)
```

## Contact
Rodrigo R Amadeu  
rramadeu at ufl dot edu  
Horticultural Sciences Department  
University of Florida  
https://rramadeu.github.io

## Overview
AGHmatrix software is an R-package to build relationship matrices using pedigree (A matrix) and/or molecular markers (G matrix) with the possibility to build a combined matrix of Pedigree corrected by Molecular (H matrix). The package works with diploid and autopolyploid Data. If you are not familiar with `R`, we recommend the reading of vignette [Introduction to R](http://htmlpreview.github.io/?https://github.com/augusto-garcia/onemap/blob/master/inst/doc/Introduction_R.html).

## Matrices computation implemented in the `AGHmatrix` 
Currently can compute the following 14 different relationship matrices:

### Pedigree-based relationship matrix (A matrix)

|               |&nbsp; &nbsp; &nbsp; &nbsp; | Additive                  |&nbsp; &nbsp; &nbsp; &nbsp; |Non-Additive                |
|---------------|--------------|:-------------------------:|--------------|:--------------------------:|
| **Diploid**   |&nbsp; &nbsp; &nbsp; &nbsp; | Henderson (1976)          |&nbsp; &nbsp; &nbsp; &nbsp; |Cockerham (1954)            |
| **Polyploid** |&nbsp; &nbsp; &nbsp; &nbsp; | Kerr (2012), Slater (2013)|&nbsp; &nbsp; &nbsp; &nbsp; ||                            |

 
### Molecular-based relationship matrix (G matrix)

|           |&nbsp; &nbsp; &nbsp; &nbsp; |  Additive                     |&nbsp; &nbsp; &nbsp; &nbsp; | Non-Additive               |
|-----------|--------------|:----------------------------:|----|:---------------------------:|
| **Diploid**   |&nbsp; &nbsp; &nbsp; &nbsp; | Yang (2010), VanRaden (2012) |&nbsp; &nbsp; &nbsp; &nbsp; | Su (2012), Vitezica (2013) |
| **Polyploid** |&nbsp; &nbsp; &nbsp; &nbsp; | Slater (2016), VanRaden (2012) |&nbsp; &nbsp; &nbsp; &nbsp; | Slater (2016), Endelman (2018)              |


### Combined pedigree and molecular-based relationship matrix (H matrix)

| **Any ploidy/effect**   |
|:---------------------------:|
| Munoz (2014), Martini (2018) |
</center>

## Citation
To cite this R package:

Amadeu, R. R., C. Cellon, J. W. Olmstead, A. A. F. Garcia, M. F. R. Resende, and P. R. Munoz. 2016. AGHmatrix: R Package to Construct Relationship Matrices for Autotetraploid and Diploid Species: A Blueberry Example. *The Plant Genome* 9(3). [doi: 10.3835/plantgenome2016.01.0009](http://dx.doi.org/10.3835/plantgenome2016.01.0009)

## Installing and loading
Within R:
```{r, eval=FALSE}
## Install stable version
install.packages("AGHmatrix")

## Install development version
#install.packages("devtools")
#devtools::install_github("prmunoz/AGHmatrix")

## Load 
library(AGHmatrix)
```

## Relationship matrices using pedigree data - A matrix
`Amatrix` process the pedigree and build the A-matrix related to that given pedigree. The matrix is build according to the recursive method presented in Mrode (2014) and described by Henderson (1976). This method is expanded for higher ploidies (n-ploidy) according with Kerr et al. (2012). After loading the package you have to load your data file into the software. To do this, you can use the function `read.data()` or `read.csv()` for example. Your data should be available in R as a `data.frame` structure in the following order: column 1 must be the individual/genotype names (id), columns 2 and 3 must be the parent names. For the algorhitm does not matter who is the mom and who is the dad. In the package there is a pedigree data example (`ped.mrode`) that can be used to look at the structure and order the data. 

To load `ped.mrode`:

```{r}
data(ped.mrode)
ped.mrode
str(ped.mrode) #check the structure
```

The example `ped.mrode` has 3 columns, column 1 contains the names of the individual/genotypes, column 2 contains the names of the first parent, column 3 contains the names of the second parental. There is no header and the unknown value must be equal 0. Your data has to be in the same format of `ped.mrode`. Internally in the algorithm, the first step is the pre-processing of the pedigree: the individuals are numerated $1$ to $n$. Then, it is verified whether the genotypes in the pedigree are in chronological order (i.e. if the parents of a given individual are located prior to this individual in the pedigree dataset). If this order is not followed, the algorithm performs the necessary permutations to correct them. After this pre-processing, the matrices computation proceeds as in Mrode (2014) for diploid - for additive or dominance relationship - and as in Kerr et al. (2012) for autotetraploids - for additive relationship. For autotetraploids there is the option to include double-reduction fraction (as presented Slater et al., 2014). For diploids there is the option to compute the non-additive relationship matrix (Cockerham, 1954).

It follows some usage examples with the `ped.mrode`.
```{r, eval=FALSE}
#Computing additive relationship matrix for diploids:
Amatrix(ped.mrode, ploidy=2)

#Computing non-additive relationship matrix considering diploidy:
Amatrix(ped.mrode, ploidy=2, dominance=TRUE)

#Computing additive relationship matrix considering autotetraploidy:
Amatrix(ped.mrode, ploidy=4)

#Computing additive relationship matrix considering autooctaploidy:
Amatrix(ped.mrode, ploidy=8)

#Computing additive relationship matrix considering autotetraploidy 
# and double-reduction of 10%:
Amatrix(ped.mrode, ploidy=4, w=0.1)

#Computing additive relationship matrix considering autotetraploidy
# and double-reduction of 10% as Slater et al. (2014):
Amatrix(ped.mrode, ploidy=4, w=0.1, slater = TRUE)

#Computing additive relationship matrix considering autohexaploidy
# and double-reduction of 10%:
Amatrix(ped.mrode, ploidy=6, w=0.1)
```

More information about `Amatrix` can be found with:
```{r, eval=FALSE}
?Amatrix
```

## Relationship matrices using the molecular data - G matrix for diploids
`Gmatrix` handles the molecular-markers matrix and builds the relationship matrix. Molecular markers data should be organized in a matrix format (individuals in rows and markers in columns) coded as 0,1,2 and missing data value (numeric or `NA`). Import your molecular marker data into `R` with the function `read.table()` or `read.csv()` and convert to a matrix format with the function `as.matrix()`. The function `Gmatrix` can be used to construct the additive relationship either as proposed by Yang et al. (2010) or the proposed by VanRaden (2008). The function can also construct the dominance relationship matrix either as proposed by Su et al. (2012) or as proposed by Vitezica et al. (2013). As an example, here we build the four matrices using real data from Resende et al. (2012).

To load `snp.pine` and to check its structure:
```{r}
data(snp.pine)
snp.pine[1:5,1:5]
str(snp.pine)
```

In this dataset we have 926 individuals with 4853 markers and the missing data value is `-9`.

It follows some usage examples with the `snp.pine` data where the unknown value (`missingValue`) is `-9`. Here we set minimum allele frequency to `0.05`.

```{r, eval=FALSE}
#Computing the additive relationship matrix based on VanRaden 2008
G_VanRadenPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                          maf=0.05, method="VanRaden")

#Computing the additive relationship matrix based on Yang 2010
G_YangPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                      maf=0.05, method="Yang")

#Computing the dominance relationship matrix based on Su 2012
G_SuPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                    maf=0.05, method="Su")

#Computing the dominance relationship matrix based on Vitezica 2013
G_VitezicaPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                          maf=0.05, method="Vitezica")
```

More information about `Gmatrix` can be found with:
```{r, eval=FALSE}
?Gmatrix
```

## Relationship matrices using the molecular data - G matrix for autopolyploids
Molecular markers data should be organized in a matrix format (individual in rows and markers in columns) coded according to the dosage level: 0,1,2,...,ploidy level, and missing data value (numeric or `NA`). As an example, an autotetraploid should be coded as 0,1,2,3,4, and missing data value. In autopolyploids, the function `Gmatrix` can be used to construct: i) the additive relationship based on VanRaden (2008) and extended by Ashraf (2016); ii) the full-autopolyploid including additive and non-additive model as equations 8 and 9 in Slater et al. (2016); iii) the pseudo-diploid model as equations 5, 6, and 7 Slater et al. (2016). iv) the digenic-dominant model based on Endelman et al. (2018). As an example, here we build the matrices using data from Endelman et al. (2018) (`snp.sol`).

```{r, eval=FALSE}
#Loading the data
data(snp.sol)
str(snp.sol)

#Computing the additive relationship matrix based on VanRaden 2008
# adapted by Ashraf 2016
G_VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)

#Computing the dominance (digenic) matrix based on Endelman 2018 (Eq. 19)
G_Dominance <- Gmatrix(snp.sol, method="Endelman", ploidy=4)

#Computing the full-autopolyploid matrix based on Slater 2016 (Eq. 8
# and 9)
G_FullAutopolyploid <- Gmatrix(snp.sol, method="Slater", ploidy=4)

#Computing the pseudodiploid matrix based on Slater 2016 (Eq. 5, 6, 
# and 7)
G_Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE) 
```

More information about `Gmatrix` can be found with:
```{r, eval=FALSE}
?Gmatrix
```

## Combined relationship matrix - H matrix
H matrix is the relationship matrix using combined information from the pedigree and genomic relationship matrices. First, you need to compute the matrices separated and then use them as input to build the combined H matrix. Two methods are implemented. `Munoz` shirinks the G matrix towards the A matrix scaling the molecular relatadness by each relationship classes. `Martini` is a modified version from Legarra et al. 2009 where combines A and G matrix using scaling factors. As an example, here we build the matrices using data from Endelman et al. (2018) (`ped.sol` and `snp.sol`).

```{r, eval=FALSE}
data(ped.sol)
data(snp.sol)

#Computing the numerator relationship matrix 10% of double-reduction
Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)

#Computing the additive relationship matrix based on VanRaden (modified)
Gmat <- Gmatrix(snp.sol, ploidy=4, missingValue=-9, 
                maf=0.05, method="VanRaden")

#Computing H matrix (Martini)
Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini", 
                        ploidy=4, missingValue=-9, maf=0.05)

#Computing H matrix (Munoz)
Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol, 
                      ploidy=4, method="Munoz", 
                      missingValue=-9, maf=0.05)
```

## Covariance matrices due to epistatic terms
Here we present how to compute the epistasis relationship matrices using Hadamard
products (i.e. cell-by-cell product), denoted by `*`. For more information please see Munoz
et al. (2014). In this example we are using the molecular-based relationship matrices. First, build the additive and dominance matrices:

```{r, eval=FALSE}
data(snp.pine)
A <- Gmatrix(SNPmatrix=snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
D <- Gmatrix(SNPmatrix=snp.pine, method="Vitezica", missingValue=-9,maf=0.05)
```

For the first degree epistatic terms:
```{r, eval=FALSE}
#Additive-by-Additive Interactions
A_A <- A*A
#Dominance-by-Additive Interactions
D_A <- D*A
#Dominance-by-Dominance Interactions
D_D <- D*D
```

For the second degree epistatic terms:
```{r, eval=FALSE}
#Additive-by-Additive-by-Additive Interactions
A_A_A <- A*A*A
#Additive-by-Additive-by-Dominance Interactions
A_A_D <- A*A*D
#Additive-by-Dominance-by-Dominance Interactions
A_D_D <- A*D*D
#Dominance-by-Dominance-by-Dominance Interactions
D_D_D <- D*D*D
```

And so on...

## Exporting your matrix as three columns and sparse format (ASReml - csv format)
That is the lower diagonal matrix formatted in three columns in .csv format (other ASCII extension could be used as well). In order to do this, we need to build a matrix, its inverse, and export it using `formatmatrix` function. ASReml can invert the relationship matrix as well, probably more efficiently than R for large matrices (i.e. `solve()` function), so no need to invert the matrix in R if matrix is large. This function has as options: `round.by`, which let you decide the number of decimals you want; `exclude.0`, if `TRUE`, remove all the zeros from your data (i.e., transforms into sparse); and, name that defines the name to be used in the exported file. Use the default if not sure what parameter use in these function. Here an example using `ped.mrode` data:

```{r, eval=FALSE}
#Loading the data example
data(ped.mrode)

#Computing the matrix
A <- Amatrix(data=ped.mrode, ploidy=4, w=0.1)

#Building its inverse
Ainv <- solve(A)

#Exporting it. The function "formatmatrix" 
# will convert it and save in your working directory
formatmatrix(Ainv, round.by=12, exclude.0=TRUE, name="Ainv")
```

## Bibliography
Amadeu, RR, et al., 2016 AGHmatrix: R package to construct relationship matrices for autotetraploid and diploid species: a blueberry example. *The Plant Genome* 9(4). https://doi.org/10.3835/plantgenome2016.01.0009

Ashraf, BH, et a., 2016 Estimating genomic heritabilities at the level of family-pool samples of perennial ryegrass using genotyping-by-sequencing. *Theoretical and Applied Genetics* 129: 45-52. https://doi.org/0.1007/s00122-015-2607-9

Endelman, JB, et al., 2018. Genetic variance partitioning and genome-wide prediction with allele dosage information in autotetraploid potato. *Genetics*, 209(1) pp. 77-87. https://doi.org/10.1534/genetics.118.300685

Hamilton, MG, et al., 2017 Computation of the inverse additive relationship matrix for autopolyploid and multiple-ploidy populations. *Theoretical and Applied Genetics*. https://doi.org/10.1007/s00122-017-3041-y

Henderson, C, 1976 A simple method for computing the inverse of a numerator relationship matrix used in prediction of breeding values. *Biometrics* pp. 69–83. https://doi.org/10.2307/2529339

Kerr, RJ, et al., 2012 Use of the numerator relation ship matrix in genetic analysis of autopolyploid species. *Theoretical and Applied Genetics* 124: 1271–1282. https://doi.org/10.1007/s00122-012-1785-y

Martini, JW, et al., 2018, The effect of the H− 1 scaling factors τ and ω on the structure of H in the single-step procedure. Genetics Selection Evolution, 50(1), 16. https://doi.org/10.1186/s12711-018-0386-x

Mrode, R. A., 2014 *Linear models for the prediction of animal breeding values*. Cabi. 3rd ed.

Munoz, PR, et al., 2014 Unraveling additive from nonadditive effects using genomic relationship matrices. *Genetics* 198: 1759–1768. https://doi.org/10.1534/genetics.114.171322

R Core Team, 2016 *R*: A Language and Environment for Statistical Computing. R Foundation for Statistical Computing, Vienna, Austria.

Resende, MF, et al., 2012 Accuracy of genomic selection methods in a standard data set of loblolly pine (*Pinus taeda* l.). *Genetics* 190: 1503–1510. https://doi.org/10.1534/genetics.111.137026

Slater, AT, et al., 2014 Improving the analysis of low heritability complex traits for enhanced genetic gain in potato. *Theoretical and applied genetics* 127: 809–820. https://doi.org/10.1007/s00122-013-2258-7

Slater AT, et al., 2016 Improving genetic gain with genomic selection in autotetraploid potato. *The Plant Genome* 9.  https://doi.org/10.3835/plantgenome2016.02.0021 

Su, G, et al., 2012 Estimating additive and non-additive genetic variances and predicting genetic merits using genome-wide dense single nucleotide polymorphism markers. *PloS one* 7:e45293. https://doi.org/10.1371/journal.pone.0045293

VanRaden, P, 2008 Efficient methods to compute genomic predictions. *Journal of dairy science* 91: 4414–4423. https://doi.org/10.3168/jds.2007-0980

Vitezica, ZG, et al., 2013 On the additive and dominant variance and covariance of individuals within the genomic selection scope. *Genetics* 195: 1223–1230. https://doi.org/10.1534/genetics.113.155176

Yang, J, et al., 2010 Common snps explain a large proportion of the heritability for human height. *Nature genetics* 42: 565–569. https://doi.org/10.1038/ng.608

```{r,eval=FALSE,echo=FALSE}
#To knit an this vignette into an .R file
knitr::purl("vignettes/Tutorial_AGHmatrix.Rmd")
```


```{r}
sessionInfo()
```

---
title: "AGHmatrix Tutorial"
author: "Rodrigo Amadeu"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{AGHmatrix Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r knitr_init, echo=FALSE, cache=FALSE}
library(knitr)
library(rmarkdown)

knitr::opts_chunk$set(collapse = TRUE,
                      comment = "#>",
                      fig.width = 7,
                      fig.height = 8,
                      fig.align = "center",
                      dev = "png",
                      dpi = 72,
                      cache = TRUE)

```

```{r, echo=FALSE, results='hide'}
library(AGHmatrix)
```

## Contact
Rodrigo R Amadeu  
rramadeu at gmail dot com  
https://rramadeu.github.io

## Overview
AGHmatrix software is an R-package to build relationship matrices using pedigree (A matrix) and/or molecular markers (G matrix) with the possibility to build a combined matrix of Pedigree corrected by Molecular (H matrix). The package works with diploid and autopolyploid data.

## Matrices computation implemented in the `AGHmatrix` 
Currently the package computes the following 17 different relationship matrices:

### Pedigree-based relationship matrix (A matrix)

|                   | Additive                   | Non-Additive     |
|-------------------|----------------------------|------------------|
| **Diploid**       | Henderson (1976)           | Cockerham (1954) |
| **Autopolyploid** | Kerr (2012), Slater (2013) |                  |

### Molecular-based relationship matrix (G matrix)
|                   | Additive                                  | Non-Additive                   |
|-------------------|-------------------------------------------|--------------------------------|
| **Diploid**       | Yang (2010), VanRaden (2012), Liu (2020)  | Su (2012), Vitezica (2013)     |
| **Polyploid**     | Slater (2016), de Bem Oliveira (2019)     | Slater (2016), Endelman (2018) |


### Combined pedigree and molecular-based relationship matrix (H matrix)

| **Any ploidy/effect**                        |
|----------------------------------------------|
| Legarra (2009), Munoz (2014), Martini (2018) |

Additionally there is a beta implementation to compute A matrix when parentage is not deterministic as in a polycross design. See `?AmatrixPolycross`.

## Citation
To cite this R package:

Amadeu RR, Garcia AA, Munoz PR, Ferrão LF. AGHmatrix: genetic relationship matrices in R. Bioinformatics. 2023 Jul 1;39(7):btad445. https://doi.org/10.1093/bioinformatics/btad445

## Installing and loading
Within R:
```{r, eval=FALSE}
## Install stable version
install.packages("AGHmatrix")

## Install development version
install.packages("devtools")
devtools::install_github("rramadeu/AGHmatrix")

## Load 
library(AGHmatrix)
```

## Relationship matrices using pedigree data - A matrix
`Amatrix` process the pedigree and build the A-matrix related to that given pedigree. The matrix is built based in the recursive method presented in Mrode (2014) and described by Henderson (1976). This method is expanded for higher ploidies (n-ploidy) as detailed in Kerr et al. (2012). After loading the package you have to load your data file into the software. To do this, you can use the function `read.data()` or `read.csv()` for example. Your data should be available in R as a `data.frame` structure in the following order: column 1 must be the individual/genotype names (id), columns 2 and 3 must be the parent names. For the algorithm, it does not matter who is the mother and who is the father (so, no sex column). There is a pedigree data example (`ped.mrode`) that can be used to look at the structure and order the data.

To load `ped.mrode`:

```{r}
data(ped.mrode)
ped.mrode
str(ped.mrode) #check the structure
```

The example `ped.mrode` has 3 columns, column 1 contains the names of the individual/genotypes, column 2 contains the names of the first parent, column 3 contains the names of the second parental (example from Table 2.1 of Mrode 2014). There is no header template, and the unknown value must be equal 0. Your data has to be in the same format of `ped.mrode`.

Internally the algorithm first pre-process the pedigree: the individuals are numerated $1$ to $n$. Then, it is verified whether the genotypes in the pedigree are in chronological order (i.e. if the parents of a given individual are located before to this individual in the pedigree data set). If this order is not followed, the algorithm performs the necessary changes to correct them in a iterative way. After this pre-processing, the matrix computation proceeds as in Henderson (1976) for diploid - for additive or dominance relationship - and as in Kerr et al. (2012) for autotetraploids - for additive relationship. For autotetraploids, there is the option to include double-reduction fraction. For diploids there is the option to compute the dominant relationship matrix (Cockerham, 1954).

It follows some usage examples with the `ped.mrode`.
```{r, eval=FALSE}
#Computing additive relationship matrix for diploids (Henderson 1976):
Amatrix(ped.mrode, ploidy=2)

#Computing dominant relationship matrix for diploids (Cockerham 1954):
Amatrix(ped.mrode, ploidy=2, dominance=TRUE)

#Computing additive relationship matrix for autotetraploids (Kerr 2012):
Amatrix(ped.mrode, ploidy=4)

#Computing additive relationship matrix for autooctaploids (Kerr 2012):
Amatrix(ped.mrode, ploidy=8)

#Computing additive relationship matrix for autotetraploids
# and double-reduction of 0.1 (Kerr 2012):
Amatrix(ped.mrode, ploidy=4, w=0.1)

#Computing additive relationship matrix for autotetraploids
# and double-reduction of 0.1 as in Slater et al. (2014):
Amatrix(ped.mrode, ploidy=4, w=0.1, slater = TRUE) 
#not recommended, but kept in the package to reproduce some former analysis

#Computing additive relationship matrix for autohexaploids
# and double-reduction of 0.1 (Kerr 2012):
Amatrix(ped.mrode, ploidy=6, w=0.1)
```

More information about `Amatrix` can be found with:
```{r, eval=FALSE}
?Amatrix
```

## Diploid G matrix: relationship matrices using the molecular data
`Gmatrix` handles the molecular-marker matrix and builds the relationship matrix. Molecular markers data should be organized in a matrix format (individuals in rows and markers in columns) coded as 0, 1, 2 and missing data value (numeric or `NA`). Import your molecular marker data into `R` with the function `read.table()` or `read.csv()` and convert to a matrix format with the function `as.matrix()`. The function `Gmatrix` can be used to construct the additive relationship either as proposed by Yang et al. (2010) or the proposed by VanRaden (2008). The function can also construct the dominance relationship matrix either as proposed by Su et al. (2012) or as proposed by Vitezica et al. (2013). As an example, here we build the four matrices using real data from Resende et al. (2012).

To load `snp.pine` and to check its structure:
```{r}
data(snp.pine)
snp.pine[1:5,1:5]
str(snp.pine)
```

In this dataset, we have 926 individuals with 4853 markers and the missing data value is `-9`.

It follows some examples with the `snp.pine` data where the unknown value (`missingValue`) is `-9`. Here we set minimum allele frequency to `0.05`, so markers with minor allele frequency lower than 0.05 are removed from the dataset prior to the G matrix construction.

```{r, eval=FALSE}
#Computing the additive relationship matrix based on VanRaden 2008
G_VanRadenPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                          maf=0.05, method="VanRaden")

#Computing the additive relationship matrix based on Yang 2010
G_YangPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                      maf=0.05, method="Yang")

#Computing the dominance relationship matrix based on Su 2012
G_SuPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                    maf=0.05, method="Su")

#Computing the dominance relationship matrix based on Vitezica 2013
G_VitezicaPine <- Gmatrix(SNPmatrix=snp.pine, missingValue=-9, 
                          maf=0.05, method="Vitezica")
```

More information about `Gmatrix` can be found with:
```{r, eval=FALSE}
?Gmatrix
```

## Autopolyploid G matrix: relationship matrices using the molecular data
Molecular markers data should be organized in a matrix format (individual in rows and markers in columns) coded according to the dosage level: 0, 1, 2, ..., ploidy level, and missing data value (numeric user-defined or `NA`). As an example, an autotetraploid should be coded as 0, 1, 2, 3, 4, and a missing data value. In autopolyploids, the function `Gmatrix` can be used to construct: i) the additive relationship based on VanRaden (2008) and extended by Ashraf (2016); ii) the full-autopolyploid including additive and non-additive model as equations 8 and 9 in Slater et al. (2016); iii) the pseudo-diploid model as equations 5, 6, and 7 Slater et al. (2016). iv) the digenic-dominant model based on Endelman et al. (2018). As an example, here we build the matrices using data from Endelman et al. (2018) (`snp.sol`). There is also an option to build weighted relationship matrices as in Liu et al. (2020).

The argument `ploidy.correction` defines the denominator of the formula for the `VanRaden` method. If `ploidy.correction=TRUE`, it uses the parametric correction as $\sum_i p f_i(1-f_i)$, where $p$ is the ploidy level and $f_i$ is the minor allele frequency of the $i_th$ marker. If `ploidy.correction=FALSE`, it uses the sampling variance correction as $\sum_i \frac{1}{p} s^2(m_i)$, where $s^2(m_i)$ is the sampling variance of the $i_th$ marker. Both corrections are equivalent when sampling size goes to the infinity. The default is to use the sampling variance as the correction (i.e., `ploidy.correction=FALSE`).

```{r, eval=FALSE}
#Loading the data
data(snp.sol)
str(snp.sol)

#Computing the additive relationship matrix based on VanRaden 2008
# adapted by Ashraf 2016
G_VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)

#Computing the dominance (digenic) matrix based on Endelman 2018 (Eq. 19)
G_Dominance <- Gmatrix(snp.sol, method="Endelman", ploidy=4)

#Computing the full-autopolyploid matrix based on Slater 2016 (Eq. 8
#and 9)
G_FullAutopolyploid <- Gmatrix(snp.sol, method="Slater", ploidy=4)

#Computing the pseudodiploid matrix based on Slater 2016 (Eq. 5, 6,
#and 7)
G_Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE) 

#Computing G matrix with specific weight for each marker as 
# in Liu et al. (2020).
Gmatrix_weighted <- Gmatrix(snp.sol, method="VanRaden", weights = runif(3895,0.001,0.1), ploidy=4)
```

More information about `Gmatrix` can be found with:
```{r, eval=FALSE}
?Gmatrix
```

## Ratio (non-dosage) G matrix: relationship matrices using the molecular data without dosage calling
Molecular markers data should be organized in a matrix format (individual in rows and markers in columns) coded according to a fraction that represents its molecular information, this can be any number between 0 to 1. Such ratio can represent the count of alternative alleles over the read depth for each individual-marker combination (GBS-like technique). It can be the signal of the alternative allele over the sum of the signals of the alternative and reference alleles (GCMS-like technique). It can also be used for family-pool genotypes.

```{r, eval=FALSE}
#Loading the data
library(AGHmatrix)
data(snp.sol) 
snp.sol.ratio = snp.sol/4 #transforming it in a ratio of the minor allele frequency
Gmatrix <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, ratio=FALSE)  
Gmatrix.ratio <- Gmatrix(snp.sol.ratio, method="VanRaden", ploidy=4, ratio=TRUE)
Gmatrix[1:5,1:5]==Gmatrix.ratio[1:5,1:5]

## it also has the ploidy.correction option
Gmatrix.alternative <- Gmatrix(snp.sol, 
                               method="VanRaden", 
                               ploidy=4, 
                               ratio=FALSE, 
                               ploidy.correction=TRUE) 

Gmatrix.ratio.alternative <- Gmatrix(snp.sol.ratio, 
                                     method="VanRaden", 
                                     ploidy=4, 
                                     ratio=TRUE, 
                                     ploidy.correction=TRUE)
Gmatrix[1:5,1:5]==Gmatrix.alternative[1:5,1:5]
Gmatrix.alternative[1:5,1:5]==Gmatrix.ratio.alternative[1:5,1:5]
```

## Combined relationship matrix - H matrix
H matrix is the relationship matrix using combined information from the pedigree and genomic relationship matrices. First, you need to compute the matrices separated and then use them as input to build the combined H matrix. Two methods are implemented: `Munoz` shrinks the G matrix towards the A matrix scaling the molecular relatadness by each relationship classes; `Martini` is a modified version from Legarra et al. 2009 where combines A and G matrix using scaling factors. As an example, here we build the matrices using data from Endelman et al. (2018) (`ped.sol` and `snp.sol`).

```{r, eval=FALSE}
data(ped.sol)
data(snp.sol)

#Computing the numerator relationship matrix 10% of double-reduction
Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)
Gmat <- Gmatrix(snp.sol, ploidy=4,
                maf=0.05, method="VanRaden")
Gmat <- round(Gmat,3) #see appendix

#Computing H matrix (Martini)
Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini", 
                        ploidy=4, missingValue=-9, maf=0.05)

#Computing H matrix (Munoz)
Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol, 
                      ploidy=4, method="Munoz", 
                      missingValue=-9, maf=0.05)
```
## Covariance matrices due to epistatic terms
Here we present how to compute the epistasis relationship matrices using Hadamard
products (i.e. cell-by-cell product), denoted by `*`. For more information please see Munoz
et al. (2014). In this example we are using the molecular-based relationship matrices. First, build the additive and dominance matrices:

```{r, eval=FALSE}
data(snp.pine)
A <- Gmatrix(SNPmatrix=snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
D <- Gmatrix(SNPmatrix=snp.pine, method="Vitezica", missingValue=-9,maf=0.05)
```

For the first degree epistatic terms:
```{r, eval=FALSE}
#Additive-by-Additive Interactions
A_A <- A*A
#Dominance-by-Additive Interactions
D_A <- D*A
#Dominance-by-Dominance Interactions
D_D <- D*D
```

For the second degree epistatic terms:
```{r, eval=FALSE}
#Additive-by-Additive-by-Additive Interactions
A_A_A <- A*A*A
#Additive-by-Additive-by-Dominance Interactions
A_A_D <- A*A*D
#Additive-by-Dominance-by-Dominance Interactions
A_D_D <- A*D*D
#Dominance-by-Dominance-by-Dominance Interactions
D_D_D <- D*D*D
```

And so on...

## Exporting your matrix as three columns and sparse format (ASReml - csv format)
That is the lower diagonal matrix formatted in three columns in .csv format (other ASCII extension could be used as well). In order to do this, we need to build a matrix, its inverse, and export it using `formatmatrix` function. ASReml can invert the relationship matrix as well, probably more efficiently than R for large matrices (i.e. `solve()` function), so no need to invert the matrix in R if matrix is large. This function has as options: `round.by`, which let you decide the number of decimals you want; `exclude.0`, if `TRUE`, remove all the zeros from your data (i.e., transforms into sparse); and, name that defines the name to be used in the exported file. Use the default if not sure what parameter use in these function. Here an example using `ped.mrode` data:

```{r, eval=FALSE}
#Loading the data example
data(ped.mrode)

#Computing the matrix
A <- Amatrix(data=ped.mrode, ploidy=4, w=0.1)

#Building its inverse
Ainv <- solve(A)

#Exporting it. The function "formatmatrix" 
# will convert it and save in your working directory
formatmatrix(Ainv, round.by=12, exclude.0=TRUE, name="Ainv")
```

## Relationship matrices using pedigree data for polycrosses - A matrix (beta)
Creates an additive relationship matrix A based on a non-deterministic pedigree with 4+ columns where each column represents a possible parent. This function was built with the following designs in mind. 1) A mating design where you have equally possible parents. For example, a generation of insects derived from the mating of three insects in a cage. All the insects in this generation will have the same expected relatedness with all the possible parents (1/3). If there are only two parents in the cage, the function assumes no-inbreeding and the pedigree is deterministic (the individual is offspring of the cross between the two parents). Another example, a population of 10 open-pollinated plants where you harvest the seeds without tracking the mother. 2) When `fixedParent` is TRUE: a mating design where you know one parent and might know the other possible parents. For example, a polycross design where you have seeds harvested from a mother plant and possible polén donors.

The following pedigree has the id of the individual followed by possible parents. The possible parents are filled from left to right, in the `pedigree` data frame: id 1,2,3,4 have unknown parents and are assumed unrelated; id 5 has three possible parents (1,2,3); id 6 has three possible parents (2,3,4); id 7 has two parents (deterministic case here, the parents are 3 and 4); id 8 has four possible parents (5,6,7,1).

```{r}
pedigree = data.frame(id=1:8,
                      parent1 = c(0,0,0,0,1,2,3,5),
                      parent2 = c(0,0,0,0,2,3,4,6),
                      parent3 = c(0,0,0,0,3,4,0,7),
                      parent4 = c(0,0,0,0,0,0,0,1),
                      parent5 = 0)

print(pedigree)
AmatrixPolyCross(pedigree)
```

If `fixedParent=TRUE`, the above pedigree will be interpreted with the possible parents are filled from left to right after the known parent, in the `pedigree` data frame: id 1,2,3,4 have unknown parents and are assumed unrelated; id 5 is offspring of parent 1 in a deterministic way and two other possible parents (2,3); id 6 is offspring of parent 2 in a deterministic way and two other possible parents (3,4); id 7 has two parents (deterministic case here, the parents are 3 and 4); as before; id 8 is offspring of parent 5 in a deterministic way and has three other possible parents (6,7,1).

```{r}
AmatrixPolyCross(pedigree,fixedParent=TRUE)
```

## Amatrix() benchmark

It follows a small memory and computational time profiling for the `Amatrix()` function. The required RAM was computed based on the peak of the process for different pedigree sizes (based on /usr/bin/time -v output). The time profiling was done using AMD Milan 2.95GHz, so it might be an underestimated value when compared with lower speed processors. Numerator relationship matrices for pedigrees with less than 20,000 rows can built with low-specs user-end machines (<8GB RAM) using our package.

```{r, eval=TRUE,echo=FALSE}
x = c(1000,5000,10000,20000,30000,40000,50000,60000,70000,80000,90000,100000)/1000 #Pedigree Size
y = c(252156,622500,1795260,6481064,14313448,25227680,49081224,70622336,96017144,125320048,158444856,194731908)/1e+6 #RAM GB 
ytime = c(0.0025, 0.080, 0.2, 0.89, 1.62,3.01,4.52,7.12,9.15,13.13,15.13,20) #minutes
df = data.frame(size=x,ram=y,time=ytime)

plot(x=df$size,y=df$ram, 
     ylab = "RAM (GB) at the peak of Amatrix() function",
     xlab = "Pedigree size (in 1,000 rows)",
     type="b",
     axes=FALSE)
axis(side = 2, at = c(0,4,8,16,32,48,64,96,144,192),cex.axis=.75)
axis(side = 1, at = c(1,5,10,20,30,40,50,60,70,80,90,100),cex.axis=.75)

plot(x=df$size,y=df$time, type="b",
     ylab = "Time to run (minutes) the Amatrix() function",
     xlab = "Pedigree size (in 1,000 rows)",
     axes=FALSE)
axis(side = 2, at = seq(0,20,2),cex.axis=.75)
axis(side = 1, at = c(1,5,10,20,30,40,50,60,70,80,90,100),cex.axis=.75)
```


## Bibliography
Amadeu, RR, et al. 2023 AGHmatrix: genetic relationship matrices in R. Bioinformatics 39, 7. 

Amadeu, RR, et al. 2016 AGHmatrix: R package to construct relationship matrices for autotetraploid and diploid species: a blueberry example. The Plant Genome 9, 4. 

Ashraf, BH, et al. 2016. Estimating genomic heritabilities at the level of family-pool samples of perennial ryegrass using genotyping-by-sequencing. Theoretical and Applied Genetics 129, 45-52. 

Cockerham, CC. 1954 An extension of the concept of partitioning hereditary variance for analysis of covariances among relatives when epistasis is present. Genetics 39, 859–882. 

de Bem Oliveira, I, et al. 2019 Genomic prediction of autotetraploids; influence of relationship matrices, allele dosage, and continuous genotyping calls in phenotype prediction. G3: Genes, Genomes, Genetics, 9(4), pp.1189-1198.

Endelman, JB, et al. 2018 Genetic variance partitioning and genome-wide prediction with allele dosage information in autotetraploid potato. Genetics 209, 77-87. 

Hamilton, MG, et al. 2017 Computation of the inverse additive relationship matrix for autopolyploid and multiple-ploidy populations. Theoretical and Applied Genetics 131, 851-890.  

Henderson, C. 1976 A simple method for computing the inverse of a numerator relationship matrix used in prediction of breeding values. Biometrics 32, 69–83. 

Kerr, RJ, et al., 2012 Use of the numerator relation ship matrix in genetic analysis of autopolyploid species. Theoretical and Applied Genetics 124, 1271–1282. 

Legarra, A, et al. 2009 A relationship matrix including full pedigree and genomic information. Journal of Dairy Science 92, 4656–4663.

Liu, A, et al. 2020. Weighted single-step genomic best linear unbiased prediction integrating variants selected from sequencing data by association and bioinformatics analyses. Genet Sel Evol 52, 48.

Martini, JW, et al. 2018, The effect of the H$^{1}$ scaling factors $\tau$ and $\omega$ on the structure of H in the single-step procedure. Genetics Selection Evolution 50(1), 16. 

Mrode, RA. 2014 *Linear models for the prediction of animal breeding values*. Cabi. 3rd ed.

Munoz, PR, et al., 2014 Unraveling additive from nonadditive effects using genomic relationship matrices. Genetics 198, 1759–1768. 

R Core Team, 2016 *R*: A Language and Environment for Statistical Computing. R Foundation for Statistical Computing, Vienna, Austria.

Resende, MF, et al. 2012 Accuracy of genomic selection methods in a standard data set of loblolly pine (*Pinus taeda* l.). Genetics 190, 1503–1510. 

Slater, AT, et al. 2014 Improving the analysis of low heritability complex traits for enhanced genetic gain in potato. Theoretical and applied genetics 127, 809–820. 

Slater, AT, et al. 2016 Improving genetic gain with genomic selection in autotetraploid potato. The Plant Genome 9. 

Su, G, et al. 2012 Estimating additive and non-additive genetic variances and predicting genetic merits using genome-wide dense single nucleotide polymorphism markers. PloS one 7, e45293. https://doi.org/10.1371/journal.pone.0045293

VanRaden, P. 2008 Efficient methods to compute genomic predictions. Journal of dairy science 91, 4414–4423.

Vitezica, ZG, et al. 2013 On the additive and dominant variance and covariance of individuals within the genomic selection scope. Genetics 195, 1223–1230. 

Yang, J, et al. 2010 Common snps explain a large proportion of the heritability for human height. Nature genetics 42, 565–569. 


```{r,eval=FALSE,echo=FALSE}
#To knit an this vignette into an .R file
knitr::purl("vignettes/Tutorial_AGHmatrix.Rmd")
```


```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datatreat.R
\name{asciitonumber}
\alias{asciitonumber}
\title{Convert pedigree names to numeric indices using C++}
\usage{
asciitonumber(pedigree.data, unk = 0)
}
\arguments{
\item{pedigree.data}{A 3-column matrix or data frame with individual, sire, and dam}

\item{unk}{Code for unknown parent (default = "0")}
}
\value{
A list with sire, dam, and individual names
}
\description{
Convert pedigree names to numeric indices using C++
}
\author{
Rodrigo R. Amadeu \email{rramadeu@gmail.com}

Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expandAmatrix.R
\name{expandAmatrix}
\alias{expandAmatrix}
\title{Add new crosses to a current A matrix}
\usage{
expandAmatrix(newPedigree = NULL, A = NULL, returnAll = TRUE)
}
\arguments{
\item{newPedigree}{pedigree data name (3-column way format). Unknown value should be equal 0.}

\item{A}{numerator relationship matrix output from Amatrix function.}

\item{returnAll}{if TRUE returns old A with new A, if FALSE returns only new A}
}
\value{
Matrix with the Relationship between the individuals.
}
\description{
Expand a current A matrix with a new pedigree. The parents in the new pedigree should also be in the A matrix.
}
\examples{
data(ped.sol)
ped.initial = ped.sol[1:1120,]
ped.new = ped.sol[-c(1:1120),]
#Computing additive relationship matrix:
A = Amatrix(ped.initial, ploidy=2)
Anew = expandAmatrix(ped.new, A)

#Comparing with one-step building..
Afull = Amatrix(ped.sol, ploidy=2)
test = Anew-Afull
which(test!=0)

}
\author{
Rodrigo R Amadeu, \email{rramadeu@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ped.sol-data.R
\docType{data}
\name{ped.sol}
\alias{ped.sol}
\title{Pedigree data for autopolyploid examples}
\format{
data.frame
}
\usage{
data(ped.sol)
}
\description{
Dataset extract from  supplementary material from Endelman et al. (2018). 
Pedigree data frame of Potato population, missing data as 0.
}
\examples{
data(ped.sol)

}
\references{
Endelman, JB, et al., 2018 Genetic variance partitioning and genome-wide prediction with allele dosage information in autotetraploid potato. Genetics, 209(1) pp. 77-87.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatmatrix.R
\name{formatmatrix}
\alias{formatmatrix}
\title{Transform a matrix in 3 columns}
\usage{
formatmatrix(
  data = NULL,
  save = TRUE,
  return = FALSE,
  name = deparse(substitute(data)),
  round.by = 12,
  exclude.0 = TRUE
)
}
\arguments{
\item{data}{matrix (nxn).}

\item{save}{if TRUE save the output in a file. Default=TRUE.}

\item{return}{if TRUE return the output in a object. Default=FALSE.}

\item{name}{name of the csv file to be saved. Default=data name.}

\item{round.by}{select the number of digits after 0 you want in your data. Default = 12}

\item{exclude.0}{if TRUE, remove all lines equal to zero (ASREML option). Default = TRUE}
}
\value{
a object or a csv file with a table with 3 columns representing the matrix.
}
\description{
Given any square matrix transform it in a 3 columns way (row, column, value) mainly to be used in outsourcing data processing (as ASREML-standalone)
}
\examples{
#Example with random matrix
data<-matrix(c(1,0.1,0,0.1,1,0,0,0,1.1),3)
formatmatrix(data=data,save=FALSE,return=TRUE,exclude.0=TRUE)

#Example with pedigree matrix
#Reading the example data
data(ped.mrode)
#Making Relationship Matrix
Amrode<-Amatrix(ped.mrode)
#Inverting the Matrix
Amrode.inv<-solve(Amrode)
#Making the 3 columns format
Amrode.inv.ASREML<-formatmatrix(Amrode,save=FALSE,return=TRUE,exclude.0=TRUE)
#Printing it
Amrode.inv.ASREML 

}
\author{
Rodrigo R Amadeu, \email{rramadeu@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datatreat.R
\name{datatreat}
\alias{datatreat}
\title{Organizes pedigree data in a chronological way using C++}
\usage{
datatreat(data = NULL, n.max = 50, unk = 0, save = FALSE)
}
\arguments{
\item{data}{A 3-column data frame or matrix with individual, sire, and dam}

\item{n.max}{Max number of iterations (default = 50)}

\item{unk}{Code for unknown parent (default = 0)}

\item{save}{If TRUE, saves the reordered pedigree to a text file}
}
\value{
A list with elements: sire (numeric), dire (numeric), ind_data (names)
}
\description{
This function organizes pedigree data in a chronological way and returns a list:
i) parental 1 values (numeric); ii) parental 2 values (numeric); iii) individual names.
Also saves a `.txt` file if `save = TRUE`.
}
\examples{
# data(ped.mrode)
# datatreat(ped.mrode)
}
\author{
Rodrigo R. Amadeu \email{rramadeu@gmail.com}

Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Amatrix.R
\name{Amatrix}
\alias{Amatrix}
\title{Construction of Relationship Matrix A}
\usage{
Amatrix(
  data = NULL,
  ploidy = 2,
  w = 0,
  verify = TRUE,
  dominance = FALSE,
  slater = FALSE,
  ASV = FALSE,
  ...
)
}
\arguments{
\item{data}{Pedigree data frame in a 3-column format. Unknown values should 
be set to 0.}

\item{ploidy}{An even number representing the ploidy level (default is 2).}

\item{w}{Proportion of parental gametes that are IBD due to double 
reduction (default is 0). Applicable only when \code{ploidy = 4}.}

\item{verify}{Logical; if TRUE (default), the pedigree file is checked for 
conflicting entries.}

\item{dominance}{Logical; if TRUE, computes the dominance relationship 
matrix (valid only for \code{ploidy = 2}).}

\item{slater}{Logical; if TRUE and \code{ploidy = 4}, computes the additive 
autotetraploid relationship matrix using Slater (2013).}

\item{ASV}{Logical; if TRUE, transforms the matrix into the average 
semivariance (ASV) defined as 
\eqn{K = K / (\text{trace}(K) / (\text{nrow}(K) - 1))}. See Feldmann et al. 
(2022), Formula 2.}

\item{...}{Additional arguments passed to \code{datatreat()}.}
}
\value{
A square matrix representing the relationship between individuals 
in the pedigree.
}
\description{
Constructs an additive relationship matrix (A) from pedigree data in a 
three-column format. The calculation depends on the specified ploidy level 
(must be an even  number). If \code{ploidy = 4}, the matrix can incorporate a 
user-defined proportion (\code{w}) of parental gametes that are IBD 
(Identical by Descent) due to double reduction.

When \code{ploidy = 2} and \code{dominance = FALSE}, the diploid additive 
numerator relationship matrix is built using Henderson's (1976) method. If 
\code{dominance = TRUE}, a diploid dominance numerator relationship matrix 
is computed following Cockerham (1954). The recursive approach is detailed 
in Mrode (2005).

For \code{ploidy > 2} and \code{dominance = FALSE}, an autopolyploid 
additive relationship matrix is computed based on Kerr et al. (2012). 
If \code{slater = TRUE} and \code{ploidy = 4}, Slater's (2013) method is 
used instead.
}
\note{
This function uses compiled C++ code via Rcpp for improved performance.
}
\examples{
data(ped.mrode)
# Additive relationship matrix for diploids (Henderson 1976):
Amatrix(ped.mrode, ploidy = 2)

# Dominance relationship matrix for diploids (Cockerham 1954):
Amatrix(ped.mrode, ploidy = 2, dominance = TRUE)

# Additive relationship matrix for autotetraploids (Kerr 2012):
Amatrix(ped.mrode, ploidy = 4)

# Additive relationship matrix for autooctaploids (Kerr 2012):
Amatrix(ped.mrode, ploidy = 8)

# Additive relationship matrix for autotetraploids with double reduction 
# (Kerr 2012):
Amatrix(ped.mrode, ploidy = 4, w = 0.1)

# Additive relationship matrix for autotetraploids with double reduction 
# using Slater (2013):
Amatrix(ped.mrode, ploidy = 4, w = 0.1, slater = TRUE)

# Additive relationship matrix for autohexaploids with double reduction 
# (Kerr 2012):
Amatrix(ped.mrode, ploidy = 6, w = 0.1)

}
\references{
\emph{Cockerham, C.C. (1954). An extension of the concept of partitioning 
  hereditary variance for analysis of covariances among relatives when 
  epistasis is present. Genetics, 39, 859–882.}

\emph{Feldmann, M.J., et al. (2022). Average semivariance directly yields 
  accurate estimates of the genomic variance in complex trait analyses. 
  G3 (Bethesda), 12(6).}

\emph{Henderson, C.R. (1976). A simple method for computing the inverse 
  of a numerator relationship matrix used in prediction of breeding values. 
  Biometrics, 32, 69–83.}

\emph{Kerr, R.J., et al. (2012). Use of the numerator relationship matrix 
  in genetic analysis of autopolyploid species. Theoretical and Applied 
  Genetics, 124, 1271–1282.}

\emph{Mrode, R.A. (2014). Linear Models for the Prediction of Animal 
  Breeding Values (3rd ed.). CABI.}

\emph{Slater, A.T., et al. (2013). Improving the analysis of low 
  heritability complex traits for enhanced genetic gain in potato. 
  Theoretical and Applied Genetics, 127, 809–820.}
}
\author{
Rodrigo R Amadeu, \email{rramadeu@gmail.com}

Thiago de Paula Oliveira, \email{toliveira@abacusbio.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AmatrixPolyCross.R
\name{AmatrixPolyCross}
\alias{AmatrixPolyCross}
\title{Construction of pedigree-based relationship matrix with parental guessing possibility}
\usage{
AmatrixPolyCross(data = NULL, fixedParent = FALSE)
}
\arguments{
\item{data}{pedigree data name. Unknown value should be equal 0. See example for construction.}

\item{fixedParent}{if false, assumes that all the parents are equally possible parents. If true, assumes that the first parental is known and the others are equally possible parents. Default = FALSE.}
}
\value{
Matrix with the relationship between the individuals.
}
\description{
Creates an additive relationship matrix A based on a non-deterministic pedigree with 4+ columns where each column represents a possible parent. This function was built with the following designs in mind. 
1) A mating design where you have equally possible parents. For example, a generation of insects derived from the mating of three insects in a cage. All the insects in this generation will have the same expected relatedness with all the possible parents (1/3). If there are only two parents in the cage, the function assumes no-inbreeding and the pedigree is deterministic (the individual is offspring of the cross between the two parents). Another example, a population of 10 open-pollinated plants where you harvest the seeds without tracking the mother. 
2) When fixedParent is TRUE: a mating design where you know one parent and might know the other possible parents. For example, a polycross design where you have seeds harvested from a mother plant and possible polen donors.
}
\examples{
#the following pedigree has the id of the individual followed by possible parents
#if 0 is unknown
#the possible parents are filled from left to right
#in the pedigree data frame examples:
#id 1,2,3,4 have unknown parents and are assumed unrelated
#id 5 has three possible parents (1,2,3)
#id 6 has three possible parents (2,3,4)
#id 7 has two parents (deterministic case here, the parents are 3 and 4)
#id 8 has four possible parents (5,6,7,1)

pedigree = data.frame(id=1:8,
                      parent1 = c(0,0,0,0,1,2,3,5),
                      parent2 = c(0,0,0,0,2,3,4,6),
                      parent3 = c(0,0,0,0,3,4,0,7),
                      parent4 = c(0,0,0,0,0,0,0,1),
                      parent5 = 0)

print(pedigree)

AmatrixPolyCross(pedigree)

#when polyCross is set to be true:
#id 5 is offspring of parent 1 in a deterministic way and two other possible parents (2,3)
#id 6 is offspring of parent 2 in a deterministic way and two other possible parents (3,4)
#id 7 has two parents (deterministic case here, the parents are 3 and 4); as before
#id 8 is offspring of parent 5 in a deterministic way and has three other possible parents (6,7,1)

AmatrixPolyCross(pedigree,fixedParent=TRUE)

}
\author{
Rodrigo R Amadeu, \email{rramadeu@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snp.pine-data.R
\docType{data}
\name{snp.pine}
\alias{snp.pine}
\title{Molecular data for diploid examples}
\format{
matrix
}
\usage{
data(snp.pine)
}
\description{
Dataset extract from  supplementary material from Resende et al. (2012). 
SNP marker matrix from Pine tree coded as 0,1, and 2, and missing value as -9.
}
\examples{
data(snp.pine)

}
\references{
Resende, MF, et al., 2012 Accuracy of genomic selection methods in a standard data set of loblolly pine (Pinus taeda l.). Genetics 190: 1503–1510.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Mcheck.R
\name{Mcheck}
\alias{Mcheck}
\title{Check and filter markers}
\usage{
Mcheck(
  SNPmatrix = NULL,
  ploidy = 2,
  missingValue = -9,
  thresh.maf = 0.05,
  thresh.missing = 0.9,
  thresh.htzy = 0,
  impute.method = "mean",
  rmv.mono = TRUE
)
}
\arguments{
\item{SNPmatrix}{matrix (n x m), where n is is individual names and m is marker names (coded inside the matrix as 0, 1, 2, ..., ploidy, and, missingValue).}

\item{ploidy}{data ploidy (an even number between 2 and 20). Default=2.}

\item{missingValue}{missing value in data. Default=-9.}

\item{thresh.maf}{minimum allele frequency accepted to each marker. Default=0.05.}

\item{thresh.missing}{threshold on missing data, SNPs below of this frequency value will be maintained, if equal to 1, no threshold and imputation is considered. Default = 0.50.}

\item{thresh.htzy}{threshold heterozigosity, remove SNPs below this threshold. Default=0.}

\item{impute.method}{"mean" to impute the missing data by the mean per marker, "mode" to impute the missing data by the mode per marker, "global.mean" to impute the missing data by the mean across all markers, "global.mode" to impute the missing data my the mode across all marker. Default = "mean".}

\item{rmv.mono}{if monomorphic markers should be removed. Default=TRUE.}
}
\value{
SNPmatrix after filtering steps.
}
\description{
This function does different filtering on the marker matrix
}
\examples{
data(snp.pine)
M = Mcheck(snp.pine)

}
\author{
Luis F V Ferrao

Rodrigo Amadeu, \email{rramadeu@gmail.com}

Thiago de Paula Oliveira \email{toliveira@abacusbio.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{H_martini_blocks}
\alias{H_martini_blocks}
\title{Compute Martini block corrections (H11, H12, H21, H22 - A22)}
\usage{
H_martini_blocks(A12, A22, G22, tau, omega)
}
\arguments{
\item{A12}{n1 x n2 block from A}

\item{A22}{n2 x n2 block from A (genotyped)}

\item{G22}{n2 x n2 block from G (genotyped)}

\item{tau}{scalar}

\item{omega}{scalar}
}
\value{
list(H11, H12, H21, H22corr) where H22corr = H22 - A22
}
\description{
Compute Martini block corrections (H11, H12, H21, H22 - A22)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{munoz_var_mean_by_Aclass}
\alias{munoz_var_mean_by_Aclass}
\title{Compute var/mean of G by classes of rounded A, and fill varA/meanG}
\usage{
munoz_var_mean_by_Aclass(G, A, round_digits)
}
\arguments{
\item{G}{square matrix}

\item{A}{square matrix (same dim as G)}

\item{round_digits}{integer number of decimal places to round A}
}
\value{
list(varA, meanG, classes, var_by_class, mean_by_class)
}
\description{
Compute var/mean of G by classes of rounded A, and fill varA/meanG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/missingdata.R
\name{missingdata}
\alias{missingdata}
\title{Survying on missing data}
\usage{
missingdata(data, unk = 0)
}
\arguments{
\item{data}{data name from a pedigree list}

\item{unk}{unknown value of your data}
}
\value{
list with $conflict: rows of the data which are at least one parental name equal to the individual. $missing.sire: rows of the data which arie missing data sire (Parental 1) information. $missing.dire: same as above for dire (Parental 2). $summary.missing: summary of the missing data. 2 columns, 1st for the name of the parental listed, 2nd for the how many times appeared in the data.
}
\description{
This function verify which rows in a pedigree data has missing parental or conflictuos data
}
\examples{
data(ped.mrode)
missingdata(ped.mrode)

}
\author{
Rodrigo R Amadeu, \email{rramadeu@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gmatrix.R
\name{Gmatrix}
\alias{Gmatrix}
\title{Construction of Relationship Matrix G}
\usage{
Gmatrix(
  SNPmatrix = NULL,
  method = "VanRaden",
  missingValue = -9,
  maf = 0,
  thresh.missing = 0.5,
  verify.posdef = FALSE,
  ploidy = 2,
  pseudo.diploid = FALSE,
  integer = TRUE,
  ratio = FALSE,
  impute.method = "mean",
  rmv.mono = FALSE,
  thresh.htzy = 0,
  ratio.check = TRUE,
  weights = NULL,
  ploidy.correction = FALSE,
  ASV = FALSE
)
}
\arguments{
\item{SNPmatrix}{matrix (n x m), where n is is individual names and m is 
marker names (coded inside the matrix as 0, 1, 2, ..., ploidy, and, 
missingValue).}

\item{method}{"Yang" or "VanRaden" for marker-based additive relationship 
matrix. "Su" or "Vitezica" for marker-based dominance relationship matrix. 
"Slater" for full-autopolyploid model including non-additive effects.
"Endelman" for autotetraploid dominant (digentic) relationship matrix. 
"MarkersMatrix" for a matrix with the amount of shared markers between 
individuals (3).
Default is "VanRaden", for autopolyploids will be computed a scaled 
product (similar to Covarrubias-Pazaran, 2006).}

\item{missingValue}{missing value in data. Default=-9.}

\item{maf}{minimum allele frequency accepted to each marker. Default=0.}

\item{thresh.missing}{threshold on missing data, SNPs below of this 
frequency value will be maintained, if equal to 1, no threshold and 
imputation is considered. Default = 1.}

\item{verify.posdef}{verify if the resulting matrix is positive-definite. 
Default=FALSE.}

\item{ploidy}{data ploidy (an even number between 2 and 20). Default=2.}

\item{pseudo.diploid}{if TRUE, uses pseudodiploid parametrization of 
Slater (2016).}

\item{integer}{if FALSE, not check for integer numbers. Default=TRUE.}

\item{ratio}{if TRUE, molecular data are considered ratios and its 
computed the scaled product of the matrix (as in "VanRaden" method).}

\item{impute.method}{"mean" to impute the missing data by the mean per 
marker, "mode" to impute the missing data by the mode per marker, 
"global.mean" to impute the missing data by the mean across all markers,
"global.mode" to impute the missing data my the mode across all marker. 
Default = "none".}

\item{rmv.mono}{if monomorphic markers should be removed. Default=FALSE.}

\item{thresh.htzy}{threshold heterozigosity, remove SNPs below this 
threshold. Default=0.}

\item{ratio.check}{if TRUE, run Mcheck with ratio data.}

\item{weights}{vector with weights for each marker. Only works if 
method="VanRaden". Default is a vector of 1's (equal weight).}

\item{ploidy.correction}{It sets the denominator (correction) of the 
crossprod. Used only when ploidy > 2 for "VanRaden" and ratio models.
If TRUE, it uses the sum of "Ploidy" times "Frequency" times "(1-Frequency)" 
of each marker as method 1 in VanRaden 2008 and Endelman (2018).
When ratio=TRUE, it uses "1/Ploidy" times "Frequency" times "(1-Frequency)". 
If FALSE, it uses the sum of the sampling variance of each marker. 
Default = FALSE.}

\item{ASV}{if TRUE, transform matrix into average semivariance (ASV) 
equivalent (K = K / (trace(K) / (nrow(K)-1))). Details formula 2 of 
Fieldmann et al. (2022). Default = FALSE.}
}
\value{
Matrix with the marker-bases relationships between the individuals
}
\description{
Given a matrix (individual x markers), a method, a missing value, and a maf 
threshold, return a additive or non-additive relationship matrix.
For diploids, the methods "Yang" and "VanRaden" for additive relationship 
matrices, and "Su" and "Vitezica" for non-additive relationship matrices 
are implemented. For autopolyploids, the method "VanRaden" for additive 
relationship, method "Slater" for full-autopolyploid model including 
non-additive effects, and pseudo-diploid parametrization are implemented.
Weights are implemented for "VanRaden" method as described in Liu (2020).
}
\examples{
\dontrun{
## Diploid Example
data(snp.pine)
str(snp.pine)
Gmatrix.Yang <- Gmatrix(snp.pine, method="Yang", missingValue=-9, maf=0.05)
Gmatrix.VanRaden <- Gmatrix(snp.pine, method="VanRaden", missingValue=-9, maf=0.05)
Gmatrix.Su <- Gmatrix(snp.pine, method="Su", missingValue=-9, maf=0.05)
Gmatrix.Vitezica <- Gmatrix(snp.pine, method="Vitezica", missingValue=-9, maf=0.05)

## Autetraploid example
data(snp.sol)
Gmatrix.VanRaden <- Gmatrix(snp.sol, method="VanRaden", ploidy=4)
Gmatrix.Endelman <- Gmatrix(snp.sol, method="Endelman", ploidy=4) 
Gmatrix.Slater <- Gmatrix(snp.sol, method="Slater", ploidy=4)
Gmatrix.Pseudodiploid <- Gmatrix(snp.sol, method="VanRaden", ploidy=4, pseudo.diploid=TRUE) 
Gmatrix.weighted <- Gmatrix(snp.sol, method="VanRaden", weights = runif(3895,0.001,0.1), ploidy=4)
}

}
\references{
\emph{Covarrubias-Pazaran, G. 2016. Genome assisted prediction 
of quantitative traits using the R package sommer. PLoS ONE 11(6):1-15.}

\emph{Endelman, JB, et al., 2018. Genetic variance partitioning 
and genome-wide prediction with allele dosage information in autotetraploid 
potato. Genetics, 209(1) pp. 77-87.}

\emph{Feldmann MJ, et al. 2022. Average semivariance directly 
yields accurate estimates of the genomic variance in complex trait analyses. 
G3 (Bethesda), 12(6).}

\emph{Liu, A, et al. 2020. Weighted single-step genomic best 
linear unbiased prediction integrating variants selected from sequencing 
data by association and bioinformatics analyses. Genet Sel Evol 52, 48.}

\emph{Slater, AT, et al. 2016. Improving genetic gain with 
genomic selection in autotetraploid potato. The Plant Genome 9(3), pp.1-15.}

\emph{Su, G, et al. 2012. Estimating additive and non-additive 
genetic variances and predicting genetic merits using genome-wide dense 
single nucleotide polymorphism markers. PloS one, 7(9), p.e45293.}

\emph{VanRaden, PM, 2008. Efficient methods to compute genomic 
predictions. Journal of dairy science, 91(11), pp.4414-4423.}

\emph{Vitezica, ZG, et al. 2013. On the additive and dominant 
variance and covariance of individuals within the genomic selection scope. 
Genetics, 195(4), pp.1223-1230.}

\emph{Yang, J, et al. 2010. Common SNPs explain a large 
proportion of the heritability for human height. Nature genetics, 42(7), 
pp.565-569.}
}
\author{
Rodrigo R. Amadeu \email{rramadeu@gmail.com}

Thiago de Paula Oliveira \email{toliveira@abacusbio.com}

Marcio Resende Jr.

Letícia A. C. Lara

Ivone Oliveira

Felipe V. Ferrao
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snp.sol-data.R
\docType{data}
\name{snp.sol}
\alias{snp.sol}
\title{Molecular data for autopolyploid examples}
\format{
data.frame
}
\usage{
data(snp.sol)
}
\description{
Dataset extract from  supplementary material from Endelman et al. (2018). 
SNP marker matrix from Pine tree coded as 0,1,2,3,4 and missing value as -9.
}
\examples{
data(snp.sol)

}
\references{
Endelman, JB, et al., 2018 Genetic variance partitioning and genome-wide prediction with allele dosage information in autotetraploid potato. Genetics, 209(1) pp. 77-87.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterpedigree.R
\name{filterpedigree}
\alias{filterpedigree}
\title{Filter the pedigree to keep only the genealogy of a subset of individuals}
\usage{
filterpedigree(inds, data)
}
\arguments{
\item{inds}{vector with strings of individuals to keep their genealogy in the matrix}

\item{data}{name of the pedigree data frame. Default=NULL.}
}
\value{
a data frame with pedigree containing the genealogy of the selected individuals
}
\description{
Filter the pedigree to keep only the genealogy of a subset of individuals
}
\examples{
data(ped.sol)
new.ped.sol = filterpedigree(inds = c("MSW168-2","W14090-3","W14090-4"),data=ped.sol)

}
\author{
Rodrigo R Amadeu, \email{rramadeu@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ped.mrode-data.R
\docType{data}
\name{ped.mrode}
\alias{ped.mrode}
\title{Pedigree Data}
\format{
table
}
\usage{
data(ped.mrode)
}
\description{
Data from pedigree example proposed by Mrode 2005
}
\examples{
data(ped.mrode)

}
\references{
R. A. Mrode, R. Thompson. Linear Models for the Prediction of Animal Breeding Values. CABI, 2005.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hmatrix.R
\name{Hmatrix}
\alias{Hmatrix}
\title{Construction of Combined Relationship Matrix H}
\usage{
Hmatrix(
  A = NULL,
  G = NULL,
  markers = NULL,
  c = 0,
  method = "Martini",
  tau = 1,
  omega = 1,
  missingValue = -9,
  maf = 0,
  ploidy = 2,
  roundVar = 3,
  ASV = FALSE
)
}
\arguments{
\item{A}{A matrix from function Amatrix}

\item{G}{G matrix from function Gmatrix}

\item{markers}{matrix marker which generated the Gmatrix}

\item{c}{constant value of H computation, default: c=0}

\item{method}{"Martini" or "Munoz", default="Martini"}

\item{tau}{to be used for Martini's method, default=1.}

\item{omega}{to be used of Martini's method, default=1.}

\item{missingValue}{missing value in data, default=-9.}

\item{maf}{max of missing data accepted to each markerm default=0.05.}

\item{ploidy}{data ploidy (an even number between 2 and 20), default=2.}

\item{roundVar}{only used for Munoz's method, how many digits to consider the relationship be of same class, default=2.}

\item{ASV}{if TRUE, transform matrix into average semivariance (ASV) equivalent (K = K / (trace(K) / (nrow(K)-1))). Details formula 2 of Fieldmann et al. (2022). Default = FALSE.}
}
\value{
H Matrix with the relationship between the individuals based on pedigree and corrected by molecular information
}
\description{
Given a matrix A and a matrix G returns a H matrix. H matrix is the relationship matrix using combined information from the pedigree and genomic relationship matrices. First, you need to compute the matrices separated and then use them as input to build the combined H matrix. 
Two methods are implemented: `Munoz` shrinks the G matrix towards the A matrix scaling the molecular relatadness by each relationship classes; 
`Martini` is a modified version from Legarra et al. (2009) where combines A and G matrix using scaling factors. When method is equal `Martini` and `tau=1` and `omega=1` you have the same H matrix as in Legarra et al. (2009).
}
\examples{
\dontrun{
data(ped.sol)
data(snp.sol)
#Computing the numerator relationship matrix 10\% of double-reduction
Amat <- Amatrix(ped.sol, ploidy=4, w = 0.1)
#Computing the additive relationship matrix based on VanRaden (modified)
Gmat <- Gmatrix(snp.sol, ploidy=4, 
                maf=0.05, method="VanRaden")
Gmat <- round(Gmat,3) #to be easy to invert

#Computing H matrix (Martini)
Hmat_Martini <- Hmatrix(A=Amat, G=Gmat, method="Martini", 
                     ploidy=4, 
                     maf=0.05)
                     
#Computing H matrix (Munoz)
Hmat_Munoz <- Hmatrix(A=Amat, G=Gmat, markers = snp.sol, 
                      ploidy=4, method="Munoz",
                      roundVar=2,
                      maf=0.05)
}

}
\references{
\emph{Feldmann MJ, et al. 2022. Average semivariance directly yields accurate estimates of the genomic variance in complex trait analyses. G3 (Bethesda), 12(6).}

\emph{Munoz, PR. 2014 Unraveling additive from nonadditive effects using genomic relationship matrices. Genetics 198, 1759-1768}

\emph{Martini, JW, et al. 2018 The effect of the H-1 scaling factors tau and omega on the structure of H in the single-step procedure. Genetics Selection Evolution 50(1), 16}

\emph{Legarra, A, et al. 2009 A relationship matrix including full pedigree and genomic information. Journal of Dairy Science 92, 4656-4663}
}
\author{
Rodrigo R Amadeu, \email{rramadeu@gmail.com}

Thiago de Paula Oliveira \email{toliveira@abacusbio.com}
}
